@@ -47,9 +47,9 @@
 #include "adblockrule.h"
 #include "adblocksubscription.h"
 #include "qztools.h"
+#include "qzregexp.h"
 
 #include <QDebug>
-#include "qzregexp.h"
 #include <QUrl>
 #include <QString>
 #include <QStringList>
@@ -62,8 +62,8 @@
 static QString toSecondLevelDomain(const QUrl &url)
 {
 #if QT_VERSION >= 0x040800
-    const QString &topLevelDomain = url.topLevelDomain();
-    const QString &urlHost = url.host();
+    const QString topLevelDomain = url.topLevelDomain();
+    const QString urlHost = url.host();
 
     if (topLevelDomain.isEmpty() || urlHost.isEmpty()) {
         return QString();
@@ -189,8 +189,8 @@ bool AdBlockRule::urlMatch(const QUrl &url) const
         return false;
     }
 
-    const QString &encodedUrl = url.toEncoded();
-    const QString &domain = url.host();
+    const QString encodedUrl = url.toEncoded();
+    const QString domain = url.host();
 
     return networkMatch(QNetworkRequest(url), domain, encodedUrl);
 }
@@ -299,15 +299,15 @@ bool AdBlockRule::matchDomain(const QString &domain) const
 
 bool AdBlockRule::matchThirdParty(const QNetworkRequest &request) const
 {
-    const QString &referer = request.attribute(QNetworkRequest::Attribute(QNetworkRequest::User + 151), QString()).toString();
+    const QString referer = request.attribute(QNetworkRequest::Attribute(QNetworkRequest::User + 151), QString()).toString();
 
     if (referer.isEmpty()) {
         return false;
     }
 
     // Third-party matching should be performed on second-level domains
-    const QString &refererHost = toSecondLevelDomain(QUrl(referer));
-    const QString &host = toSecondLevelDomain(request.url());
+    const QString refererHost = toSecondLevelDomain(QUrl(referer));
+    const QString host = toSecondLevelDomain(request.url());
 
     bool match = refererHost != host;
 
@@ -361,7 +361,11 @@ void AdBlockRule::parseFilter()
 
     // Empty rule or just comment
     if (m_filter.trimmed().isEmpty() || m_filter.startsWith(QLatin1Char('!'))) {
+        // We want to differentiate rule disabled by user and rule disabled in subscription file
+        // m_isInternalDisabled is also used when rule is disabled due to all options not being supported
         m_isEnabled = false;
+        m_isInternalDisabled = true;
+        m_type = Invalid;
         return;
     }
 
@@ -377,7 +381,6 @@ void AdBlockRule::parseFilter()
         }
 
         m_cssSelector = parsedLine.mid(pos + 2);
-        m_cssSelector.remove('\\');
 
         // CSS rule cannot have more options -> stop parsing
         return;
@@ -446,6 +449,7 @@ void AdBlockRule::parseFilter()
         // If we don't handle all options, it's safer to just disable this rule
         if (handledOptions != options.count()) {
             m_isInternalDisabled = true;
+            m_type = Invalid;
             return;
         }
 
@@ -569,7 +573,7 @@ QStringList AdBlockRule::parseRegExpFilter(const QString &parsedFilter) const
     list.removeDuplicates();
 
     for (int i = 0; i < list.length(); ++i) {
-        const QString &part = list.at(i);
+        const QString part = list.at(i);
 
         if (part.length() < 2) {
             list.removeAt(i);
