@@ -62,6 +62,7 @@
 #ifdef Q_OS_MAC
 #include "macmenureceiver.h"
 #include <QFileOpenEvent>
+#include <QMenu>
 #endif
 #include <QNetworkDiskCache>
 #include <QDesktopServices>
@@ -114,6 +115,7 @@ MainApplication::MainApplication(int &argc, char** argv)
 #endif
 #ifdef Q_OS_MAC
     , m_macMenuReceiver(0)
+    , m_macDockMenu(0)
 #endif
 {
 #if defined(QZ_WS_X11) && !defined(NO_SYSTEM_DATAPATH)
@@ -264,7 +266,7 @@ MainApplication::MainApplication(int &argc, char** argv)
     translateApp();
 
     QupZilla* qupzilla = new QupZilla(Qz::BW_FirstAppWindow, startUrl);
-    m_mainWindows.append(qupzilla);
+    m_mainWindows.prepend(qupzilla);
 
     connect(qupzilla, SIGNAL(message(Qz::AppMessageType,bool)), this, SLOT(sendMessages(Qz::AppMessageType,bool)));
     connect(qupzilla, SIGNAL(startingCompleted()), this, SLOT(restoreCursor()));
@@ -284,17 +286,12 @@ MainApplication::MainApplication(int &argc, char** argv)
         int afterLaunch = settings.value("Web-URL-Settings/afterLaunch", 1).toInt();
         settings.setValue("SessionRestore/isRunning", true);
 
-#ifndef PORTABLE_BUILD
-        bool alwaysCheckDefaultBrowser = settings.value("Web-Browser-Settings/CheckDefaultBrowser", DEFAULT_CHECK_DEFAULTBROWSER).toBool();
-        if (alwaysCheckDefaultBrowser) {
-            alwaysCheckDefaultBrowser = checkDefaultWebBrowser();
-            settings.setValue("Web-Browser-Settings/CheckDefaultBrowser", alwaysCheckDefaultBrowser);
-        }
-#endif
-
         if (checkUpdates) {
             new Updater(qupzilla);
         }
+
+        backupSavedSessions();
+
         if (m_startingAfterCrash || afterLaunch == 3) {
             m_restoreManager = new RestoreManager(m_activeProfil + "session.dat");
             if (!m_restoreManager->isValid()) {
@@ -327,6 +324,15 @@ void MainApplication::postLaunch()
 
     connect(this, SIGNAL(messageReceived(QString)), this, SLOT(receiveAppMessage(QString)));
     connect(this, SIGNAL(aboutToQuit()), this, SLOT(saveSettings()));
+
+#ifndef PORTABLE_BUILD
+    Settings settings;
+    bool alwaysCheckDefaultBrowser = settings.value("Web-Browser-Settings/CheckDefaultBrowser", DEFAULT_CHECK_DEFAULTBROWSER).toBool();
+    if (alwaysCheckDefaultBrowser) {
+        alwaysCheckDefaultBrowser = checkDefaultWebBrowser();
+        settings.setValue("Web-Browser-Settings/CheckDefaultBrowser", alwaysCheckDefaultBrowser);
+    }
+#endif
 }
 
 void MainApplication::loadSettings()
@@ -335,53 +341,13 @@ void MainApplication::loadSettings()
     settings.beginGroup("Themes");
     QString activeTheme = settings.value("activeTheme", DEFAULT_THEME_NAME).toString();
     settings.endGroup();
-    m_activeThemePath = THEMESDIR + activeTheme + "/";
-    QFile cssFile(m_activeThemePath + "main.css");
-    cssFile.open(QFile::ReadOnly);
-    QString css = cssFile.readAll();
-    cssFile.close();
-#ifdef QZ_WS_X11
-    if (QFile(m_activeThemePath + "linux.css").exists()) {
-        cssFile.setFileName(m_activeThemePath + "linux.css");
-        cssFile.open(QFile::ReadOnly);
-        css.append(cssFile.readAll());
-        cssFile.close();
-    }
-#endif
-#ifdef Q_OS_MAC
-    if (QFile(m_activeThemePath + "mac.css").exists()) {
-        cssFile.setFileName(m_activeThemePath + "mac.css");
-        cssFile.open(QFile::ReadOnly);
-        css.append(cssFile.readAll());
-        cssFile.close();
-    }
-#endif
-#if defined(Q_OS_WIN) || defined(Q_OS_OS2)
-    if (QFile(m_activeThemePath + "windows.css").exists()) {
-        cssFile.setFileName(m_activeThemePath + "windows.css");
-        cssFile.open(QFile::ReadOnly);
-        css.append(cssFile.readAll());
-        cssFile.close();
-    }
-#endif
 
-    //RTL Support
-    //loading 'rtl.css' when layout is right to left!
-    if (isRightToLeft() && QFile(m_activeThemePath + "rtl.css").exists()) {
-        cssFile.setFileName(m_activeThemePath + "rtl.css");
-        cssFile.open(QFile::ReadOnly);
-        css.append(cssFile.readAll());
-        cssFile.close();
-    }
-
-    QString relativePath = QDir::current().relativeFilePath(m_activeThemePath);
-    css.replace(QzRegExp("url\\s*\\(\\s*([^\\*:\\);]+)\\s*\\)", Qt::CaseSensitive),
-                QString("url(%1\\1)").arg(relativePath + "/"));
-    setStyleSheet(css);
+    loadTheme(activeTheme);
 
+    // Create global QWebSettings object
     webSettings();
 
-    //Web browsing settings
+    // Web browsing settings
     settings.beginGroup("Web-Browser-Settings");
 
     if (!m_isPrivateSession) {
@@ -402,6 +368,7 @@ void MainApplication::loadSettings()
     m_websettings->setAttribute(QWebSettings::XSSAuditingEnabled, settings.value("XSSAuditing", false).toBool());
     m_websettings->setMaximumPagesInCache(settings.value("maximumCachedPages", 3).toInt());
     m_websettings->setDefaultTextEncoding(settings.value("DefaultEncoding", m_websettings->defaultTextEncoding()).toString());
+    m_websettings->setAttribute(QWebSettings::SpatialNavigationEnabled, settings.value("SpatialNavigation", false).toBool());
 
 #if QTWEBKIT_FROM_2_3
     m_websettings->setAttribute(QWebSettings::CaretBrowsingEnabled, settings.value("CaretBrowsing", false).toBool());
@@ -572,12 +539,12 @@ void MainApplication::receiveAppMessage(QString message)
     QUrl actUrl;
 
     if (message.startsWith(QLatin1String("URL:"))) {
-        const QUrl &url = QUrl::fromUserInput(message.mid(4));
+        const QUrl url = QUrl::fromUserInput(message.mid(4));
         addNewTab(url);
         actWin = getWindow();
     }
     else if (message.startsWith(QLatin1String("ACTION:"))) {
-        const QString &text = message.mid(7);
+        const QString text = message.mid(7);
         if (text == QLatin1String("NewTab")) {
             addNewTab();
         }
@@ -633,12 +600,23 @@ QupZilla* MainApplication::makeNewWindow(Qz::BrowserWindow type, const QUrl &sta
     }
 
     QupZilla* newWindow = new QupZilla(type, startUrl);
-    m_mainWindows.append(newWindow);
+    m_mainWindows.prepend(newWindow);
 
     return newWindow;
 }
 
 #ifdef Q_OS_MAC
+extern void qt_mac_set_dock_menu(QMenu* menu);
+
+QMenu* MainApplication::macDockMenu()
+{
+    if (!m_macDockMenu) {
+        m_macDockMenu = new QMenu(0);
+        qt_mac_set_dock_menu(m_macDockMenu);
+    }
+    return m_macDockMenu;
+}
+
 MacMenuReceiver* MainApplication::macMenuReceiver()
 {
     if (!m_macMenuReceiver) {
@@ -692,6 +670,85 @@ void MainApplication::connectDatabase()
     m_databaseConnected = true;
 }
 
+void MainApplication::loadTheme(const QString &name)
+{
+    // Themes are loaded from the following directories:
+    //  1. Directory "themes" in user profile
+    //  2. Directory "themes" in root profile directory
+    //  3. System data path
+    //       > /usr/share/qupzilla/themes on Linux
+    //       > $EXECUTABLE_DIR/themes on Windows
+
+    QStringList themePaths;
+    themePaths << m_activeProfil + "themes/"
+               << PROFILEDIR + "themes/"
+               << THEMESDIR;
+
+    foreach (const QString &path, themePaths) {
+        const QString theme = path + name + "/";
+        if (QFile::exists(theme + "main.css")) {
+            m_activeThemePath = theme;
+            break;
+        }
+    }
+
+    if (m_activeThemePath.isEmpty()) {
+        qWarning("Cannot load theme '%s'!", qPrintable(name));
+        m_activeThemePath = THEMESDIR + DEFAULT_THEME_NAME + "/";
+    }
+
+    QFile cssFile(m_activeThemePath + "main.css");
+    cssFile.open(QFile::ReadOnly);
+    QString css = cssFile.readAll();
+    cssFile.close();
+
+    /*
+     * #id[style=QtStyle] (QtStyle = QMacStyle, QWindowsVistaStyle, QGtkStyle, ...)
+     * should be enough instead of loading special stylesheets
+     */
+#ifdef QZ_WS_X11
+    if (QFile(m_activeThemePath + "linux.css").exists()) {
+        cssFile.setFileName(m_activeThemePath + "linux.css");
+        cssFile.open(QFile::ReadOnly);
+        css.append(cssFile.readAll());
+        cssFile.close();
+    }
+#endif
+
+#ifdef Q_OS_MAC
+    if (QFile(m_activeThemePath + "mac.css").exists()) {
+        cssFile.setFileName(m_activeThemePath + "mac.css");
+        cssFile.open(QFile::ReadOnly);
+        css.append(cssFile.readAll());
+        cssFile.close();
+    }
+#endif
+
+#if defined(Q_OS_WIN) || defined(Q_OS_OS2)
+    if (QFile(m_activeThemePath + "windows.css").exists()) {
+        cssFile.setFileName(m_activeThemePath + "windows.css");
+        cssFile.open(QFile::ReadOnly);
+        css.append(cssFile.readAll());
+        cssFile.close();
+    }
+#endif
+
+    // RTL Support
+    // Loading 'rtl.css' when layout is right to left!
+    if (isRightToLeft() && QFile(m_activeThemePath + "rtl.css").exists()) {
+        cssFile.setFileName(m_activeThemePath + "rtl.css");
+        cssFile.open(QFile::ReadOnly);
+        css.append(cssFile.readAll());
+        cssFile.close();
+    }
+
+    QString relativePath = QDir::current().relativeFilePath(m_activeThemePath);
+    css.replace(QzRegExp("url\\s*\\(\\s*([^\\*:\\);]+)\\s*\\)", Qt::CaseSensitive),
+                QString("url(%1\\1)").arg(relativePath + "/"));
+
+    setStyleSheet(css);
+}
+
 void MainApplication::translateApp()
 {
     Settings settings;
@@ -713,6 +770,27 @@ void MainApplication::translateApp()
     installTranslator(sys);
 }
 
+void MainApplication::backupSavedSessions()
+{
+    // session.dat      - current
+    // session.dat.old  - first backup
+    // session.dat.old1 - second backup
+
+    const QString sessionFile = m_activeProfil + "session.dat";
+
+    if (!QFile::exists(sessionFile)) {
+        return;
+    }
+
+    if (QFile::exists(sessionFile + ".old")) {
+        QFile::remove(sessionFile + ".old1");
+        QFile::copy(sessionFile + ".old", sessionFile + ".old1");
+    }
+
+    QFile::remove(sessionFile + ".old");
+    QFile::copy(sessionFile, sessionFile + ".old");
+}
+
 void MainApplication::quitApplication()
 {
     if (m_downloadManager && !m_downloadManager->canClose()) {
@@ -733,6 +811,10 @@ void MainApplication::quitApplication()
     //
     //  * this can occur on Mac OS (see #157)
 
+    if (!m_isPrivateSession) {
+        removeLockFile();
+    }
+
     quit();
 }
 
@@ -855,7 +937,7 @@ DownloadManager* MainApplication::downManager()
 AutoFill* MainApplication::autoFill()
 {
     if (!m_autofill) {
-        m_autofill = new AutoFill(getWindow());
+        m_autofill = new AutoFill(this);
     }
     return m_autofill;
 }
@@ -872,10 +954,10 @@ QNetworkDiskCache* MainApplication::networkCache()
 {
     if (!m_networkCache) {
         Settings settings;
-        const QString &basePath = settings.value("Web-Browser-Settings/CachePath",
-                                  QString("%1networkcache/").arg(m_activeProfil)).toString();
+        const QString basePath = settings.value("Web-Browser-Settings/CachePath",
+                                                QString("%1networkcache/").arg(m_activeProfil)).toString();
 
-        const QString &cachePath = QString("%1/%2-Qt%3/").arg(basePath, qWebKitVersion(), qVersion());
+        const QString cachePath = QString("%1/%2-Qt%3/").arg(basePath, qWebKitVersion(), qVersion());
         m_networkCache = new QNetworkDiskCache(this);
         m_networkCache->setCacheDirectory(cachePath);
     }
@@ -913,7 +995,9 @@ void MainApplication::startPrivateBrowsing()
 {
     QStringList args;
     foreach (const QString &arg, arguments()) {
-        if (arg.startsWith(QLatin1Char('-'))) {
+        if (arg.startsWith(QLatin1Char('-')) &&
+                arg != QLatin1String("--private-browsing") &&
+                arg != QLatin1String("-pb")) {
             args.append(arg);
         }
     }
@@ -938,9 +1022,10 @@ bool MainApplication::checkDefaultWebBrowser()
 #ifdef Q_OS_WIN
     bool showAgain = true;
     if (!associationManager()->isDefaultForAllCapabilities()) {
-        CheckBoxDialog dialog(QDialogButtonBox::Yes | QDialogButtonBox::No);
+        CheckBoxDialog dialog(QDialogButtonBox::Yes | QDialogButtonBox::No, getWindow());
         dialog.setText(tr("QupZilla is not currently your default browser. Would you like to make it your default browser?"));
         dialog.setCheckBoxText(tr("Always perform this check when starting QupZilla."));
+        dialog.setDefaultCheckState(Qt::Checked);
         dialog.setWindowTitle(tr("Default Browser"));
         dialog.setIcon(qIconProvider->standardIcon(QStyle::SP_MessageBoxWarning));
 
@@ -994,7 +1079,7 @@ QUrl MainApplication::userStyleSheet(const QString &filePath) const
     userStyle += QString("::selection {background: %1; color: %2;} ").arg(highlightColor, highlightedTextColor);
 #endif
 
-    userStyle += AdBlockManager::instance()->elementHidingRules() + "{ display:none !important;}";
+    userStyle += AdBlockManager::instance()->elementHidingRules();
 
     QFile file(filePath);
     if (!filePath.isEmpty() && file.open(QFile::ReadOnly)) {
@@ -1004,8 +1089,8 @@ QUrl MainApplication::userStyleSheet(const QString &filePath) const
         file.close();
     }
 
-    const QString &encodedStyle = userStyle.toLatin1().toBase64();
-    const QString &dataString = QString("data:text/css;charset=utf-8;base64,%1").arg(encodedStyle);
+    const QString encodedStyle = userStyle.toLatin1().toBase64();
+    const QString dataString = QString("data:text/css;charset=utf-8;base64,%1").arg(encodedStyle);
 
     return QUrl(dataString);
 }
@@ -1092,7 +1177,9 @@ bool MainApplication::restoreStateSlot(QupZilla* window, RestoreData recoveryDat
     else {
         // QTabWidget::count() - count of tabs is not updated after closing
         // recovery tab ...
-        int tabCount = window->tabWidget()->count();
+        // update: it seems with ComboTabBar QTabWidget::count() is updated,
+        // we add pinnedTabCounts to currentTab!
+        int tabCount = window->tabWidget()->pinnedTabsCount();
         RestoreManager::WindowData data = recoveryData.first();
         data.currentTab += tabCount;
         recoveryData.remove(0);
@@ -1103,6 +1190,8 @@ bool MainApplication::restoreStateSlot(QupZilla* window, RestoreData recoveryDat
     foreach (const RestoreManager::WindowData &data, recoveryData) {
         QupZilla* window = makeNewWindow(Qz::BW_OtherRestoredWindow);
         window->restoreWindowState(data);
+        // for correct geometry calculation in QupZilla::setupUi()
+        mApp->processEvents();
     }
 
     destroyRestoreManager();
@@ -1200,8 +1289,9 @@ QString MainApplication::tempPath() const
     if (!dir.exists()) {
 #ifdef QZ_WS_X11
         // Symlink it to standard temporary path /tmp
-        QDir().mkpath(QDir::tempPath() + "/qupzilla-tmp/");
-        QFile::link(QDir::tempPath() + "/qupzilla-tmp/", PROFILEDIR + "tmp");
+        QDir().mkpath(QDir::tempPath() + "/qupzilla/tmp");
+        QFile::remove(PROFILEDIR + "tmp");
+        QFile::link(QDir::tempPath() + "/qupzilla/tmp/", PROFILEDIR + "tmp");
 #else
         dir.mkdir(path);
 #endif
@@ -1213,4 +1303,7 @@ QString MainApplication::tempPath() const
 MainApplication::~MainApplication()
 {
     delete m_uaManager;
+#ifdef Q_OS_MAC
+    delete m_macDockMenu;
+#endif
 }
