@@ -81,15 +81,19 @@
 #include <QNetworkRequest>
 #include <QDesktopServices>
 #include <QPrintPreviewDialog>
+#include <QPrinter>
 #include <QWebFrame>
 #include <QWebHistory>
 #include <QMessageBox>
+#include <QDesktopWidget>
+#include <QToolTip>
+#include <QScrollArea>
 
 #if QT_VERSION < 0x050000
 #include "qwebkitversion.h"
 #endif
 
-#ifdef QZ_WS_X11
+#if defined(QZ_WS_X11) && !defined(NO_X11)
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #endif
@@ -101,7 +105,7 @@
 #define MENU_RECEIVER mApp->macMenuReceiver()
 #endif
 
-const QString QupZilla::VERSION = "1.4.1";
+const QString QupZilla::VERSION = "1.6.0";
 const QString QupZilla::BUILDTIME =  __DATE__" "__TIME__;
 const QString QupZilla::AUTHOR = "David Rosca";
 const QString QupZilla::COPYRIGHT = "2010-2013";
@@ -117,11 +121,13 @@ QupZilla::QupZilla(Qz::BrowserWindow type, QUrl startUrl)
     , m_isStarting(false)
     , m_startingUrl(startUrl)
     , m_windowType(type)
+    , m_startTab(0)
     , m_menuBookmarksAction(0)
     , m_actionPrivateBrowsing(0)
     , m_sideBarManager(new SideBarManager(this))
     , m_statusBarMessage(new StatusBarMessage(this))
     , m_usingTransparentBackground(false)
+    , m_tabsOnTopState(-1)
 {
     setObjectName("mainwindow");
     setAttribute(Qt::WA_DeleteOnClose);
@@ -148,6 +154,18 @@ QupZilla::QupZilla(Qz::BrowserWindow type, QUrl startUrl)
     connect(mApp, SIGNAL(message(Qz::AppMessageType,bool)), this, SLOT(receiveMessage(Qz::AppMessageType,bool)));
 
     QTimer::singleShot(0, this, SLOT(postLaunch()));
+
+    if (mApp->isPrivateSession()) {
+        QzTools::setWmClass("QupZilla Browser (Private Window)", this);
+    }
+    else {
+        QzTools::setWmClass("QupZilla Browser", this);
+    }
+}
+
+void QupZilla::openWithTab(WebTab* tab)
+{
+    m_startTab = tab;
 }
 
 void QupZilla::postLaunch()
@@ -193,6 +211,9 @@ void QupZilla::postLaunch()
         break;
 
     case Qz::BW_MacFirstWindow:
+#ifdef Q_OS_MAC
+        QTimer::singleShot(0, this, SLOT(refreshStateOfAllActions()));
+#endif
     case Qz::BW_NewWindow:
         addTab = true;
         break;
@@ -209,6 +230,11 @@ void QupZilla::postLaunch()
         addTab = true;
     }
 
+    if (m_startTab) {
+        addTab = false;
+        m_tabWidget->addView(m_startTab);
+    }
+
     if (addTab) {
         QNetworkRequest request(startUrl);
         request.setRawHeader("X-QupZilla-UserLoadAction", QByteArray("1"));
@@ -221,7 +247,7 @@ void QupZilla::postLaunch()
     }
 
     if (m_tabWidget->getTabBar()->normalTabsCount() <= 0 && m_windowType != Qz::BW_OtherRestoredWindow) {
-        //Something went really wrong .. add one tab
+        // Something went really wrong .. add one tab
         QNetworkRequest request(m_homepage);
         request.setRawHeader("X-QupZilla-UserLoadAction", QByteArray("1"));
 
@@ -231,7 +257,7 @@ void QupZilla::postLaunch()
     aboutToHideEditMenu();
 
 #ifdef Q_OS_MAC
-    // fill menus even if user don't call them
+    // Fill menus even if user don't call them
     if (m_windowType == Qz::BW_FirstAppWindow) {
         aboutToShowBookmarksMenu();
         aboutToShowHistoryMostMenu();
@@ -247,6 +273,10 @@ void QupZilla::postLaunch()
     QMainWindow::setWindowTitle(m_lastWindowTitle);
 
     setUpdatesEnabled(true);
+    raise();
+    activateWindow();
+
+    QTimer::singleShot(0, tabWidget()->getTabBar(), SLOT(ensureVisible()));
 }
 
 void QupZilla::setupUi()
@@ -261,17 +291,30 @@ void QupZilla::setupUi()
         setWindowState(Qt::WindowMaximized);
     }
     else {
-        if (!restoreGeometry(settings.value("WindowGeometry").toByteArray())) {
-            setGeometry(QRect(20, 20, 800, 550));
+        // Let the WM decides where to put new browser window
+        if ((m_windowType != Qz::BW_FirstAppWindow && m_windowType != Qz::BW_MacFirstWindow) && mApp->getWindow()) {
+#ifdef Q_WS_WIN
+            // Windows WM places every new window in the middle of screen .. for some reason
+            QPoint p = mApp->getWindow()->geometry().topLeft();
+            p.setX(p.x() + 30);
+            p.setY(p.y() + 30);
+
+            if (!mApp->desktop()->availableGeometry(mApp->getWindow()).contains(p)) {
+                p.setX(mApp->desktop()->availableGeometry(mApp->getWindow()).x() + 30);
+                p.setY(mApp->desktop()->availableGeometry(mApp->getWindow()).y() + 30);
+            }
+            setGeometry(QRect(p, mApp->getWindow()->size()));
+#else
+            resize(mApp->getWindow()->size());
+#endif
         }
-
-        if (m_windowType == Qz::BW_NewWindow) {
-            // Moving window +40 x,y to be visible that this is new window
-            QPoint p = pos();
-            p.setX(p.x() + 40);
-            p.setY(p.y() + 40);
-
-            move(p);
+        else if (!restoreGeometry(settings.value("WindowGeometry").toByteArray())) {
+#ifdef Q_WS_WIN
+            setGeometry(QRect(mApp->desktop()->availableGeometry(mApp->getWindow()).x() + 30,
+                              mApp->desktop()->availableGeometry(mApp->getWindow()).y() + 30, 800, 550));
+#else
+            resize(800, 550);
+#endif
         }
     }
 
@@ -304,7 +347,7 @@ void QupZilla::setupUi()
     m_navigationContainer->setLayout(l);
 
     m_mainSplitter->addWidget(m_tabWidget);
-    m_mainLayout->addWidget(m_navigationContainer);
+    triggerTabsOnTop(tabsOnTop());
     m_mainLayout->addWidget(m_mainSplitter);
     m_mainSplitter->setCollapsible(0, false);
 
@@ -320,10 +363,17 @@ void QupZilla::setupUi()
     m_ipLabel->setObjectName("statusbar-ip-label");
     m_ipLabel->setToolTip(tr("IP Address of current page"));
 
-    statusBar()->insertPermanentWidget(0, m_progressBar);
-    statusBar()->insertPermanentWidget(1, m_ipLabel);
-    statusBar()->insertPermanentWidget(2, m_privateBrowsing);
-    statusBar()->insertPermanentWidget(3, m_adblockIcon);
+    statusBar()->addPermanentWidget(m_progressBar);
+    statusBar()->addPermanentWidget(m_ipLabel);
+    statusBar()->addPermanentWidget(m_privateBrowsing);
+    statusBar()->addPermanentWidget(m_adblockIcon);
+
+    // Workaround for Oxygen tooltips not having transparent background
+    QPalette pal = QToolTip::palette();
+    QColor col = pal.window().color();
+    col.setAlpha(0);
+    pal.setColor(QPalette::Window, col);
+    QToolTip::setPalette(pal);
 }
 
 void QupZilla::setupMenu()
@@ -346,37 +396,32 @@ void QupZilla::setupMenu()
     m_actionAbout->setMenuRole(QAction::AboutRole);
     connect(m_actionAbout, SIGNAL(triggered()), MENU_RECEIVER, SLOT(aboutQupZilla()));
 
-    m_actionPreferences = new QAction(QIcon(":/icons/faenza/settings.png"), tr("Pr&eferences"), 0);
+    m_actionPreferences = new QAction(QIcon::fromTheme("preferences-desktop", QIcon(":/icons/faenza/settings.png")), tr("Pr&eferences"), 0);
     m_actionPreferences->setMenuRole(QAction::PreferencesRole);
     m_actionPreferences->setShortcut(QKeySequence(QKeySequence::Preferences));
     connect(m_actionPreferences, SIGNAL(triggered()), MENU_RECEIVER, SLOT(showPreferences()));
 
     m_actionQuit = new QAction(QIcon::fromTheme("application-exit"), tr("Quit"), 0);
     m_actionQuit->setMenuRole(QAction::QuitRole);
-    QKeySequence quitSequence = QKeySequence(QKeySequence::Quit);
-#ifdef QZ_WS_X11
-    // QKeySequence::Quit returns a non-empty sequence on X11 only when running Gnome or Kde
-    if (quitSequence.isEmpty()) {
-        quitSequence = QKeySequence(Qt::CTRL + Qt::Key_Q);
-    }
-#endif
-    m_actionQuit->setShortcut(quitSequence);
+
+    // QKeySequence::Quit returns an empty sequence on Windows and X11 when running desktop other than Gnome and Kde
+    m_actionQuit->setShortcut(actionShortcut(QKeySequence::Quit, Qt::CTRL + Qt::Key_Q));
     connect(m_actionQuit, SIGNAL(triggered()), MENU_RECEIVER, SLOT(quitApp()));
 
     /*************
      * File Menu *
      *************/
     m_menuFile = new QMenu(tr("&File"));
-    m_menuFile->addAction(QIcon(":/icons/menu/new-tab.png"), tr("New Tab"), MENU_RECEIVER, SLOT(addTab()))->setShortcut(QKeySequence("Ctrl+T"));
+    m_menuFile->addAction(QIcon::fromTheme("tab-new", QIcon(":/icons/menu/new-tab.png")), tr("New Tab"), MENU_RECEIVER, SLOT(addTab()))->setShortcut(QKeySequence("Ctrl+T"));
     m_menuFile->addAction(QIcon::fromTheme("window-new"), tr("&New Window"), MENU_RECEIVER, SLOT(newWindow()))->setShortcut(QKeySequence("Ctrl+N"));
     m_menuFile->addAction(QIcon::fromTheme("document-open-remote"), tr("Open Location"), MENU_RECEIVER, SLOT(openLocation()))->setShortcut(QKeySequence("Ctrl+L"));
-    m_menuFile->addAction(QIcon::fromTheme("document-open"), tr("Open &File"), MENU_RECEIVER, SLOT(openFile()))->setShortcut(QKeySequence("Ctrl+O"));
+    m_menuFile->addAction(QIcon::fromTheme("document-open"), tr("Open &File..."), MENU_RECEIVER, SLOT(openFile()))->setShortcut(QKeySequence("Ctrl+O"));
     m_menuFile->addAction(tr("Close Tab"), MENU_RECEIVER, SLOT(closeTab()))->setShortcut(QKeySequence("Ctrl+W"));
     m_actionCloseWindow = m_menuFile->addAction(QIcon::fromTheme("window-close"), tr("Close Window"), MENU_RECEIVER, SLOT(closeWindow()));
     m_actionCloseWindow->setShortcut(QKeySequence("Ctrl+Shift+W"));
     m_menuFile->addSeparator();
     m_menuFile->addAction(QIcon::fromTheme("document-save"), tr("&Save Page As..."), MENU_RECEIVER, SLOT(savePage()))->setShortcut(QKeySequence("Ctrl+S"));
-    m_menuFile->addAction(tr("Save Page Screen"), MENU_RECEIVER, SLOT(savePageScreen()));
+    m_menuFile->addAction(QIcon::fromTheme("image-loading"), tr("Save Page Screen"), MENU_RECEIVER, SLOT(savePageScreen()))->setShortcut(QKeySequence("Ctrl+Shift+S"));
     m_menuFile->addAction(QIcon::fromTheme("mail-message-new"), tr("Send Link..."), MENU_RECEIVER, SLOT(sendLink()));
     m_menuFile->addAction(QIcon::fromTheme("document-print"), tr("&Print..."), MENU_RECEIVER, SLOT(printPage()))->setShortcut(QKeySequence("Ctrl+P"));
     m_menuFile->addSeparator();
@@ -496,10 +541,12 @@ void QupZilla::setupMenu()
      * History Menu *
      ****************/
     m_menuHistory = new Menu(tr("Hi&story"));
-    m_menuHistory->addAction(qIconProvider->standardIcon(QStyle::SP_ArrowBack), tr("&Back"), MENU_RECEIVER, SLOT(goBack()))->setShortcut(QKeySequence("Ctrl+Left"));
-    m_menuHistory->addAction(qIconProvider->standardIcon(QStyle::SP_ArrowForward), tr("&Forward"), MENU_RECEIVER, SLOT(goNext()))->setShortcut(QKeySequence("Ctrl+Right"));
+    m_menuHistory->addAction(qIconProvider->standardIcon(QStyle::SP_ArrowBack), tr("&Back"),
+                             MENU_RECEIVER, SLOT(goBack()))->setShortcut(actionShortcut(QKeySequence::Back, Qt::ALT + Qt::Key_Left, QKeySequence::Forward, Qt::ALT + Qt::Key_Right));
+    m_menuHistory->addAction(qIconProvider->standardIcon(QStyle::SP_ArrowForward), tr("&Forward"),
+                             MENU_RECEIVER, SLOT(goNext()))->setShortcut(actionShortcut(QKeySequence::Forward, Qt::ALT + Qt::Key_Right, QKeySequence::Back, Qt::ALT + Qt::Key_Left));
     m_menuHistory->addAction(qIconProvider->fromTheme("go-home"), tr("&Home"), MENU_RECEIVER, SLOT(goHome()))->setShortcut(QKeySequence("Alt+Home"));
-    m_menuHistory->addAction(QIcon(":/icons/menu/history.png"), tr("Show &All History"), MENU_RECEIVER, SLOT(showHistoryManager()))->setShortcut(QKeySequence("Ctrl+Shift+H"));
+    m_menuHistory->addAction(QIcon::fromTheme("view-history", QIcon(":/icons/menu/history.png")), tr("Show &All History"), MENU_RECEIVER, SLOT(showHistoryManager()))->setShortcut(QKeySequence("Ctrl+Shift+H"));
     m_menuHistory->addSeparator();
     connect(m_menuHistory, SIGNAL(aboutToShow()), MENU_RECEIVER, SLOT(aboutToShowHistoryMenu()));
     connect(m_menuHistory, SIGNAL(aboutToHide()), MENU_RECEIVER, SLOT(aboutToHideHistoryMenu()));
@@ -521,9 +568,9 @@ void QupZilla::setupMenu()
      * Bookmarks Menu *
      ******************/
     m_menuBookmarks = new Menu(tr("&Bookmarks"));
-    m_menuBookmarks->addAction(tr("Bookmark &This Page"), MENU_RECEIVER, SLOT(bookmarkPage()))->setShortcut(QKeySequence("Ctrl+D"));
-    m_menuBookmarks->addAction(tr("Bookmark &All Tabs"), MENU_RECEIVER, SLOT(bookmarkAllTabs()));
-    m_menuBookmarks->addAction(qIconProvider->fromTheme("user-bookmarks"), tr("Organize &Bookmarks"), MENU_RECEIVER, SLOT(showBookmarksManager()))->setShortcut(QKeySequence("Ctrl+Shift+O"));
+    m_menuBookmarks->addAction(QIcon::fromTheme("bookmark-new"), tr("Bookmark &This Page"), MENU_RECEIVER, SLOT(bookmarkPage()))->setShortcut(QKeySequence("Ctrl+D"));
+    m_menuBookmarks->addAction(QIcon::fromTheme("bookmark-new-list"), tr("Bookmark &All Tabs"), MENU_RECEIVER, SLOT(bookmarkAllTabs()));
+    m_menuBookmarks->addAction(qIconProvider->fromTheme("bookmarks-organize"), tr("Organize &Bookmarks"), MENU_RECEIVER, SLOT(showBookmarksManager()))->setShortcut(QKeySequence("Ctrl+Shift+O"));
     m_menuBookmarks->addSeparator();
 
     connect(m_menuBookmarks, SIGNAL(aboutToShow()), MENU_RECEIVER, SLOT(aboutToShowBookmarksMenu()));
@@ -544,9 +591,8 @@ void QupZilla::setupMenu()
     m_menuTools->addAction(QIcon(":/icons/menu/rss.png"), tr("RSS &Reader"), MENU_RECEIVER,  SLOT(showRSSManager()));
     m_menuTools->addAction(tr("Web In&spector"), MENU_RECEIVER, SLOT(showWebInspector()))->setShortcut(QKeySequence("Ctrl+Shift+I"));
     m_menuTools->addAction(QIcon::fromTheme("edit-clear"), tr("Clear Recent &History"), MENU_RECEIVER, SLOT(showClearPrivateData()))->setShortcut(QKeySequence("Ctrl+Shift+Del"));
-    m_actionPrivateBrowsing = new QAction(tr("&Private Browsing"), MENU_RECEIVER);
+    m_actionPrivateBrowsing = new QAction(QIcon(":/icons/locationbar/privatebrowsing.png"), tr("New &Private Window"), MENU_RECEIVER);
     m_actionPrivateBrowsing->setShortcut(QKeySequence("Ctrl+Shift+P"));
-    m_actionPrivateBrowsing->setVisible(!mApp->isPrivateSession());
     connect(m_actionPrivateBrowsing, SIGNAL(triggered(bool)), mApp, SLOT(startPrivateBrowsing()));
     m_menuTools->addAction(m_actionPrivateBrowsing);
     m_menuTools->addSeparator();
@@ -590,24 +636,26 @@ void QupZilla::setupMenu()
     setupOtherActions();
 
 #ifndef Q_OS_MAC
-    m_superMenu->addAction(new ActionCopy(m_menuFile->actions().at(0), this));
-    m_superMenu->addAction(new ActionCopy(m_menuFile->actions().at(1), this));
-    m_superMenu->addAction(new ActionCopy(m_menuFile->actions().at(3), this));
+    m_superMenu->addAction(m_menuFile->actions().at(0));
+    m_superMenu->addAction(m_menuFile->actions().at(1));
+    m_superMenu->addAction(m_actionPrivateBrowsing);
+    m_superMenu->addAction(m_menuFile->actions().at(3));
 
     m_superMenu->addSeparator();
-    m_superMenu->addAction(new ActionCopy(m_menuFile->actions().at(7), this));
-    m_superMenu->addAction(new ActionCopy(m_menuFile->actions().at(10), this));
+    m_superMenu->addAction(m_menuFile->actions().at(7));
+    m_superMenu->addAction(m_menuFile->actions().at(8));
+    m_superMenu->addAction(m_menuFile->actions().at(10));
 
     m_superMenu->addSeparator();
-    m_superMenu->addAction(new ActionCopy(m_menuEdit->actions().at(7), this));
-    m_superMenu->addAction(new ActionCopy(m_menuEdit->actions().at(8), this));
+    m_superMenu->addAction(m_menuEdit->actions().at(7));
+    m_superMenu->addAction(m_menuEdit->actions().at(8));
 
     m_superMenu->addSeparator();
-    m_superMenu->addAction(new ActionCopy(m_menuHistory->actions().at(3), this));
-    m_superMenu->addAction(new ActionCopy(m_menuBookmarks->actions().at(2), this));
+    m_superMenu->addAction(m_menuHistory->actions().at(3));
+    m_superMenu->addAction(m_menuBookmarks->actions().at(2));
 
     m_superMenu->addSeparator();
-    m_superMenu->addAction(new ActionCopy(m_actionPreferences, this));
+    m_superMenu->addAction(m_actionPreferences);
 
     m_superMenu->addSeparator();
     m_superMenu->addMenu(m_menuView);
@@ -616,13 +664,19 @@ void QupZilla::setupMenu()
     m_superMenu->addMenu(m_menuTools);
 
     m_superMenu->addSeparator();
-    m_superMenu->addAction(new ActionCopy(m_actionAbout, this));
-    m_superMenu->addAction(new ActionCopy(m_menuHelp->actions().at(3), this));
-    m_superMenu->addAction(new ActionCopy(m_menuHelp->actions().at(4), this));
-    m_superMenu->addAction(new ActionCopy(m_menuHelp->actions().at(5), this));
+    m_superMenu->addAction(m_actionAbout);
+    m_superMenu->addAction(m_menuHelp->actions().at(3));
+    m_superMenu->addAction(m_menuHelp->actions().at(4));
+    m_superMenu->addAction(m_menuHelp->actions().at(5));
 
     m_superMenu->addSeparator();
-    m_superMenu->addAction(new ActionCopy(m_actionQuit, this));
+    m_superMenu->addAction(m_actionQuit);
+#else
+    ActionCopy* copyActionPrivateBrowsing = new ActionCopy(m_actionPrivateBrowsing);
+    copyActionPrivateBrowsing->setText(copyActionPrivateBrowsing->text().remove(QLatin1Char('&')));
+    mApp->macDockMenu()->addAction(copyActionPrivateBrowsing);
+    mApp->macDockMenu()->addAction(m_menuFile->actions().at(1));
+    mApp->macDockMenu()->addAction(m_menuFile->actions().at(0));
 #endif
 }
 
@@ -641,12 +695,6 @@ void QupZilla::setupOtherActions()
     QShortcut* reloadAction = new QShortcut(QKeySequence("Ctrl+R"), this);
     connect(reloadAction, SIGNAL(activated()), MENU_RECEIVER, SLOT(reload()));
 
-    QShortcut* backAction = new QShortcut(QKeySequence("Alt+Left"), this);
-    connect(backAction, SIGNAL(activated()), MENU_RECEIVER, SLOT(goBack()));
-
-    QShortcut* forwardAction = new QShortcut(QKeySequence("Alt+Right"), this);
-    connect(forwardAction, SIGNAL(activated()), MENU_RECEIVER, SLOT(goNext()));
-
     QShortcut* openLocationAction = new QShortcut(QKeySequence("Alt+D"), this);
     connect(openLocationAction, SIGNAL(activated()), MENU_RECEIVER, SLOT(openLocation()));
 
@@ -655,7 +703,8 @@ void QupZilla::setupOtherActions()
 
     // Make shortcuts available even in fullscreen (menu hidden)
     QList<QAction*> actions = menuBar()->actions();
-    foreach (QAction* action, actions) {
+    for (int i = 0; i < actions.size(); ++i) {
+        QAction* action = actions.at(i);
         if (action->menu()) {
             actions += action->menu()->actions();
         }
@@ -663,6 +712,17 @@ void QupZilla::setupOtherActions()
     }
 }
 
+QKeySequence QupZilla::actionShortcut(QKeySequence shortcut, QKeySequence fallBack,
+                                      QKeySequence shortcutRTL, QKeySequence fallbackRTL)
+{
+    if (isRightToLeft() && (!shortcutRTL.isEmpty() || !fallbackRTL.isEmpty())) {
+        return (shortcutRTL.isEmpty() ? fallbackRTL : shortcutRTL);
+    }
+    else {
+        return (shortcut.isEmpty() ? fallBack : shortcut);
+    }
+}
+
 #ifdef Q_OS_MAC
 void QupZilla::setupMacMenu()
 {
@@ -701,6 +761,16 @@ void QupZilla::setupMacMenu()
     m_actionPageInfo = m_menuTools->actions().at(1);
     m_actionPrivateBrowsing = m_menuTools->actions().at(9);
 }
+
+void QupZilla::refreshStateOfAllActions()
+{
+    mApp->macMenuReceiver()->aboutToShowFileMenu(m_menuFile);
+    mApp->macMenuReceiver()->aboutToShowHistoryMenu(m_menuHistory);
+    mApp->macMenuReceiver()->aboutToShowBookmarksMenu(m_menuBookmarks);
+    mApp->macMenuReceiver()->aboutToShowViewMenu(m_menuView);
+    mApp->macMenuReceiver()->aboutToShowEditMenu(m_menuEdit);
+    mApp->macMenuReceiver()->aboutToShowToolsMenu(m_menuTools);
+}
 #endif
 
 void QupZilla::loadSettings()
@@ -729,7 +799,7 @@ void QupZilla::loadSettings()
     bool makeTransparent = settings.value("useTransparentBackground", false).toBool();
     m_sideBarWidth = settings.value("SideBarWidth", 250).toInt();
     m_webViewWidth = settings.value("WebViewWidth", 2000).toInt();
-    const QString &activeSideBar = settings.value("SideBar", "None").toString();
+    const QString activeSideBar = settings.value("SideBar", "None").toString();
 
     // Make sure both menubar and navigationbar are not hidden
     // Fixes #781
@@ -764,14 +834,14 @@ void QupZilla::loadSettings()
 
     m_sideBarManager->showSideBar(activeSideBar, false);
 
-    //Private browsing
+    // Private browsing
     m_privateBrowsing->setVisible(mApp->isPrivateSession());
 
 #ifdef Q_OS_WIN
     if (m_usingTransparentBackground && !makeTransparent) {
         QtWin::extendFrameIntoClientArea(this, 0, 0, 0, 0);
         QtWin::enableBlurBehindWindow(this, false);
-        QtWin::enableBlurBehindWindow(m_tabWidget->getTabBar(), false);
+        m_tabWidget->getTabBar()->enableBluredBackground(false);
         m_usingTransparentBackground = false;
     }
 #endif
@@ -779,7 +849,8 @@ void QupZilla::loadSettings()
     if (!makeTransparent) {
         return;
     }
-    //Opacity
+
+    // Transparency on X11 (no blur like on Windows)
 #ifdef QZ_WS_X11
     setAttribute(Qt::WA_TranslucentBackground);
     setAttribute(Qt::WA_NoSystemBackground, false);
@@ -798,12 +869,14 @@ void QupZilla::loadSettings()
 
         m_usingTransparentBackground = true;
 
-        QtWin::enableBlurBehindWindow(m_tabWidget->getTabBar(), true);
-        applyBlurToMainWindow();
-        update();
+        if (!isFullScreen()) {
+            m_tabWidget->getTabBar()->enableBluredBackground(true);
+            QtWin::extendFrameIntoClientArea(this);
+        }
 
         //install event filter
         menuBar()->installEventFilter(this);
+        m_tabWidget->getTabBar()->installEventFilter(this);
         m_navigationBar->installEventFilter(this);
         m_bookmarksToolbar->installEventFilter(this);
         statusBar()->installEventFilter(this);
@@ -856,15 +929,6 @@ Qz::BrowserWindow QupZilla::windowType() const
     return m_windowType;
 }
 
-QWidget* QupZilla::navigationContainer() const
-{
-    if (!qzSettings->tabsOnTop) {
-        return 0;
-    }
-
-    return m_navigationContainer;
-}
-
 void QupZilla::popupToolbarsMenu(const QPoint &pos)
 {
     aboutToShowViewMenu();
@@ -872,6 +936,20 @@ void QupZilla::popupToolbarsMenu(const QPoint &pos)
     aboutToHideViewMenu();
 }
 
+bool QupZilla::isTransparentBackgroundAllowed()
+{
+    return m_usingTransparentBackground && !isFullScreen();
+}
+
+bool QupZilla::tabsOnTop() const
+{
+    if (m_tabsOnTopState == -1) {
+        m_tabsOnTopState = qzSettings->tabsOnTop ? 1 : 0;
+    }
+
+    return m_tabsOnTopState == 1;
+}
+
 void QupZilla::setWindowTitle(const QString &t)
 {
     QString title = t;
@@ -955,8 +1033,8 @@ void QupZilla::aboutToShowBookmarksMenu()
     query.exec("SELECT title, url, icon FROM bookmarks WHERE folder='bookmarksMenu'");
     while (query.next()) {
         QString title = query.value(0).toString();
-        const QUrl &url = query.value(1).toUrl();
-        const QIcon &icon = qIconProvider->iconFromImage(QImage::fromData(query.value(2).toByteArray()));
+        const QUrl url = query.value(1).toUrl();
+        const QIcon icon = qIconProvider->iconFromImage(QImage::fromData(query.value(2).toByteArray()));
         if (title.length() > 40) {
             title.truncate(40);
             title += "..";
@@ -975,8 +1053,8 @@ void QupZilla::aboutToShowBookmarksMenu()
     query.exec("SELECT title, url, icon FROM bookmarks WHERE folder='bookmarksToolbar'");
     while (query.next()) {
         QString title = query.value(0).toString();
-        const QUrl &url = query.value(1).toUrl();
-        const QIcon &icon = qIconProvider->iconFromImage(QImage::fromData(query.value(2).toByteArray()));
+        const QUrl url = query.value(1).toUrl();
+        const QIcon icon = qIconProvider->iconFromImage(QImage::fromData(query.value(2).toByteArray()));
         if (title.length() > 40) {
             title.truncate(40);
             title += "..";
@@ -995,7 +1073,7 @@ void QupZilla::aboutToShowBookmarksMenu()
 
     query.exec("SELECT name FROM folders");
     while (query.next()) {
-        const QString &folderName = query.value(0).toString();
+        const QString folderName = query.value(0).toString();
         Menu* tempFolder = new Menu(folderName, m_menuBookmarks);
         tempFolder->setIcon(QIcon(style()->standardIcon(QStyle::SP_DirOpenIcon)));
 
@@ -1005,8 +1083,8 @@ void QupZilla::aboutToShowBookmarksMenu()
         query2.exec();
         while (query2.next()) {
             QString title = query2.value(0).toString();
-            const QUrl &url = query2.value(1).toUrl();
-            const QIcon &icon = qIconProvider->iconFromImage(QImage::fromData(query2.value(2).toByteArray()));
+            const QUrl url = query2.value(1).toUrl();
+            const QIcon icon = qIconProvider->iconFromImage(QImage::fromData(query2.value(2).toByteArray()));
             if (title.length() > 40) {
                 title.truncate(40);
                 title += "..";
@@ -1029,8 +1107,8 @@ void QupZilla::aboutToShowBookmarksMenu()
     query.exec("SELECT title, url, icon FROM bookmarks WHERE folder='unsorted'");
     while (query.next()) {
         QString title = query.value(0).toString();
-        const QUrl &url = query.value(1).toUrl();
-        const QIcon &icon = qIconProvider->iconFromImage(QImage::fromData(query.value(2).toByteArray()));
+        const QUrl url = query.value(1).toUrl();
+        const QIcon icon = qIconProvider->iconFromImage(QImage::fromData(query.value(2).toByteArray()));
         if (title.length() > 40) {
             title.truncate(40);
             title += "..";
@@ -1092,7 +1170,7 @@ void QupZilla::aboutToShowHistoryRecentMenu()
     QSqlQuery query;
     query.exec("SELECT title, url FROM history ORDER BY date DESC LIMIT 15");
     while (query.next()) {
-        const QUrl &url = query.value(1).toUrl();
+        const QUrl url = query.value(1).toUrl();
         QString title = query.value(0).toString();
         if (title.length() > 40) {
             title.truncate(40);
@@ -1147,7 +1225,7 @@ void QupZilla::aboutToShowViewMenu()
 
     m_actionShowStatusbar->setChecked(statusBar()->isVisible());
     m_actionShowBookmarksToolbar->setChecked(m_bookmarksToolbar->isVisible());
-    m_actionTabsOnTop->setChecked(qzSettings->tabsOnTop);
+    m_actionTabsOnTop->setChecked(tabsOnTop());
 
     m_actionPageSource->setEnabled(true);
 
@@ -1212,7 +1290,7 @@ void QupZilla::aboutToShowEncodingMenu()
 
     QList<QByteArray> available = QTextCodec::availableCodecs();
     qSort(available);
-    const QString &activeCodec = mApp->webSettings()->defaultTextEncoding();
+    const QString activeCodec = mApp->webSettings()->defaultTextEncoding();
 
     foreach (const QByteArray &name, available) {
         QTextCodec* codec = QTextCodec::codecForName(name);
@@ -1275,7 +1353,7 @@ void QupZilla::changeEncoding(QObject* obj)
     }
 
     if (QAction* action = qobject_cast<QAction*>(obj)) {
-        const QString &encoding = action->data().toString();
+        const QString encoding = action->data().toString();
         mApp->webSettings()->setDefaultTextEncoding(encoding);
 
         Settings settings;
@@ -1424,7 +1502,7 @@ void QupZilla::loadActionUrlInNewNotSelectedTab(QObject* obj)
 
 void QupZilla::loadFolderBookmarks(Menu* menu)
 {
-    const QString &folder = BookmarksModel::fromTranslatedFolder(menu->title());
+    const QString folder = BookmarksModel::fromTranslatedFolder(menu->title());
     if (folder.isEmpty()) {
         return;
     }
@@ -1611,15 +1689,27 @@ void QupZilla::showBookmarkImport()
 void QupZilla::triggerTabsOnTop(bool enable)
 {
     if (enable) {
-        m_tabWidget->showNavigationBar(m_navigationContainer);
+        m_mainLayout->insertWidget(0, m_tabWidget->getTabBar());
+        m_mainLayout->insertWidget(1, m_navigationContainer);
     }
     else {
         m_mainLayout->insertWidget(0, m_navigationContainer);
+        m_mainLayout->insertWidget(1, m_tabWidget->getTabBar());
     }
 
-    Settings settings;
-    settings.setValue("Browser-Tabs-Settings/TabsOnTop", enable);
-    qzSettings->tabsOnTop = enable;
+    m_tabsOnTopState = enable ? 1 : 0;
+    if (enable != qzSettings->tabsOnTop) {
+        Settings settings;
+        settings.setValue("Browser-Tabs-Settings/TabsOnTop", enable);
+        qzSettings->tabsOnTop = enable;
+    }
+
+#ifdef Q_OS_WIN
+    // workaround for changing TabsOnTop state when sidebar is visible
+    // TODO: we need a solution that changing TabsOnTop state
+    //       doesn't call applyBlurToMainWindow() from eventFilter()
+    QTimer::singleShot(0, this, SLOT(applyBlurToMainWindow()));
+#endif
 }
 
 void QupZilla::refreshHistory()
@@ -1706,9 +1796,10 @@ void QupZilla::webSearch()
 void QupZilla::searchOnPage()
 {
     SearchToolBar* toolBar = searchToolBar();
-    const int searchPos = qzSettings->tabsOnTop ? 1 : 2;
 
     if (!toolBar) {
+        const int searchPos = 3;
+
         toolBar = new SearchToolBar(weView(), this);
         m_mainLayout->insertWidget(searchPos, toolBar);
     }
@@ -1725,12 +1816,12 @@ void QupZilla::searchOnPage()
 
 void QupZilla::openFile()
 {
-    const QString &fileTypes = QString("%1(*.html *.htm *.shtml *.shtm *.xhtml);;"
-                                       "%2(*.png *.jpg *.jpeg *.bmp *.gif *.svg *.tiff);;"
-                                       "%3(*.txt);;"
-                                       "%4(*.*)").arg(tr("HTML files"), tr("Image files"), tr("Text files"), tr("All files"));
+    const QString fileTypes = QString("%1(*.html *.htm *.shtml *.shtm *.xhtml);;"
+                                      "%2(*.png *.jpg *.jpeg *.bmp *.gif *.svg *.tiff);;"
+                                      "%3(*.txt);;"
+                                      "%4(*.*)").arg(tr("HTML files"), tr("Image files"), tr("Text files"), tr("All files"));
 
-    const QString &filePath = QFileDialog::getOpenFileName(this, tr("Open file..."), QDir::homePath(), fileTypes);
+    const QString filePath = QzTools::getOpenFileName("MainWindow-openFile", this, tr("Open file..."), QDir::homePath(), fileTypes);
 
     if (!filePath.isEmpty()) {
         loadAddress(QUrl::fromLocalFile(filePath));
@@ -1801,6 +1892,7 @@ bool QupZilla::event(QEvent* event)
             emit setWebViewMouseTracking(true);
 #ifdef Q_OS_WIN
             if (m_usingTransparentBackground) {
+                m_tabWidget->getTabBar()->enableBluredBackground(false);
                 QtWin::extendFrameIntoClientArea(this, 0, 0, 0 , 0);
                 QtWin::enableBlurBehindWindow(this, false);
             }
@@ -1813,8 +1905,8 @@ bool QupZilla::event(QEvent* event)
             menuBar()->setVisible(m_menuBarVisible);
             statusBar()->setVisible(m_statusBarVisible);
             m_navigationContainer->show();
-            m_tabWidget->showTabBar();
             m_tabWidget->getTabBar()->updateVisibilityWithFullscreen(true);
+            m_tabWidget->getTabBar()->setVisible(true);
 #ifndef Q_OS_MAC
             m_navigationBar->setSuperMenuVisible(!m_menuBarVisible);
 #endif
@@ -1824,6 +1916,7 @@ bool QupZilla::event(QEvent* event)
             emit setWebViewMouseTracking(false);
 #ifdef Q_OS_WIN
             if (m_usingTransparentBackground) {
+                m_tabWidget->getTabBar()->enableBluredBackground(true);
                 applyBlurToMainWindow(true);
             }
 #endif
@@ -1861,11 +1954,16 @@ void QupZilla::printPage(QWebFrame* frame)
 {
     QPrintPreviewDialog* dialog = new QPrintPreviewDialog(this);
     dialog->resize(800, 750);
+    dialog->printer()->setCreator(tr("QupZilla %1 (%2)").arg(VERSION, WWWADDRESS));
 
     if (!frame) {
+        dialog->printer()->setDocName(QzTools::getFileNameFromUrl(weView()->url()));
+
         connect(dialog, SIGNAL(paintRequested(QPrinter*)), weView(), SLOT(print(QPrinter*)));
     }
     else {
+        dialog->printer()->setDocName(QzTools::getFileNameFromUrl(frame->url()));
+
         connect(dialog, SIGNAL(paintRequested(QPrinter*)), frame, SLOT(print(QPrinter*)));
     }
 
@@ -2056,7 +2154,7 @@ void QupZilla::keyPressEvent(QKeyEvent* event)
             return;
         }
         if (event->modifiers() & Qt::ControlModifier && m_useSpeedDialNumberShortcuts) {
-            const QUrl &url = mApp->plugins()->speedDial()->urlForShortcut(number - 1);
+            const QUrl url = mApp->plugins()->speedDial()->urlForShortcut(number - 1);
             if (url.isValid()) {
                 loadAddress(url);
                 return;
@@ -2122,6 +2220,8 @@ void QupZilla::closeEvent(QCloseEvent* event)
 
         return;
     }
+#else
+    QTimer::singleShot(0, this, SLOT(refreshStateOfAllActions()));
 #endif
 
     mApp->aboutToCloseWindow(this);
@@ -2133,7 +2233,7 @@ void QupZilla::closeEvent(QCloseEvent* event)
 SearchToolBar* QupZilla::searchToolBar()
 {
     SearchToolBar* toolBar = 0;
-    const int searchPos = qzSettings->tabsOnTop ? 1 : 2;
+    const int searchPos = 3;
 
     if (m_mainLayout->count() == searchPos + 1) {
         toolBar = qobject_cast<SearchToolBar*>(m_mainLayout->itemAt(searchPos)->widget());
@@ -2270,7 +2370,7 @@ void QupZilla::setMenuBookmarksAction(QAction* action)
 
 QByteArray QupZilla::saveState(int version) const
 {
-#ifdef QZ_WS_X11
+#if defined(QZ_WS_X11) && !defined(NO_X11)
     QByteArray data;
     QDataStream stream(&data, QIODevice::WriteOnly);
 
@@ -2285,7 +2385,7 @@ QByteArray QupZilla::saveState(int version) const
 
 bool QupZilla::restoreState(const QByteArray &state, int version)
 {
-#ifdef QZ_WS_X11
+#if defined(QZ_WS_X11) && !defined(NO_X11)
     QByteArray windowState;
     int desktopId = -1;
 
@@ -2301,7 +2401,7 @@ bool QupZilla::restoreState(const QByteArray &state, int version)
 #endif
 }
 
-#ifdef QZ_WS_X11
+#if defined(QZ_WS_X11) && !defined(NO_X11)
 int QupZilla::getCurrentVirtualDesktop() const
 {
     Display* display = static_cast<Display*>(QzTools::X11Display(this));
@@ -2358,17 +2458,6 @@ void QupZilla::moveToVirtualDesktop(int desktopId)
 bool QupZilla::winEvent(MSG* message, long* result)
 {
 #else
-void QupZilla::paintEvent(QPaintEvent* event)
-{
-    if (m_usingTransparentBackground && !m_actionShowFullScreen->isChecked()) {
-        QPainter p(this);
-        p.setCompositionMode(QPainter::CompositionMode_Clear);
-        p.fillRect(event->rect(), QColor(0, 0, 0, 0));
-    }
-
-    QMainWindow::paintEvent(event);
-}
-
 bool QupZilla::nativeEvent(const QByteArray &eventType, void* _message, long* result)
 {
     Q_UNUSED(eventType)
@@ -2410,6 +2499,9 @@ bool QupZilla::nativeEvent(const QByteArray &eventType, void* _message, long* re
             }
             setUpdatesEnabled(true);
         }
+        else {
+            m_usingTransparentBackground = false;
+        }
     }
 #if (QT_VERSION < 0x050000)
     return QMainWindow::winEvent(message, result);
@@ -2418,13 +2510,24 @@ bool QupZilla::nativeEvent(const QByteArray &eventType, void* _message, long* re
 #endif
 }
 
+void QupZilla::paintEvent(QPaintEvent* event)
+{
+    if (isTransparentBackgroundAllowed()) {
+        QPainter p(this);
+        p.setCompositionMode(QPainter::CompositionMode_Clear);
+        p.fillRect(event->rect(), QColor(0, 0, 0, 0));
+    }
+
+    QMainWindow::paintEvent(event);
+}
+
 void QupZilla::applyBlurToMainWindow(bool force)
 {
-    if (isClosing()) {
+    if (isClosing() || m_isStarting) {
         return;
     }
 
-    if (!force && (m_actionShowFullScreen->isChecked() || !m_usingTransparentBackground)) {
+    if (!force && !isTransparentBackgroundAllowed()) {
         return;
     }
     int topMargin = 0;
