@@ -30,7 +30,7 @@ class AutoFillWidget;
 }
 
 class WebView;
-struct AutoFillData;
+struct PasswordEntry;
 
 class QT_QUPZILLA_EXPORT AutoFillWidget : public LocationBarPopup
 {
@@ -40,7 +40,7 @@ public:
     explicit AutoFillWidget(WebView* view, QWidget* parent = 0);
     ~AutoFillWidget();
 
-    void setFormData(const QVector<AutoFillData> &data);
+    void setFormData(const QVector<PasswordEntry> &data);
 
 private slots:
     void loginToPage();
@@ -49,7 +49,7 @@ private:
     Ui::AutoFillWidget* ui;
 
     WebView* m_view;
-    QVector<AutoFillData> m_data;
+    QVector<PasswordEntry> m_data;
 };
 
 #endif // AUTOFILLWIDGET_H
