@@ -37,6 +37,7 @@
 #include <QShortcut>
 #include <QMenu>
 #include <QSqlQuery>
+#include <QLabel>
 
 BookmarksManager::BookmarksManager(QupZilla* mainClass, QWidget* parent)
     : QWidget(parent)
@@ -70,8 +71,11 @@ BookmarksManager::BookmarksManager(QupZilla* mainClass, QWidget* parent)
     connect(m_bookmarksModel, SIGNAL(folderParentChanged(QString,bool)), this, SLOT(changeFolderParent(QString,bool)));
     connect(m_bookmarksModel, SIGNAL(bookmarkParentChanged(QString,QByteArray,int,QUrl,QString,QString)), this, SLOT(changeBookmarkParent(QString,QByteArray,int,QUrl,QString,QString)));
 
-    connect(ui->optimizeDb, SIGNAL(clicked(QPoint)), this, SLOT(optimizeDb()));
-    connect(ui->importBookmarks, SIGNAL(clicked(QPoint)), this, SLOT(importBookmarks()));
+    QMenu* menu = new QMenu;
+    menu->addAction(tr("Import Bookmarks..."), this, SLOT(importBookmarks()));
+    menu->addAction(tr("Export Bookmarks to HTML..."), this, SLOT(exportBookmarks()));
+
+    ui->importExport->setMenu(menu);
 
     QShortcut* deleteAction = new QShortcut(QKeySequence("Del"), ui->bookmarksTree);
     connect(deleteAction, SIGNAL(activated()), this, SLOT(deleteItem()));
@@ -86,6 +90,15 @@ void BookmarksManager::importBookmarks()
     b->show();
 }
 
+void BookmarksManager::exportBookmarks()
+{
+    QString file = QzTools::getSaveFileName("BookmarksManager-Export", this, tr("Export to HTML..."), QDir::homePath() + "/bookmarks.html");
+
+    if (!file.isEmpty()) {
+        m_bookmarksModel->exportToHtml(file);
+    }
+}
+
 void BookmarksManager::search(const QString &string)
 {
     ui->bookmarksTree->filterString(string);
@@ -115,7 +128,7 @@ void BookmarksManager::addFolder(QWidget* parent, QString* folder, bool showInse
     QBoxLayout* layout = new QBoxLayout(QBoxLayout::TopToBottom, &dialog);
     QLabel* labelParent = new QLabel(tr("Choose parent folder for new folder: "), &dialog);
     QComboBox* combo = new QComboBox(&dialog);
-    combo->addItem(qIconProvider->fromTheme("user-bookmarks"), tr("Bookmarks"), "NO_PARENT");
+    combo->addItem(qIconProvider->fromTheme("bookmarks-organize"), tr("Bookmarks"), "NO_PARENT");
     combo->addItem(style()->standardIcon(QStyle::SP_DirOpenIcon), _bookmarksToolbar, "bookmarksToolbar");
     combo->setCurrentIndex(0);
     QLabel* labelFolder = new QLabel(tr("Choose name for new bookmark folder: "), &dialog);
@@ -242,7 +255,7 @@ void BookmarksManager::itemControlClicked(QTreeWidgetItem* item)
         return;
     }
 
-    const QUrl &url = QUrl::fromEncoded(item->text(1).toUtf8());
+    const QUrl url = QUrl::fromEncoded(item->text(1).toUtf8());
     getQupZilla()->tabWidget()->addView(url, item->text(0));
 }
 
@@ -753,16 +766,6 @@ void BookmarksManager::insertAllTabs()
     delete dialog;
 }
 
-void BookmarksManager::optimizeDb()
-{
-    BrowsingLibrary* b = qobject_cast<BrowsingLibrary*>(parentWidget()->parentWidget());
-    if (!b) {
-        return;
-    }
-    b->optimizeDatabase();
-}
-
-
 BookmarksManager::~BookmarksManager()
 {
     delete ui;
