@@ -51,11 +51,12 @@ LocationBar::LocationBar(QupZilla* mainClass)
     , m_webView(0)
     , m_pasteAndGoAction(0)
     , m_clearAction(0)
+    , m_rssIconVisible(false)
     , m_holdingAlt(false)
     , m_loadProgress(0)
     , m_progressVisible(false)
     , m_forcePaintEvent(false)
-    , m_drawCursor(true)
+    , m_inlineCompletionVisible(false)
     , m_popupClosed(false)
 {
     setObjectName("locationbar");
@@ -76,9 +77,9 @@ LocationBar::LocationBar(QupZilla* mainClass)
     addWidget(m_siteIcon, LineEdit::LeftSide);
 
     addWidget(m_autofillIcon, LineEdit::RightSide);
-    addWidget(m_goIcon, LineEdit::RightSide);
     addWidget(m_bookmarkIcon, LineEdit::RightSide);
     addWidget(m_rssIcon, LineEdit::RightSide);
+    addWidget(m_goIcon, LineEdit::RightSide);
     addWidget(down, LineEdit::RightSide);
 
     m_completer.setLocationBar(this);
@@ -99,7 +100,7 @@ LocationBar::LocationBar(QupZilla* mainClass)
     updatePlaceHolderText();
 
     // Hide icons by default
-    m_goIcon->hide();
+    hideGoButton();
     m_rssIcon->hide();
     m_autofillIcon->hide();
 }
@@ -135,6 +136,8 @@ void LocationBar::updatePlaceHolderText()
 
 void LocationBar::showCompletion(const QString &newText)
 {
+    m_inlineCompletionVisible = false;
+
     LineEdit::setText(newText);
 
     // Move cursor to the end
@@ -143,15 +146,15 @@ void LocationBar::showCompletion(const QString &newText)
 
 void LocationBar::completionPopupClosed()
 {
+    m_inlineCompletionVisible = false;
     m_popupClosed = true;
-    m_drawCursor = true;
 }
 
 QUrl LocationBar::createUrl()
 {
     QUrl urlToLoad;
 
-    //Check for Search Engine shortcut
+    // Check for Search Engine shortcut
     int firstSpacePos = text().indexOf(QLatin1Char(' '));
     if (firstSpacePos != -1) {
         QString shortcut = text().left(firstSpacePos);
@@ -163,6 +166,10 @@ QUrl LocationBar::createUrl()
         }
     }
 
+    if (m_inlineCompletionVisible) {
+        urlToLoad = WebView::guessUrlFromString(text() + m_completer.domainCompletion());
+    }
+
     if (urlToLoad.isEmpty()) {
         QUrl guessedUrl = WebView::guessUrlFromString(text());
         if (!guessedUrl.isEmpty()) {
@@ -189,35 +196,34 @@ QString LocationBar::convertUrlToText(const QUrl &url) const
 
 void LocationBar::urlEnter()
 {
+    const QUrl url = createUrl();
+    const QString urlString = convertUrlToText(url);
+
     m_completer.closePopup();
     m_webView->setFocus();
 
-    emit loadUrl(createUrl());
+    if (urlString != text()) {
+        setText(convertUrlToText(url));
+    }
+
+    emit loadUrl(url);
 }
 
 void LocationBar::textEdit()
 {
     if (!text().isEmpty()) {
         m_completer.complete(text());
+        m_inlineCompletionVisible = true;
     }
     else {
         m_completer.closePopup();
     }
 
-    // Decide whether to draw cursor
-    if (text().length() <= 1 && m_drawCursor && !m_popupClosed) {
-        m_drawCursor = false;
-    }
-
     showGoButton();
 }
 
 void LocationBar::showGoButton()
 {
-    if (m_goIcon->isVisible()) {
-        return;
-    }
-
     m_rssIconVisible = m_rssIcon->isVisible();
 
     m_bookmarkIcon->hide();
@@ -229,13 +235,12 @@ void LocationBar::showGoButton()
 
 void LocationBar::hideGoButton()
 {
-    if (!m_goIcon->isVisible()) {
-        return;
-    }
-
     m_rssIcon->setVisible(m_rssIconVisible);
     m_bookmarkIcon->show();
-    m_goIcon->hide();
+
+    if (!qzSettings->alwaysShowGoIcon) {
+        m_goIcon->hide();
+    }
 
     updateTextMargins();
 }
@@ -253,7 +258,7 @@ void LocationBar::showUrl(const QUrl &url)
         return;
     }
 
-    const QString &stringUrl = convertUrlToText(url);
+    const QString stringUrl = convertUrlToText(url);
 
     if (stringUrl == text()) {
         return;
@@ -364,11 +369,13 @@ void LocationBar::contextMenuEvent(QContextMenuEvent* event)
 
 void LocationBar::focusInEvent(QFocusEvent* event)
 {
-    const QString &stringUrl = convertUrlToText(m_webView->url());
+    if (m_webView) {
+        const QString stringUrl = convertUrlToText(m_webView->url());
 
-    // Text has been edited, let's show go button
-    if (stringUrl != text()) {
-        showGoButton();
+        // Text has been edited, let's show go button
+        if (stringUrl != text()) {
+            showGoButton();
+        }
     }
 
     LineEdit::focusInEvent(event);
@@ -437,10 +444,48 @@ void LocationBar::keyPressEvent(QKeyEvent* event)
         }
         break;
 
+    case Qt::Key_A:
+        if (event->modifiers() == Qt::ControlModifier && m_inlineCompletionVisible) {
+            m_inlineCompletionVisible = false;
+        }
+        break;
+
     case Qt::Key_Down:
         m_completer.complete(text());
         break;
 
+    case Qt::Key_End:
+    case Qt::Key_Right: {
+        const QString completionText = m_completer.domainCompletion();
+        if (m_inlineCompletionVisible && !completionText.isEmpty()) {
+            m_inlineCompletionVisible = false;
+
+            setText(text() + completionText);
+            setCursorPosition(text().size());
+            m_completer.closePopup();
+        }
+
+        if (m_completer.isPopupVisible()) {
+            m_completer.closePopup();
+        }
+        break;
+    }
+
+    case Qt::Key_Left:
+        if (m_completer.isPopupVisible()) {
+            m_completer.closePopup();
+        }
+        break;
+
+    case Qt::Key_Delete:
+        if (m_inlineCompletionVisible) {
+            m_inlineCompletionVisible = false;
+
+            update();
+            event->accept();
+        }
+        break;
+
     case Qt::Key_Escape:
         m_webView->setFocus();
         showUrl(m_webView->url());
@@ -461,6 +506,7 @@ void LocationBar::keyPressEvent(QKeyEvent* event)
             break;
 
         case Qt::AltModifier:
+            m_completer.closePopup();
             p_QupZilla->tabWidget()->addView(createUrl());
             m_holdingAlt = false;
             break;
@@ -568,37 +614,81 @@ void LocationBar::hideProgress()
 
 void LocationBar::paintEvent(QPaintEvent* event)
 {
-#ifndef Q_OS_MAC
-    if (m_drawCursor && m_completer.isPopupVisible() && !m_completer.showingMostVisited()) {
+    QStyleOptionFrameV3 option;
+    initStyleOption(&option);
+
+    int lm, tm, rm, bm;
+    getTextMargins(&lm, &tm, &rm, &bm);
+
+    QRect contentsRect = style()->subElementRect(QStyle::SE_LineEditContents, &option, this);
+    contentsRect.adjust(lm, tm, -rm, -bm);
+
+    const QFontMetrics fm = fontMetrics();
+    const int x = contentsRect.x() + 4;
+    const int y = contentsRect.y() + (contentsRect.height() - fm.height() + 1) / 2;
+    const int width = contentsRect.width() - 6;
+    const int height = fm.height();
+    const QRect textRect(x, y, width, height);
+
+    QTextOption opt;
+    opt.setWrapMode(QTextOption::NoWrap);
+
+    QPainter p(this);
+    p.setRenderHint(QPainter::Antialiasing, true);
+    p.setRenderHint(QPainter::TextAntialiasing, true);
+
+    if (hasFocus() && m_inlineCompletionVisible) {
+        // Draw inline domain completion if available
+        const QString completionText = m_completer.domainCompletion();
+
+        if (!completionText.isEmpty()) {
+            QRect completionRect = textRect;
+            completionRect.setX(completionRect.x() + fm.width(text()) + 1);
+            completionRect.setWidth(fm.width(completionText) + 1);
+
+            style()->drawPrimitive(QStyle::PE_PanelLineEdit, &option, &p, this);
+            // Text part
+            p.drawText(textRect, text(), opt);
+            // Completion part
+            p.fillRect(completionRect, palette().color(QPalette::Highlight));
+            p.setPen(palette().color(QPalette::HighlightedText));
+            p.drawText(completionRect, completionText, opt);
+            return;
+        }
+    }
+
+    if (m_completer.isPopupVisible() && !m_completer.showingMostVisited()) {
         // We need to draw cursor when popup is visible
         // But don't paint it if we are just showing most visited sites
-        LineEdit::paintEvent(event);
-        QStyleOptionFrameV3 option;
-        initStyleOption(&option);
-
-        int lm, tm, rm, bm;
-        getTextMargins(&lm, &tm, &rm, &bm);
-
-        QRect contentsRect = style()->subElementRect(QStyle::SE_LineEditContents, &option, this);
-        contentsRect.adjust(lm, tm, -rm, -bm);
-
-        const QFontMetrics &fm = fontMetrics();
+        const int cursorWidth = style()->pixelMetric(QStyle::PM_TextCursorWidth, &option, this);
+        const int cursorHeight = fm.height();
 
         QString textPart = text().left(cursorPosition());
-        int cursorXpos = contentsRect.x() + 3 + fontMetrics().width(textPart);
-        int cursorYpos = contentsRect.y() + (contentsRect.height() - fm.height() + 1) / 2;
-        int cursorWidth = style()->pixelMetric(QStyle::PM_TextCursorWidth, &option, this);
-        int cursorHeight = fontMetrics().height();
+        int cursorXpos = x + fontMetrics().width(textPart);
 
-        QPainter p(this);
-        QRect cursorRect(cursorXpos, cursorYpos, cursorWidth, cursorHeight);
-        if (isRightToLeft()) {
-            cursorRect = style()->visualRect(Qt::RightToLeft, contentsRect, cursorRect);
+        QRect cursor = cursorRect();
+        cursor.setX(cursorXpos + 1);
+        cursor.setWidth(cursorWidth);
+        cursor.setHeight(cursorHeight);
+
+        style()->drawPrimitive(QStyle::PE_PanelLineEdit, &option, &p, this);
+
+        QRect actualTextRect = textRect;
+        actualTextRect.setWidth(fontMetrics().width(text()) + 1);
+
+        // When popup is visible, Ctrl + A (Select All) is the only way to select text
+        if (selectedText() == text()) {
+            p.fillRect(actualTextRect, palette().color(QPalette::Highlight));
+            p.setPen(palette().color(QPalette::HighlightedText));
+        }
+
+        p.drawText(actualTextRect, text(), opt);
+
+        if (textRect.contains(cursor.center().x(), cursor.center().y())) {
+            p.fillRect(cursor, option.palette.text().color());
         }
-        p.fillRect(cursorRect, option.palette.text().color());
         return;
     }
-#endif
 
     if (hasFocus() || text().isEmpty() || m_forcePaintEvent) {
         LineEdit::paintEvent(event);
@@ -609,26 +699,8 @@ void LocationBar::paintEvent(QPaintEvent* event)
         return;
     }
 
-    QStyleOptionFrameV3 option;
-    initStyleOption(&option);
-
-    QPainter p(this);
-    p.setRenderHint(QPainter::Antialiasing, true);
-    p.setRenderHint(QPainter::TextAntialiasing, true);
-
     style()->drawPrimitive(QStyle::PE_PanelLineEdit, &option, &p, this);
 
-    QRect contentsRect = style()->subElementRect(QStyle::SE_LineEditContents, &option, this);
-    int lm, tm, rm, bm;
-    getTextMargins(&lm, &tm, &rm, &bm);
-    contentsRect.adjust(lm, tm, -rm, -bm);
-    QFontMetrics fm = fontMetrics();
-    const int x = contentsRect.x() + 3;
-    const int y = contentsRect.y() + (contentsRect.height() - fm.height() + 1) / 2;
-    const int width = contentsRect.width() - 6;
-    const int height = fm.height();
-    QRect textRect(x, y, width, height);
-
     QPen oldPen = p.pen();
 
     if (qzSettings->showLoadingProgress && m_progressVisible) {
@@ -675,8 +747,6 @@ void LocationBar::paintEvent(QPaintEvent* event)
     }
 
     p.setPen(oldPen);
-    QTextOption opt;
-    opt.setWrapMode(QTextOption::NoWrap);
 
     const QString hostName = m_webView->url().host();
     QString currentText = text();
@@ -706,6 +776,11 @@ void LocationBar::paintEvent(QPaintEvent* event)
             currentRect.setX(currentRect.x() + hostWidth);
             currentRect.setWidth(textRect.width() - currentRect.x() + textRect.x());
             p.setPen(lightPen);
+
+            if (currentText.isRightToLeft()) {
+                // Insert unicode control characters: prepend LRE then append LRM+PDF
+                currentText.prepend(QChar(0x202A)).append(QChar(0x200E)).append(QChar(0x202C));
+            }
         }
     }
 
