@@ -30,6 +30,8 @@
 #include "acceptlanguage.h"
 #include "cabundleupdater.h"
 #include "settings.h"
+#include "passwordmanager.h"
+#include "sslerrordialog.h"
 #include "schemehandlers/adblockschemehandler.h"
 #include "schemehandlers/qupzillaschemehandler.h"
 #include "schemehandlers/fileschemehandler.h"
@@ -100,15 +102,18 @@ void NetworkManager::loadSettings()
     settings.endGroup();
     m_acceptLanguage = AcceptLanguage::generateHeader(settings.value("Language/acceptLanguage", AcceptLanguage::defaultLanguage()).toStringList());
 
+    // Falling back to Qt 4.7 default behavior, use SslV3 by default
+    // Fixes issue with some older servers closing the connection
+
+    // However, it also makes some servers requesting TLS ClientHello
+    // not working, or showing invalid certificates.
+    // See #921
+
+    // QSslConfiguration config = QSslConfiguration::defaultConfiguration();
+    // config.setProtocol(QSsl::SslV3);
+    // QSslConfiguration::setDefaultConfiguration(config);
+
 #if defined(Q_OS_WIN) || defined(Q_OS_HAIKU) || defined(Q_OS_OS2)
-    // From doc:
-    // QSslSocket::VerifyNone ... The connection will still be encrypted, and your socket
-    // will still send its local certificate to the peer if it's requested.
-
-    QSslConfiguration config = QSslConfiguration::defaultConfiguration();
-    config.setPeerVerifyMode(QSslSocket::VerifyNone);
-    QSslConfiguration::setDefaultConfiguration(config);
-
     QString certDir = mApp->PROFILEDIR + "certificates";
     QString bundlePath = certDir + "/ca-bundle.crt";
     QString bundleVersionPath = certDir + "/bundle_version";
@@ -157,7 +162,7 @@ void NetworkManager::setSSLConfiguration(QNetworkReply* reply)
     }
 }
 
-inline uint qHash(const QSslCertificate &cert)
+static inline uint qHash(const QSslCertificate &cert)
 {
     return qHash(cert.toPem());
 }
@@ -183,7 +188,7 @@ void NetworkManager::sslError(QNetworkReply* reply, QList<QSslError> errors)
             continue;
         }
 
-        const QSslCertificate &cert = error.certificate();
+        const QSslCertificate cert = error.certificate();
 
         if (errorHash.contains(cert)) {
             errorHash[cert].append(error.errorString());
@@ -205,10 +210,10 @@ void NetworkManager::sslError(QNetworkReply* reply, QList<QSslError> errors)
 
     QHash<QSslCertificate, QStringList>::const_iterator i = errorHash.constBegin();
     while (i != errorHash.constEnd()) {
-        const QSslCertificate &cert = i.key();
-        const QStringList &errors = i.value();
+        const QSslCertificate cert = i.key();
+        const QStringList errors = i.value();
 
-        if (m_localCerts.contains(cert) || errors.isEmpty()) {
+        if (m_localCerts.contains(cert) || m_tempAllowedCerts.contains(cert) || errors.isEmpty()) {
             ++i;
             continue;
         }
@@ -236,18 +241,34 @@ void NetworkManager::sslError(QNetworkReply* reply, QList<QSslError> errors)
     QString message = QString("<b>%1</b><p>%2</p>%3<p>%4</p>").arg(title, text1, certs, text2);
 
     if (!certs.isEmpty())  {
-        QMessageBox::StandardButton button = QMessageBox::critical(webPage->view(), tr("SSL Certificate Error!"), message, QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
-        if (button == QMessageBox::No) {
+        SslErrorDialog dialog(webPage->view());
+        dialog.setText(message);
+        dialog.exec();
+
+        switch (dialog.result()) {
+        case SslErrorDialog::Yes:
+            foreach (const QSslCertificate &cert, errorHash.keys()) {
+                if (!m_localCerts.contains(cert)) {
+                    addLocalCertificate(cert);
+                }
+            }
+
+            break;
+
+        case SslErrorDialog::OnlyForThisSession:
+            foreach (const QSslCertificate &cert, errorHash.keys()) {
+                if (!m_tempAllowedCerts.contains(cert)) {
+                    m_tempAllowedCerts.append(cert);
+                }
+            }
+
+            break;
+
+        default:
             // To prevent asking user more than once for the same certificate
             webPage->addRejectedCerts(errorHash.keys());
             return;
         }
-
-        foreach (const QSslCertificate &cert, errorHash.keys()) {
-            if (!m_localCerts.contains(cert)) {
-                addLocalCertificate(cert);
-            }
-        }
     }
 
     reply->ignoreSslErrors(errors);
@@ -279,7 +300,7 @@ void NetworkManager::authentication(QNetworkReply* reply, QAuthenticator* auth)
     connect(box, SIGNAL(accepted()), dialog, SLOT(accept()));
 
     label->setText(tr("A username and password are being requested by %1. "
-                      "The site says: \"%2\"").arg(reply->url().toEncoded(), QzTools::escape(auth->realm())));
+                      "The site says: \"%2\"").arg(reply->url().host(), QzTools::escape(auth->realm())));
     formLa->addRow(label);
 
     formLa->addRow(userLab, user);
@@ -293,13 +314,13 @@ void NetworkManager::authentication(QNetworkReply* reply, QAuthenticator* auth)
     QString storedUser;
     QString storedPassword;
     if (fill->isStored(reply->url())) {
-        const AutoFillData &data = fill->getFirstFormData(reply->url());
+        const QVector<PasswordEntry> &data = fill->getFormData(reply->url());
 
-        if (data.isValid()) {
+        if (!data.isEmpty()) {
             save->setChecked(true);
             shouldUpdateEntry = true;
-            storedUser = data.username;
-            storedPassword = data.password;
+            storedUser = data.first().username;
+            storedPassword = data.first().password;
             user->setText(storedUser);
             pass->setText(storedPassword);
         }
@@ -429,39 +450,59 @@ void NetworkManager::ftpAuthentication(const QUrl &url, QAuthenticator* auth)
 
 void NetworkManager::proxyAuthentication(const QNetworkProxy &proxy, QAuthenticator* auth)
 {
-    QDialog* dialog = new QDialog();
-    dialog->setWindowTitle(tr("Proxy authorisation required"));
+    QString userName;
+    QString password;
+    QVector<PasswordEntry> psws = mApp->autoFill()->getFormData(QUrl(proxy.hostName()));
 
-    QFormLayout* formLa = new QFormLayout(dialog);
+    if (psws.isEmpty()) {
+        QDialog* dialog = new QDialog();
+        dialog->setWindowTitle(tr("Proxy authorisation required"));
 
-    QLabel* label = new QLabel(dialog);
-    QLabel* userLab = new QLabel(dialog);
-    QLabel* passLab = new QLabel(dialog);
-    userLab->setText(tr("Username: "));
-    passLab->setText(tr("Password: "));
+        QFormLayout* formLa = new QFormLayout(dialog);
 
-    QLineEdit* user = new QLineEdit(dialog);
-    QLineEdit* pass = new QLineEdit(dialog);
-    pass->setEchoMode(QLineEdit::Password);
+        QLabel* label = new QLabel(dialog);
+        QLabel* userLab = new QLabel(dialog);
+        QLabel* passLab = new QLabel(dialog);
+        userLab->setText(tr("Username: "));
+        passLab->setText(tr("Password: "));
 
-    QDialogButtonBox* box = new QDialogButtonBox(dialog);
-    box->addButton(QDialogButtonBox::Ok);
-    box->addButton(QDialogButtonBox::Cancel);
-    connect(box, SIGNAL(rejected()), dialog, SLOT(reject()));
-    connect(box, SIGNAL(accepted()), dialog, SLOT(accept()));
+        QLineEdit* user = new QLineEdit(dialog);
+        QLineEdit* pass = new QLineEdit(dialog);
+        pass->setEchoMode(QLineEdit::Password);
 
-    label->setText(tr("A username and password are being requested by proxy %1. ").arg(proxy.hostName()));
-    formLa->addRow(label);
-    formLa->addRow(userLab, user);
-    formLa->addRow(passLab, pass);
-    formLa->addWidget(box);
+        QDialogButtonBox* box = new QDialogButtonBox(dialog);
+        box->addButton(QDialogButtonBox::Ok);
+        box->addButton(QDialogButtonBox::Cancel);
+        connect(box, SIGNAL(rejected()), dialog, SLOT(reject()));
+        connect(box, SIGNAL(accepted()), dialog, SLOT(accept()));
 
-    if (dialog->exec() != QDialog::Accepted) {
-        return;
+        QCheckBox* rememberCheck = new QCheckBox(tr("Remember username and password for this proxy."), dialog);
+
+        label->setText(tr("A username and password are being requested by proxy %1. ").arg(proxy.hostName()));
+        formLa->addRow(label);
+        formLa->addRow(userLab, user);
+        formLa->addRow(passLab, pass);
+        formLa->addRow(rememberCheck);
+        formLa->addWidget(box);
+
+        if (dialog->exec() != QDialog::Accepted) {
+            return;
+        }
+
+        if (rememberCheck->isChecked()) {
+            mApp->autoFill()->addEntry(QUrl(proxy.hostName()), user->text(), pass->text());
+        }
+
+        userName = user->text();
+        password = pass->text();
+    }
+    else {
+        userName = psws.at(0).username;
+        password = psws.at(0).password;
     }
 
-    auth->setUser(user->text());
-    auth->setPassword(pass->text());
+    auth->setUser(userName);
+    auth->setPassword(password);
 }
 
 QNetworkReply* NetworkManager::createRequest(QNetworkAccessManager::Operation op, const QNetworkRequest &request, QIODevice* outgoingData)
@@ -516,10 +557,8 @@ QNetworkReply* NetworkManager::createRequest(QNetworkAccessManager::Operation op
 
     req.setRawHeader("Accept-Language", m_acceptLanguage);
 
-    req.setAttribute(QNetworkRequest::HttpPipeliningAllowedAttribute, true);
-//    if (req.attribute(QNetworkRequest::CacheLoadControlAttribute).toInt() == QNetworkRequest::PreferNetwork) {
-//        req.setAttribute(QNetworkRequest::CacheLoadControlAttribute, QNetworkRequest::PreferCache);
-//    }
+    // #830: Disabling HttpPipeling fixes issue with loading HTML5 videos on YouTube
+    //req.setAttribute(QNetworkRequest::HttpPipeliningAllowedAttribute, true);
 
     // Adblock
     if (op == QNetworkAccessManager::GetOperation) {
@@ -532,8 +571,7 @@ QNetworkReply* NetworkManager::createRequest(QNetworkAccessManager::Operation op
         }
     }
 
-    reply = QNetworkAccessManager::createRequest(op, req, outgoingData);
-    return reply;
+    return QNetworkAccessManager::createRequest(op, req, outgoingData);
 }
 
 void NetworkManager::removeLocalCertificate(const QSslCertificate &cert)
@@ -548,13 +586,13 @@ void NetworkManager::removeLocalCertificate(const QSslCertificate &cert)
     bool deleted = false;
     QDirIterator it(mApp->currentProfilePath() + "certificates", QDir::Files, QDirIterator::FollowSymlinks | QDirIterator::Subdirectories);
     while (it.hasNext()) {
-        const QString &filePath = it.next();
+        const QString filePath = it.next();
         const QList<QSslCertificate> &certs = QSslCertificate::fromPath(filePath);
         if (certs.isEmpty()) {
             continue;
         }
 
-        const QSslCertificate &cert_ = certs.at(0);
+        const QSslCertificate cert_ = certs.at(0);
         if (cert == cert_) {
             QFile file(filePath);
             if (!file.remove()) {
@@ -598,6 +636,21 @@ void NetworkManager::addLocalCertificate(const QSslCertificate &cert)
     }
 }
 
+void NetworkManager::setIgnoreAllWarnings(bool state)
+{
+    m_ignoreAllWarnings = state;
+}
+
+bool NetworkManager::isIgnoringAllWarnings()
+{
+    return m_ignoreAllWarnings;
+}
+
+NetworkProxyFactory* NetworkManager::proxyFactory() const
+{
+    return m_proxyFactory;
+}
+
 bool NetworkManager::registerSchemeHandler(const QString &scheme, SchemeHandler* handler)
 {
     if (m_schemeHandlers.contains(scheme)) {
@@ -608,6 +661,15 @@ bool NetworkManager::registerSchemeHandler(const QString &scheme, SchemeHandler*
     return true;
 }
 
+bool NetworkManager::unregisterSchemeHandler(const QString &scheme, SchemeHandler* handler)
+{
+    if (!m_schemeHandlers.contains(scheme) || m_schemeHandlers[scheme] != handler) {
+        return false;
+    }
+
+    return m_schemeHandlers.remove(scheme) == 1;
+}
+
 void NetworkManager::saveCertificates()
 {
     Settings settings;
