@@ -221,7 +221,7 @@ QString QupZillaSchemeReply::aboutPage()
         aPage.replace(QLatin1String("%MAIN-DEVELOPER-TEXT%"), authorString(QupZilla::AUTHOR.toUtf8(), "nowrep@gmail.com"));
         aPage.replace(QLatin1String("%CONTRIBUTORS%"), tr("Contributors"));
         aPage.replace(QLatin1String("%CONTRIBUTORS-TEXT%"),
-                      authorString("Mladen Pejaković", "pejakm@gmail.com") + "<br/>" +
+                      authorString("Mladen Pejaković", "pejakm@autistici.org") + "<br/>" +
                       authorString("Alexander Samilov", "alexsamilovskih@gmail.com") + "<br/>" +
                       authorString("Seyyed Razi Alavizadeh", "s.r.alavizadeh@gmail.com") + "<br/>" +
                       authorString("Franz Fellner", "alpine.art.de@googlemail.com") + "<br/>" +
@@ -249,7 +249,7 @@ QString QupZillaSchemeReply::aboutPage()
                       authorString("Oleg Brezhnev", "oleg-423@yandex.ru") + " (Russian)<br/>" +
                       authorString("Sérgio Marques", "smarquespt@gmail.com") + " (Portuguese)<br/>" +
                       authorString("Alexandre Carvalho", "alexandre05@live.com") + " (Brazilian Portuguese)<br/>" +
-                      authorString("Mladen Pejaković", "pejakm@gmail.com") + " (Serbian)<br/>" +
+                      authorString("Mladen Pejaković", "pejakm@autistici.org") + " (Serbian)<br/>" +
                       authorString("Unink-Lio", "unink4451@163.com") + " (Chinese)<br/>" +
                       authorString("Yu Hai", "yohanprc@eml.cc") + " (Chinese)<br/>" +
                       authorString("Wu Cheng-Hong", "stu2731652@gmail.com") + " (Traditional Chinese)<br/>" +
@@ -261,7 +261,8 @@ QString QupZillaSchemeReply::aboutPage()
                       authorString("Stanislav Kuznietsov", "stanislav_kuznetsov@ukr.net") + " (Ukrainian)<br/>" +
                       authorString("Seyyed Razi Alavizadeh", "s.r.alavizadeh@gmail.com") + " (Persian)<br/>" +
                       authorString("Guillem Prats", "guprej@gmail.com") + " (Catalan)<br/>" +
-                      authorString("Clara Villalba", "cvilmon@gmail.com") + " (Catalan)"
+                      authorString("Clara Villalba", "cvilmon@gmail.com") + " (Catalan)<br/>" +
+                      authorString("Xabier Aramendi", "azpidatziak@gmail.com") + " (Basque)"
                      );
         aPage = QzTools::applyDirectionToPage(aPage);
     }
@@ -394,26 +395,34 @@ QString QupZillaSchemeReply::configPage()
                       QString("<dt>%1</dt><dd>%2<dd>").arg(tr("Themes"), mApp->THEMESDIR) +
                       QString("<dt>%1</dt><dd>%2<dd>").arg(tr("Translations"), mApp->TRANSLATIONSDIR));
 
-        QString debugBuild = tr("Disabled");
-        QString webGLEnabled = tr("Disabled");
-        QString w7APIEnabled = tr("Disabled");
-        QString KDEIntegration = tr("Disabled");
-        QString portableBuild = tr("Disabled");
-
 #ifdef QUPZILLA_DEBUG_BUILD
-        debugBuild = tr("<b>Enabled</b>");
+        QString debugBuild = tr("<b>Enabled</b>");
+#else
+        QString debugBuild = tr("Disabled");
 #endif
+
 #if defined (USE_WEBGL) || (QTWEBKIT_FROM_2_3 && defined(QZ_WS_X11))
-        webGLEnabled = tr("<b>Enabled</b>");
+        QString webGLEnabled = tr("<b>Enabled</b>");
+#else
+        QString webGLEnabled = tr("Disabled");
 #endif
+
 #if defined(Q_OS_WIN) && defined(W7API)
-        w7APIEnabled = tr("<b>Enabled</b>");
+        QString w7APIEnabled = tr("<b>Enabled</b>");
+#else
+        QString w7APIEnabled = tr("Disabled");
 #endif
-#if defined(QZ_WS_X11) && defined(KDE)
-        KDEIntegration = tr("<b>Enabled</b>");
+
+#if defined(QZ_WS_X11) && defined(KDE_INTEGRATION)
+        QString KDEIntegration = tr("<b>Enabled</b>");
+#else
+        QString KDEIntegration = tr("Disabled");
 #endif
+
 #ifdef PORTABLE_BUILD
-        portableBuild = tr("<b>Enabled</b>");
+        QString portableBuild = tr("<b>Enabled</b>");
+#else
+        QString portableBuild = tr("Disabled");
 #endif
 
         cPage.replace(QLatin1String("%BUILD-CONFIG-TEXT%"),
@@ -451,7 +460,7 @@ QString QupZillaSchemeReply::configPage()
         settings->beginGroup(group);
 
         foreach (const QString &key, settings->childKeys()) {
-            const QVariant &keyValue = settings->value(key);
+            const QVariant keyValue = settings->value(key);
             QString keyString;
 
             switch (keyValue.type()) {
