@@ -1,6 +1,6 @@
 /* ============================================================
 * QupZilla - WebKit based browser
-* Copyright (C) 2010-2012  David Rosca <nowrep@gmail.com>
+* Copyright (C) 2010-2013  David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -18,11 +18,13 @@
 #ifndef LICENSEVIEWER_H
 #define LICENSEVIEWER_H
 
-#include <QTextBrowser>
+#include <QWidget>
 
 #include "qz_namespace.h"
 
-class QT_QUPZILLA_EXPORT LicenseViewer : public QTextBrowser
+class QTextBrowser;
+
+class QT_QUPZILLA_EXPORT LicenseViewer : public QWidget
 {
     Q_OBJECT
 
@@ -30,6 +32,10 @@ public:
     explicit LicenseViewer(QWidget* parent = 0);
 
     void setLicenseFile(const QString &fileName);
+    void setText(const QString &text);
+
+private:
+    QTextBrowser* m_textBrowser;
 };
 
 #endif // LICENSEVIEWER_H
