@@ -46,6 +46,7 @@
 #include "useragentdialog.h"
 #include "registerqappassociation.h"
 #include "html5permissions/html5permissionsdialog.h"
+#include "pac/pacmanager.h"
 
 #include <QSettings>
 #include <QInputDialog>
@@ -53,15 +54,23 @@
 #include <QMessageBox>
 #include <QCloseEvent>
 #include <QColorDialog>
+#include <QDesktopWidget>
 
 Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     : QDialog(parent)
     , ui(new Ui::Preferences)
     , p_QupZilla(mainClass)
+    , m_autoFillManager(0)
     , m_pluginsList(0)
+    , m_autoFillEnabled(false)
 {
     setAttribute(Qt::WA_DeleteOnClose);
     ui->setupUi(this);
+    ui->languages->setLayoutDirection(Qt::LeftToRight);
+
+    m_themesManager = new ThemeManager(ui->themesWidget, this);
+    m_pluginsList = new PluginsManager(this);
+    ui->pluginsFrame->addWidget(m_pluginsList);
 
     if (QIcon::themeName().toLower() == QLatin1String("oxygen")) {
         ui->listWidget->item(0)->setIcon(QIcon::fromTheme("preferences-desktop", QIcon(":/icons/preferences/preferences-desktop.png")));
@@ -69,7 +78,7 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
         ui->listWidget->item(2)->setIcon(QIcon::fromTheme("tab-new-background", QIcon(":/icons/preferences/applications-internet.png")));
         ui->listWidget->item(3)->setIcon(QIcon::fromTheme("preferences-system-network", QIcon(":/icons/preferences/applications-webbrowsers.png")));
         ui->listWidget->item(4)->setIcon(QIcon::fromTheme("preferences-desktop-font", QIcon(":/icons/preferences/applications-fonts.png")));
-        ui->listWidget->item(5)->setIcon(QIcon::fromTheme("preferences-desktop-keyboard-shortcuts", QIcon(":/icons/preferences/preferences-desktop-keyboard-shortcuts.png")));
+        ui->listWidget->item(5)->setIcon(QIcon::fromTheme("configure-shortcuts", QIcon(":/icons/preferences/preferences-desktop-keyboard-shortcuts.png")));
         ui->listWidget->item(6)->setIcon(QIcon::fromTheme("download", QIcon(":/icons/preferences/mail-inbox.png")));
         ui->listWidget->item(7)->setIcon(QIcon::fromTheme("user-identity", QIcon(":/icons/preferences/dialog-password.png")));
         ui->listWidget->item(8)->setIcon(QIcon::fromTheme("preferences-system-firewall", QIcon(":/icons/preferences/preferences-system-firewall.png")));
@@ -107,11 +116,11 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
 #ifdef Q_OS_WIN
     ui->checkDefaultBrowser->setChecked(settings.value("Web-Browser-Settings/CheckDefaultBrowser", DEFAULT_CHECK_DEFAULTBROWSER).toBool());
     if (mApp->associationManager()->isDefaultForAllCapabilities()) {
-        ui->checkNowDefaultBrowser->setText(tr("QupZilla is default"));
+        ui->checkNowDefaultBrowser->setText(tr("Default"));
         ui->checkNowDefaultBrowser->setEnabled(false);
     }
     else {
-        ui->checkNowDefaultBrowser->setText(tr("Make QupZilla default"));
+        ui->checkNowDefaultBrowser->setText(tr("Set as default"));
         ui->checkNowDefaultBrowser->setEnabled(true);
         connect(ui->checkNowDefaultBrowser, SIGNAL(clicked()), this, SLOT(makeQupZillaDefault()));
     }
@@ -175,7 +184,6 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     startProfileIndexChanged(ui->startProfile->currentText());
 
     //APPEREANCE
-    m_themesManager = new ThemeManager(ui->themesWidget, this);
     settings.beginGroup("Browser-View-Settings");
     ui->showStatusbar->setChecked(settings.value("showStatusBar", true).toBool());
     if (p_QupZilla) {
@@ -219,6 +227,7 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     settings.beginGroup("AddressBar");
     ui->addressbarCompletion->setCurrentIndex(settings.value("showSuggestions", 0).toInt());
     ui->completionShowSwitchTab->setChecked(settings.value("showSwitchTab", true).toBool());
+    ui->alwaysShowGoIcon->setChecked(settings.value("alwaysShowGoIcon", false).toBool());
     ui->selectAllOnFocus->setChecked(settings.value("SelectAllTextOnDoubleClick", true).toBool());
     ui->selectAllOnClick->setChecked(settings.value("SelectAllTextOnClick", false).toBool());
     ui->addCountryWithAlt->setChecked(settings.value("AddCountryDomainWithAltKey", true).toBool());
@@ -247,6 +256,7 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     ui->allowDNSPrefetch->setChecked(settings.value("DNS-Prefetch", false).toBool());
     ui->linksInFocusChain->setChecked(settings.value("IncludeLinkInFocusChain", false).toBool());
     ui->zoomTextOnly->setChecked(settings.value("zoomTextOnly", false).toBool());
+    ui->spatialNavigation->setChecked(settings.value("SpatialNavigation", false).toBool());
     ui->caretBrowsing->setChecked(settings.value("CaretBrowsing", false).toBool());
     ui->animateScrolling->setChecked(settings.value("AnimateScrolling", true).toBool());
     ui->printEBackground->setChecked(settings.value("PrintElementBackground", true).toBool());
@@ -272,8 +282,6 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     ui->allowPassManager->setChecked(settings.value("SavePasswordsOnSites", true).toBool());
     connect(ui->allowPassManager, SIGNAL(toggled(bool)), this, SLOT(showPassManager(bool)));
 
-    m_autoFillManager = new AutoFillManager(this);
-    ui->autoFillFrame->addWidget(m_autoFillManager);
     showPassManager(ui->allowPassManager->isChecked());
 
     //PRIVACY
@@ -342,10 +350,6 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     ui->loadSpeedDialsCtrl->setChecked(settings.value("useSpeedDialNumberShortcuts", true).toBool());
     settings.endGroup();
 
-    //PLUGINS
-    m_pluginsList = new PluginsManager(this);
-    ui->pluginsFrame->addWidget(m_pluginsList);
-
     //NOTIFICATIONS
     ui->useNativeSystemNotifications->setEnabled(mApp->desktopNotifications()->supportsNativeNotifications());
 
@@ -411,6 +415,7 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     ui->systemProxy->setChecked(proxyPreference == NetworkProxyFactory::SystemProxy);
     ui->noProxy->setChecked(proxyPreference == NetworkProxyFactory::NoProxy);
     ui->manualProxy->setChecked(proxyPreference == NetworkProxyFactory::DefinedProxy);
+    ui->pacProxy->setChecked(proxyPreference == NetworkProxyFactory::ProxyAutoConfig);
     if (proxyType == QNetworkProxy::HttpProxy) {
         ui->proxyType->setCurrentIndex(0);
     }
@@ -429,14 +434,18 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     ui->httpsProxyUsername->setText(settings.value("HttpsUsername", "").toString());
     ui->httpsProxyPassword->setText(settings.value("HttpsPassword", "").toString());
 
+    ui->pacUrl->setText(settings.value("PacUrl", QUrl()).toUrl().toString());
     ui->proxyExceptions->setText(settings.value("ProxyExceptions", QStringList() << "localhost" << "127.0.0.1").toStringList().join(","));
     settings.endGroup();
 
     useDifferentProxyForHttpsChanged(ui->useHttpsProxy->isChecked());
     setManualProxyConfigurationEnabled(proxyPreference == NetworkProxyFactory::DefinedProxy);
+    setProxyAutoConfigEnabled(proxyPreference == NetworkProxyFactory::ProxyAutoConfig);
 
     connect(ui->manualProxy, SIGNAL(toggled(bool)), this, SLOT(setManualProxyConfigurationEnabled(bool)));
+    connect(ui->pacProxy, SIGNAL(toggled(bool)), this, SLOT(setProxyAutoConfigEnabled(bool)));
     connect(ui->useHttpsProxy, SIGNAL(toggled(bool)), this, SLOT(useDifferentProxyForHttpsChanged(bool)));
+    connect(ui->reloadPac, SIGNAL(clicked()), this, SLOT(reloadPacFileClicked()));
 
     //CONNECTS
     connect(ui->buttonBox, SIGNAL(clicked(QAbstractButton*)), this, SLOT(buttonClicked(QAbstractButton*)));
@@ -454,6 +463,16 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     ui->version->setText(" QupZilla v" + QupZilla::VERSION);
     ui->listWidget->setCurrentRow(currentSettingsPage);
 
+    QDesktopWidget* desktop = QApplication::desktop();
+    QSize s = size();
+    if (desktop->availableGeometry(this).size().width() < s.width()) {
+        s.setWidth(desktop->availableGeometry(this).size().width() - 50);
+    }
+    if (desktop->availableGeometry(this).size().height() < s.height()) {
+        s.setHeight(desktop->availableGeometry(this).size().height() - 50);
+    }
+    resize(s);
+
 #if QTWEBKIT_TO_2_3
     ui->caretBrowsing->setHidden(true);
     ui->animateScrolling->setHidden(true);
@@ -462,6 +481,8 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
 #if QTWEBKIT_TO_2_2
     ui->html5permissions->setDisabled(true);
 #endif
+
+    QzTools::setWmClass("Preferences", this);
 }
 
 void Preferences::showStackedPage(QListWidgetItem* item)
@@ -476,14 +497,15 @@ void Preferences::showStackedPage(QListWidgetItem* item)
     ui->stackedWidget->setCurrentIndex(index);
 
     setNotificationPreviewVisible(index == 9);
+
     if (index == 10) {
         m_pluginsList->load();
     }
 
-    // Update size
-    QWidget* w = ui->stackedWidget->currentWidget();
-    if (w) {
-        w->setMinimumSize(w->sizeHint());
+    if (index == 7 && !m_autoFillManager) {
+        m_autoFillManager = new AutoFillManager(this);
+        ui->autoFillFrame->addWidget(m_autoFillManager);
+        m_autoFillManager->setVisible(m_autoFillEnabled);
     }
 }
 
@@ -519,7 +541,7 @@ void Preferences::makeQupZillaDefault()
 #ifdef Q_OS_WIN
     disconnect(ui->checkNowDefaultBrowser, SIGNAL(clicked()), this, SLOT(makeQupZillaDefault()));
     mApp->associationManager()->registerAllAssociation();
-    ui->checkNowDefaultBrowser->setText(tr("QupZilla is default"));
+    ui->checkNowDefaultBrowser->setText(tr("Default"));
     ui->checkNowDefaultBrowser->setEnabled(false);
 #endif
 }
@@ -545,7 +567,7 @@ void Preferences::useActualNewTab()
 
 void Preferences::chooseDownPath()
 {
-    QString userFileName = QFileDialog::getExistingDirectory(this, tr("Choose download location..."), QDir::homePath());
+    QString userFileName = QzTools::getExistingDirectory("Preferences-ChooseDownPath", this, tr("Choose download location..."), QDir::homePath());
     if (userFileName.isEmpty()) {
         return;
     }
@@ -559,7 +581,7 @@ void Preferences::chooseDownPath()
 
 void Preferences::chooseUserStyleClicked()
 {
-    QString file = QFileDialog::getOpenFileName(this, tr("Choose stylesheet location..."), QDir::homePath(), "*.css");
+    QString file = QzTools::getOpenFileName("Preferences-UserStyle", this, tr("Choose stylesheet location..."), QDir::homePath(), "*.css");
     if (file.isEmpty()) {
         return;
     }
@@ -576,7 +598,7 @@ void Preferences::deleteHtml5storage()
 
 void Preferences::chooseExternalDownloadManager()
 {
-    QString path = QFileDialog::getOpenFileName(this, tr("Choose executable location..."), QDir::homePath());
+    QString path = QzTools::getOpenFileName("Preferences-ExternalDownloadManager", this, tr("Choose executable location..."), QDir::homePath());
     if (path.isEmpty()) {
         return;
     }
@@ -609,6 +631,12 @@ void Preferences::setManualProxyConfigurationEnabled(bool state)
     ui->useHttpsProxy->setEnabled(state);
 }
 
+void Preferences::setProxyAutoConfigEnabled(bool state)
+{
+    ui->pacUrl->setEnabled(state);
+    ui->reloadPac->setEnabled(state);
+}
+
 void Preferences::saveHistoryChanged(bool stat)
 {
     ui->deleteHistoryOnClose->setEnabled(stat);
@@ -694,7 +722,7 @@ void Preferences::showTabPreviewsChanged(bool state)
 
 void Preferences::changeCachePathClicked()
 {
-    QString path = QFileDialog::getExistingDirectory(this, tr("Choose cache path..."), ui->cachePath->text());
+    QString path = QzTools::getExistingDirectory("Preferences-CachePath", this, tr("Choose cache path..."), ui->cachePath->text());
     if (path.isEmpty()) {
         return;
     }
@@ -702,9 +730,19 @@ void Preferences::changeCachePathClicked()
     ui->cachePath->setText(path);
 }
 
+void Preferences::reloadPacFileClicked()
+{
+    mApp->networkManager()->proxyFactory()->pacManager()->downloadPacFile();
+}
+
 void Preferences::showPassManager(bool state)
 {
-    m_autoFillManager->setVisible(state);
+    if (m_autoFillManager) {
+        m_autoFillManager->setVisible(state);
+    }
+    else {
+        m_autoFillEnabled = state;
+    }
 }
 
 void Preferences::buttonClicked(QAbstractButton* button)
@@ -805,7 +843,7 @@ void Preferences::saveSettings()
 
     switch (ui->newTab->currentIndex()) {
     case 0:
-        settings.setValue("newTabUrl", "");
+        settings.setValue("newTabUrl", QString());
         break;
 
     case 1:
@@ -877,6 +915,7 @@ void Preferences::saveSettings()
     //FONTS
     settings.beginGroup("Browser-Fonts");
     settings.setValue("StandardFont", ui->fontStandard->currentFont().family());
+    settings.setValue("CursiveFont", ui->fontCursive->currentFont().family());
     settings.setValue("FantasyFont", ui->fontFantasy->currentFont().family());
     settings.setValue("FixedFont", ui->fontFixed->currentFont().family());
     settings.setValue("SansSerifFont", ui->fontSansSerif->currentFont().family());
@@ -902,6 +941,7 @@ void Preferences::saveSettings()
     settings.setValue("DNS-Prefetch", ui->allowDNSPrefetch->isChecked());
     settings.setValue("IncludeLinkInFocusChain", ui->linksInFocusChain->isChecked());
     settings.setValue("zoomTextOnly", ui->zoomTextOnly->isChecked());
+    settings.setValue("SpatialNavigation", ui->spatialNavigation->isChecked());
     settings.setValue("CaretBrowsing", ui->caretBrowsing->isChecked());
     settings.setValue("AnimateScrolling", ui->animateScrolling->isChecked());
     settings.setValue("PrintElementBackground", ui->printEBackground->isChecked());
@@ -946,6 +986,7 @@ void Preferences::saveSettings()
     //AddressBar
     settings.beginGroup("AddressBar");
     settings.setValue("showSuggestions", ui->addressbarCompletion->currentIndex());
+    settings.setValue("alwaysShowGoIcon", ui->alwaysShowGoIcon->isChecked());
     settings.setValue("showSwitchTab", ui->completionShowSwitchTab->isChecked());
     settings.setValue("SelectAllTextOnDoubleClick", ui->selectAllOnFocus->isChecked());
     settings.setValue("SelectAllTextOnClick", ui->selectAllOnClick->isChecked());
@@ -973,6 +1014,9 @@ void Preferences::saveSettings()
     else if (ui->noProxy->isChecked()) {
         proxyPreference = NetworkProxyFactory::NoProxy;
     }
+    else if (ui->pacProxy->isChecked()) {
+        proxyPreference = NetworkProxyFactory::ProxyAutoConfig;
+    }
     else {
         proxyPreference = NetworkProxyFactory::DefinedProxy;
     }
@@ -999,6 +1043,7 @@ void Preferences::saveSettings()
     settings.setValue("HttpsUsername", ui->httpsProxyUsername->text());
     settings.setValue("HttpsPassword", ui->httpsProxyPassword->text());
 
+    settings.setValue("PacUrl", ui->pacUrl->text());
     settings.setValue("ProxyExceptions", ui->proxyExceptions->text().split(QLatin1Char(','), QString::SkipEmptyParts));
     settings.endGroup();
 
