@@ -17,6 +17,7 @@
 * ============================================================ */
 #include "qztools.h"
 #include "mainapplication.h"
+#include "settings.h"
 
 #include <QTextDocument>
 #include <QDateTime>
@@ -45,6 +46,10 @@
 #else
 #include <QX11Info>
 #endif
+#if defined(QZ_WS_X11) && !defined(NO_X11)
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#endif
 
 #ifdef Q_OS_WIN
 #include <windows.h>
@@ -90,7 +95,7 @@ QString QzTools::readAllFileContents(const QString &filename)
 void QzTools::centerWidgetOnScreen(QWidget* w)
 {
     const QRect screen = QApplication::desktop()->screenGeometry();
-    const QRect &size = w->geometry();
+    const QRect size = w->geometry();
     w->move((screen.width() - size.width()) / 2, (screen.height() - size.height()) / 2);
 }
 
@@ -248,6 +253,28 @@ QString QzTools::filterCharsFromFilename(const QString &name)
     return value;
 }
 
+QString QzTools::lastPathForFileDialog(const QString &dialogName, const QString &fallbackPath)
+{
+    Settings settings;
+    settings.beginGroup("LastFileDialogsPaths");
+    QString path = settings.value("FileDialogs/" + dialogName).toString();
+    settings.endGroup();
+
+    return path.isEmpty() ? fallbackPath : path;
+}
+
+void QzTools::saveLastPathForFileDialog(const QString &dialogName, const QString &path)
+{
+    if (path.isEmpty()) {
+        return;
+    }
+
+    Settings settings;
+    settings.beginGroup("LastFileDialogsPaths");
+    settings.setValue(dialogName, path);
+    settings.endGroup();
+}
+
 QString QzTools::alignTextToWidth(const QString &string, const QString &text, const QFontMetrics &metrics, int width)
 {
     int pos = 0;
@@ -284,19 +311,18 @@ QString QzTools::fileSizeToString(qint64 size)
 
     double _size = size / 1024.0; // KB
     if (_size < 1000) {
-        return QString::number(_size > 1 ? _size : 1, 'f', 0) + " KB";
+        return QString::number(_size > 1 ? _size : 1, 'f', 0) + " " + QObject::tr("KB");
     }
 
     _size /= 1024; // MB
     if (_size < 1000) {
-        return QString::number(_size, 'f', 1) + " MB";
+        return QString::number(_size, 'f', 1) + " " + QObject::tr("MB");
     }
 
     _size /= 1024; // GB
-    return QString::number(_size, 'f', 2) + " GB";
+    return QString::number(_size, 'f', 2) + " " + QObject::tr("GB");
 }
 
-
 QPixmap QzTools::createPixmapForSite(const QIcon &icon, const QString &title, const QString &url)
 {
     const QFontMetrics fontMetrics = QApplication::fontMetrics();
@@ -352,6 +378,34 @@ QString QzTools::applyDirectionToPage(QString &pageContents)
     return pageContents;
 }
 
+// Thanks to http://www.qtcentre.org/threads/3205-Toplevel-widget-with-rounded-corners?p=17492#post17492
+QRegion QzTools::roundedRect(const QRect &rect, int radius)
+{
+    QRegion region;
+
+    // middle and borders
+    region += rect.adjusted(radius, 0, -radius, 0);
+    region += rect.adjusted(0, radius, 0, -radius);
+
+    // top left
+    QRect corner(rect.topLeft(), QSize(radius * 2, radius * 2));
+    region += QRegion(corner, QRegion::Ellipse);
+
+    // top right
+    corner.moveTopRight(rect.topRight());
+    region += QRegion(corner, QRegion::Ellipse);
+
+    // bottom left
+    corner.moveBottomLeft(rect.bottomLeft());
+    region += QRegion(corner, QRegion::Ellipse);
+
+    // bottom right
+    corner.moveBottomRight(rect.bottomRight());
+    region += QRegion(corner, QRegion::Ellipse);
+
+    return region;
+}
+
 QIcon QzTools::iconFromFileName(const QString &fileName)
 {
     static QHash<QString, QIcon> iconCache;
@@ -374,7 +428,7 @@ QIcon QzTools::iconFromFileName(const QString &fileName)
 
 QString QzTools::resolveFromPath(const QString &name)
 {
-    const QString &path = qgetenv("PATH").trimmed();
+    const QString path = qgetenv("PATH").trimmed();
 
     if (path.isEmpty()) {
         return QString();
@@ -472,6 +526,117 @@ bool QzTools::isUtf8(const char* string)
     return true;
 }
 
+QString QzTools::getExistingDirectory(const QString &name, QWidget* parent, const QString &caption, const QString &dir, QFileDialog::Options options)
+{
+    Settings settings;
+    settings.beginGroup("FileDialogPaths");
+
+    QString lastDir = settings.value(name, dir).toString();
+
+    QString path = QFileDialog::getExistingDirectory(parent, caption, lastDir, options);
+
+    if (!path.isEmpty()) {
+        settings.setValue(name, QFileInfo(path).absolutePath());
+    }
+
+    settings.endGroup();
+    return path;
+}
+
+static QString getFilename(const QString &path)
+{
+    QFileInfo info(path);
+
+    if (info.isFile()) {
+        return info.fileName();
+    }
+
+    if (info.isDir()) {
+        return QString();
+    }
+
+    if (info.dir().exists()) {
+        return info.fileName();
+    }
+
+    return QString();
+}
+
+QString QzTools::getOpenFileName(const QString &name, QWidget* parent, const QString &caption, const QString &dir, const QString &filter, QString* selectedFilter, QFileDialog::Options options)
+{
+    Settings settings;
+    settings.beginGroup("FileDialogPaths");
+
+    QString lastDir = settings.value(name, QString()).toString();
+    QString fileName = getFilename(dir);
+
+    if (lastDir.isEmpty()) {
+        lastDir = dir;
+    }
+    else {
+        lastDir.append(QDir::separator() + fileName);
+    }
+
+    QString path = QFileDialog::getOpenFileName(parent, caption, lastDir, filter, selectedFilter, options);
+
+    if (!path.isEmpty()) {
+        settings.setValue(name, QFileInfo(path).absolutePath());
+    }
+
+    settings.endGroup();
+    return path;
+}
+
+QStringList QzTools::getOpenFileNames(const QString &name, QWidget* parent, const QString &caption, const QString &dir, const QString &filter, QString* selectedFilter, QFileDialog::Options options)
+{
+    Settings settings;
+    settings.beginGroup("FileDialogPaths");
+
+    QString lastDir = settings.value(name, QString()).toString();
+    QString fileName = getFilename(dir);
+
+    if (lastDir.isEmpty()) {
+        lastDir = dir;
+    }
+    else {
+        lastDir.append(QDir::separator() + fileName);
+    }
+
+    QStringList paths = QFileDialog::getOpenFileNames(parent, caption, lastDir, filter, selectedFilter, options);
+
+    if (!paths.isEmpty()) {
+        settings.setValue(name, QFileInfo(paths.first()).absolutePath());
+    }
+
+    settings.endGroup();
+    return paths;
+}
+
+QString QzTools::getSaveFileName(const QString &name, QWidget* parent, const QString &caption, const QString &dir, const QString &filter, QString* selectedFilter, QFileDialog::Options options)
+{
+    Settings settings;
+    settings.beginGroup("FileDialogPaths");
+
+    QString lastDir = settings.value(name, QString()).toString();
+    QString fileName = getFilename(dir);
+
+    if (lastDir.isEmpty()) {
+        lastDir = dir;
+    }
+    else {
+        lastDir.append(QDir::separator() + fileName);
+    }
+
+    QString path = QFileDialog::getSaveFileName(parent, caption, lastDir, filter, selectedFilter, options);
+
+    if (!path.isEmpty()) {
+        settings.setValue(name, QFileInfo(path).absolutePath());
+    }
+
+    settings.endGroup();
+    return path;
+}
+
 // Matches domain (assumes both pattern and domain not starting with dot)
 //  pattern = domain to be matched
 //  domain = site domain
@@ -523,7 +688,7 @@ QStringList QzTools::splitCommandArguments(const QString &command)
 
     const int strlen = line.length();
     for (int i = 0; i < strlen; ++i) {
-        const QChar &c = line.at(i);
+        const QChar c = line.at(i);
 
         if (inQuote && c == QUOTE && i > 0 && line.at(i - 1) != BSLASH) {
             QString str = line.mid(startPos, i - startPos);
@@ -555,7 +720,7 @@ QStringList QzTools::splitCommandArguments(const QString &command)
 
         if (inWord && (c == SPACE || i == strlen - 1)) {
             int len = (i == strlen - 1) ? -1 : i - startPos;
-            const QString &str = line.mid(startPos, len);
+            const QString str = line.mid(startPos, len);
 
             inWord = false;
             if (!str.isEmpty()) {
@@ -578,7 +743,7 @@ QStringList QzTools::splitCommandArguments(const QString &command)
 
 bool QzTools::startExternalProcess(const QString &executable, const QString &args)
 {
-    const QStringList &arguments = splitCommandArguments(args);
+    const QStringList arguments = splitCommandArguments(args);
 
     bool success = QProcess::startDetached(executable, arguments);
 
@@ -616,7 +781,7 @@ QString QzTools::escape(const QString &string)
 #endif
 }
 
-#ifdef QZ_WS_X11
+#if defined(QZ_WS_X11) && !defined(NO_X11)
 void* QzTools::X11Display(const QWidget* widget)
 {
     Q_UNUSED(widget)
@@ -629,6 +794,21 @@ void* QzTools::X11Display(const QWidget* widget)
 }
 #endif
 
+void QzTools::setWmClass(const QString &name, const QWidget* widget)
+{
+#if defined(QZ_WS_X11) && !defined(NO_X11)
+    QByteArray nameData = name.toUtf8();
+
+    XClassHint classHint;
+    classHint.res_name = const_cast<char*>(nameData.constData());
+    classHint.res_class = const_cast<char*>("QupZilla");
+    XSetClassHint((Display*)X11Display(widget), widget->winId(), &classHint);
+#else
+    Q_UNUSED(name)
+    Q_UNUSED(widget)
+#endif
+}
+
 QString QzTools::operatingSystem()
 {
 #ifdef Q_OS_MAC
