@@ -31,22 +31,22 @@
 #include <QMenu>
 #include <QMimeData>
 #include <QMouseEvent>
+#include <QMessageBox>
 #include <QStyleOption>
 #include <QApplication>
 #include <QTimer>
 #include <QRect>
-
-#define MAXIMUM_TAB_WIDTH 250
-#define MINIMUM_TAB_WIDTH 125
+#include <QScrollArea>
+#include <QHBoxLayout>
+#include <QDebug>
 
 TabBar::TabBar(QupZilla* mainClass, TabWidget* tabWidget)
-    : QTabBar()
+    : ComboTabBar()
     , p_QupZilla(mainClass)
     , m_tabWidget(tabWidget)
-    , m_tabPreview(new TabPreview(mainClass, tabWidget))
+    , m_tabPreview(new TabPreview(mainClass, mainClass))
     , m_showTabPreviews(false)
     , m_clickedTab(0)
-    , m_pinnedTabsCount(0)
     , m_normalTabWidth(0)
     , m_activeTabWidth(0)
 {
@@ -63,34 +63,51 @@ TabBar::TabBar(QupZilla* mainClass, TabWidget* tabWidget)
 
     connect(this, SIGNAL(currentChanged(int)), this, SLOT(currentTabChanged(int)));
     connect(this, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(contextMenuRequested(QPoint)));
-    connect(m_tabWidget, SIGNAL(pinnedTabClosed()), this, SLOT(pinnedTabClosed()));
-    connect(m_tabWidget, SIGNAL(pinnedTabAdded()), this, SLOT(pinnedTabAdded()));
 
-    m_tabPreviewTimer = new QTimer(this);
-    m_tabPreviewTimer->setInterval(200);
-    m_tabPreviewTimer->setSingleShot(true);
-    connect(m_tabPreviewTimer, SIGNAL(timeout()), m_tabPreview, SLOT(hideAnimated()));
+    m_tabPreviewShowTimer = new QTimer(this);
+    m_tabPreviewShowTimer->setInterval(300);
+    m_tabPreviewShowTimer->setSingleShot(true);
+    connect(m_tabPreviewShowTimer, SIGNAL(timeout()), this, SLOT(showTabPreview()));
+
+    m_tabPreviewHideTimer = new QTimer(this);
+    m_tabPreviewHideTimer->setInterval(300);
+    m_tabPreviewHideTimer->setSingleShot(true);
+    connect(m_tabPreviewHideTimer, SIGNAL(timeout()), m_tabPreview, SLOT(hideAnimated()));
+
+    // ComboTabBar features
+    setUsesScrollButtons(true);
+    setCloseButtonsToolTip(QupZilla::tr("Close Tab"));
+    setMaxVisiblePinnedTab(0);
+    connect(this, SIGNAL(overFlowChanged(bool)), this, SLOT(overFlowChange(bool)));
+    connect(this, SIGNAL(scrollBarValueChanged(int)), this, SLOT(hideTabPreview()));
 }
 
 void TabBar::loadSettings()
 {
     Settings settings;
     settings.beginGroup("Browser-Tabs-Settings");
-
+    m_hideTabBarWithOneTab = settings.value("hideTabsWithOneTab", false).toBool();
     m_tabPreview->setAnimationsEnabled(settings.value("tabPreviewAnimationsEnabled", true).toBool());
     m_showTabPreviews = settings.value("showTabPreviews", true).toBool();
     bool activateLastTab = settings.value("ActivateLastTabWhenClosingActual", false).toBool();
+    settings.endGroup();
 
     setSelectionBehaviorOnRemove(activateLastTab ? QTabBar::SelectPreviousTab : QTabBar::SelectRightTab);
 
-    settings.endGroup();
+    setUpLayout();
 }
 
 void TabBar::updateVisibilityWithFullscreen(bool visible)
 {
     // It is needed to save original geometry, otherwise
     // tabbar will get 3px height in fullscreen once it was hidden
-    QTabBar::setVisible(visible);
+
+    // Make sure to honor user preference
+    if (visible) {
+        visible = !(count() == 1 && m_hideTabBarWithOneTab);
+    }
+
+    ComboTabBar::setVisible(visible);
 
     if (visible) {
         setGeometry(m_originalGeometry);
@@ -104,11 +121,16 @@ void TabBar::updateVisibilityWithFullscreen(bool visible)
 
 void TabBar::setVisible(bool visible)
 {
-    if (visible) {
-        if (p_QupZilla->isFullScreen()) {
-            return;
-        }
+    if (visible && p_QupZilla->isFullScreen()) {
+        return;
+    }
 
+    // Make sure to honor user preference
+    if (visible) {
+        visible = !(count() == 1 && m_hideTabBarWithOneTab);
+    }
+
+    if (visible) {
         emit showButtons();
     }
     else {
@@ -117,8 +139,7 @@ void TabBar::setVisible(bool visible)
     }
 
     hideTabPreview(false);
-
-    QTabBar::setVisible(visible);
+    ComboTabBar::setVisible(visible);
 }
 
 void TabBar::contextMenuRequested(const QPoint &position)
@@ -127,13 +148,14 @@ void TabBar::contextMenuRequested(const QPoint &position)
     m_clickedTab = index;
 
     QMenu menu;
-    menu.addAction(QIcon(":/icons/menu/new-tab.png"), tr("&New tab"), p_QupZilla, SLOT(addTab()));
+    menu.addAction(QIcon::fromTheme("tab-new", QIcon(":/icons/menu/new-tab.png")), tr("&New tab"), p_QupZilla, SLOT(addTab()));
     menu.addSeparator();
     if (index != -1) {
         WebTab* webTab = qobject_cast<WebTab*>(m_tabWidget->widget(m_clickedTab));
         if (!webTab) {
             return;
         }
+
         if (p_QupZilla->weView(m_clickedTab)->isLoading()) {
             menu.addAction(qIconProvider->standardIcon(QStyle::SP_BrowserStop), tr("&Stop Tab"), this, SLOT(stopTab()));
         }
@@ -141,7 +163,12 @@ void TabBar::contextMenuRequested(const QPoint &position)
             menu.addAction(qIconProvider->standardIcon(QStyle::SP_BrowserReload), tr("&Reload Tab"), this, SLOT(reloadTab()));
         }
 
-        menu.addAction(tr("&Duplicate Tab"), this, SLOT(duplicateTab()));
+        menu.addAction(QIcon::fromTheme("tab-duplicate"), tr("&Duplicate Tab"), this, SLOT(duplicateTab()));
+
+        if (count() > 1 && !webTab->isPinned()) {
+            menu.addAction(QIcon::fromTheme("tab-detach"), tr("D&etach Tab"), this, SLOT(detachTab()));
+        }
+
         menu.addAction(webTab->isPinned() ? tr("Un&pin Tab") : tr("&Pin Tab"), this, SLOT(pinTab()));
         menu.addSeparator();
         menu.addAction(tr("Re&load All Tabs"), m_tabWidget, SLOT(reloadAllTabs()));
@@ -158,21 +185,31 @@ void TabBar::contextMenuRequested(const QPoint &position)
     }
     else {
         menu.addAction(tr("Reloa&d All Tabs"), m_tabWidget, SLOT(reloadAllTabs()));
-        menu.addAction(tr("Bookmark &All Ta&bs"), p_QupZilla, SLOT(bookmarkAllTabs()));
+        menu.addAction(tr("Bookmark &All Tabs"), p_QupZilla, SLOT(bookmarkAllTabs()));
         menu.addSeparator();
         QAction* action = menu.addAction(QIcon::fromTheme("user-trash"), tr("Restore &Closed Tab"), m_tabWidget, SLOT(restoreClosedTab()));
         action->setEnabled(m_tabWidget->canRestoreTab());
     }
 
     // Prevent choosing first option with double rightclick
-    const QPoint &pos = mapToGlobal(position);
+    const QPoint pos = mapToGlobal(position);
     QPoint p(pos.x(), pos.y() + 1);
     menu.exec(p);
 
     p_QupZilla->actionRestoreTab()->setEnabled(true);
 }
 
-QSize TabBar::tabSizeHint(int index) const
+void TabBar::closeAllButCurrent()
+{
+    QMessageBox::StandardButton button = QMessageBox::question(this, tr("Close Tabs"), tr("Do you really want to close other tabs?"),
+                                         QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
+
+    if (button == QMessageBox::Yes) {
+        emit closeAllButCurrent(m_clickedTab);
+    }
+}
+
+QSize TabBar::tabSizeHint(int index, bool fast) const
 {
     if (!isVisible() || !mApp->proxyStyle()) {
         // Don't calculate it when tabbar is not visible
@@ -182,93 +219,66 @@ QSize TabBar::tabSizeHint(int index) const
         return QSize(-1, -1);
     }
 
-    static int PINNED_TAB_WIDTH = -1;
-    static int MINIMUM_ACTIVE_TAB_WIDTH = -1;
+    static int PINNED_TAB_WIDTH = comboTabBarPixelMetric(ComboTabBar::PinnedTabWidth);
+    static int MINIMUM_ACTIVE_TAB_WIDTH = comboTabBarPixelMetric(ComboTabBar::ActiveTabMinimumWidth);
+    static int MAXIMUM_TAB_WIDTH = comboTabBarPixelMetric(ComboTabBar::NormalTabMaximumWidth);
+    static int MINIMUM_TAB_WIDTH = comboTabBarPixelMetric(ComboTabBar::NormalTabMinimumWidth);
 
-    if (PINNED_TAB_WIDTH == -1) {
-        PINNED_TAB_WIDTH = 16 + mApp->proxyStyle()->pixelMetric(QStyle::PM_TabBarTabHSpace, 0, this);
-        MINIMUM_ACTIVE_TAB_WIDTH = PINNED_TAB_WIDTH + mApp->proxyStyle()->pixelMetric(QStyle::PM_TabCloseIndicatorWidth, 0, this);
+    QSize size = ComboTabBar::tabSizeHint(index);
 
-        // We want to be sure buttonAddTab and buttonListTabs can't cover the active tab
-        MINIMUM_ACTIVE_TAB_WIDTH = qMax(MINIMUM_ACTIVE_TAB_WIDTH, 6 + m_tabWidget->buttonListTabs()->width() + m_tabWidget->buttonAddTab()->width());
+    // The overflowed tabs have similar size and we can use this fast method
+    if (fast) {
+        size.setWidth(index >= pinnedTabsCount() ? MINIMUM_TAB_WIDTH : PINNED_TAB_WIDTH);
+        return size;
     }
 
-    QSize size = QTabBar::tabSizeHint(index);
     WebTab* webTab = qobject_cast<WebTab*>(m_tabWidget->widget(index));
     TabBar* tabBar = const_cast <TabBar*>(this);
-    bool adjustingActiveTab = false;
 
     if (webTab && webTab->isPinned()) {
         size.setWidth(PINNED_TAB_WIDTH);
     }
     else {
-        int availableWidth = width() - (PINNED_TAB_WIDTH * m_pinnedTabsCount) - m_tabWidget->buttonListTabs()->width() - m_tabWidget->buttonAddTab()->width();
+        int availableWidth = mainTabBarWidth();
+
+        if (!m_tabWidget->buttonListTabs()->isForceHidden()) {
+            availableWidth -= comboTabBarPixelMetric(ExtraReservedWidth);
+        }
+
         if (availableWidth < 0) {
             return QSize(-1, -1);
         }
 
-        int normalTabsCount = count() - m_pinnedTabsCount;
+        const int normalTabsCount = ComboTabBar::normalTabsCount();
+
         if (availableWidth >= MAXIMUM_TAB_WIDTH * normalTabsCount) {
             m_normalTabWidth = MAXIMUM_TAB_WIDTH;
             size.setWidth(m_normalTabWidth);
         }
-        else {
+        else if (normalTabsCount > 0) {
             int maxWidthForTab = availableWidth / normalTabsCount;
             int realTabWidth = maxWidthForTab;
+            bool adjustingActiveTab = false;
+
             if (realTabWidth < MINIMUM_ACTIVE_TAB_WIDTH) {
-                maxWidthForTab = (availableWidth - MINIMUM_ACTIVE_TAB_WIDTH) / (normalTabsCount - 1);
+                maxWidthForTab = normalTabsCount > 1 ? (availableWidth - MINIMUM_ACTIVE_TAB_WIDTH) / (normalTabsCount - 1) : 0;
                 realTabWidth = MINIMUM_ACTIVE_TAB_WIDTH;
                 adjustingActiveTab = true;
             }
 
-            bool tryAdjusting = false;
+            bool tryAdjusting = availableWidth >= MINIMUM_TAB_WIDTH * normalTabsCount;
 
-            if (availableWidth < MINIMUM_TAB_WIDTH * normalTabsCount) {
-                // Tabs don't fit at all in tabbar even with MINIMUM_TAB_WIDTH
-                // We will try to use as low width of tabs as possible
-                // to try avoid overflowing tabs into tabbar buttons
-                if (maxWidthForTab < PINNED_TAB_WIDTH) {
-                    // FIXME: It overflows now
-                    m_normalTabWidth = PINNED_TAB_WIDTH;
-                    if (index == currentIndex()) {
-                        size.setWidth(realTabWidth);
-                    }
-                    else {
-                        size.setWidth(m_normalTabWidth);
-                    }
-                }
-                else {
-                    tryAdjusting = true;
-
-                    if (tabsClosable()) {
-                        // Hiding close buttons to save some space
-                        tabBar->setTabsClosable(false);
-
-                        tabBar->showCloseButton(currentIndex());
-                    }
-                }
-            }
-            else {
-                // Tabs fit into tabbar with size between MAXIMUM_TAB_WIDTH and MINIMUM_TAB_WIDTH
-                // There won't be any empty space in tabbar
-                tryAdjusting = true;
-
-                // Restore close buttons according to preferences
-                if (!tabsClosable()) {
-                    tabBar->setTabsClosable(true);
-
-                    // Hide close buttons on pinned tabs
-                    for (int i = 0; i < count(); ++i) {
-                        tabBar->updatePinnedTabCloseButton(i);
-                    }
-                }
+            if (tabsClosable() && availableWidth < (MINIMUM_TAB_WIDTH + 25) * normalTabsCount) {
+                // Hiding close buttons to save some space
+                tabBar->setTabsClosable(false);
+                tabBar->showCloseButton(currentIndex());
             }
 
             if (tryAdjusting) {
                 m_normalTabWidth = maxWidthForTab;
 
                 // Fill any empty space (we've got from rounding) with active tab
-                if (index == currentIndex()) {
+                if (index == mainTabBarCurrentIndex()) {
                     if (adjustingActiveTab) {
                         m_activeTabWidth = (availableWidth - MINIMUM_ACTIVE_TAB_WIDTH
                                             - maxWidthForTab * (normalTabsCount - 1)) + realTabWidth;
@@ -276,7 +286,6 @@ QSize TabBar::tabSizeHint(int index) const
                     else {
                         m_activeTabWidth = (availableWidth - maxWidthForTab * normalTabsCount) + maxWidthForTab;
                     }
-                    adjustingActiveTab = true;
                     size.setWidth(m_activeTabWidth);
                 }
                 else {
@@ -284,13 +293,23 @@ QSize TabBar::tabSizeHint(int index) const
                 }
             }
         }
+
+        // Restore close buttons according to preferences
+        if (!tabsClosable() && availableWidth >= (MINIMUM_TAB_WIDTH + 25) * normalTabsCount) {
+            tabBar->setTabsClosable(true);
+
+            // Hide close buttons on pinned tabs
+            for (int i = 0; i < count(); ++i) {
+                tabBar->updatePinnedTabCloseButton(i);
+            }
+        }
     }
 
     if (index == count() - 1) {
-        WebTab* currentTab = qobject_cast<WebTab*>(m_tabWidget->widget(currentIndex()));
-        int xForAddTabButton = (PINNED_TAB_WIDTH * m_pinnedTabsCount) + (count() - m_pinnedTabsCount) * (m_normalTabWidth);
+        WebTab* lastMainActiveTab = qobject_cast<WebTab*>(m_tabWidget->widget(mainTabBarCurrentIndex()));
+        int xForAddTabButton = pinTabBarWidth() + normalTabsCount() * m_normalTabWidth;
 
-        if (currentTab && !currentTab->isPinned() && m_activeTabWidth > m_normalTabWidth) {
+        if (lastMainActiveTab && m_activeTabWidth > m_normalTabWidth) {
             xForAddTabButton += m_activeTabWidth - m_normalTabWidth;
         }
 
@@ -305,6 +324,34 @@ QSize TabBar::tabSizeHint(int index) const
     return size;
 }
 
+int TabBar::comboTabBarPixelMetric(ComboTabBar::SizeType sizeType) const
+{
+    if (!mApp->proxyStyle() || !isVisible()) {
+        return -1;
+    }
+
+    switch (sizeType) {
+    case ComboTabBar::PinnedTabWidth:
+        return 16 + mApp->proxyStyle()->pixelMetric(QStyle::PM_TabBarTabHSpace, 0, this);
+
+    case ComboTabBar::ActiveTabMinimumWidth:
+    case ComboTabBar::NormalTabMinimumWidth:
+    case ComboTabBar::OverflowedTabWidth:
+        return 100;
+
+    case ComboTabBar::NormalTabMaximumWidth:
+        return 250;
+
+    case ComboTabBar::ExtraReservedWidth:
+        return m_tabWidget->buttonListTabs()->width() + m_tabWidget->buttonAddTab()->width();
+
+    default:
+        break;
+    }
+
+    return -1;
+}
+
 void TabBar::showCloseButton(int index)
 {
     if (!validIndex(index)) {
@@ -318,9 +365,7 @@ void TabBar::showCloseButton(int index)
         return;
     }
 
-    QAbstractButton* closeButton = new CloseButton(this);
-    connect(closeButton, SIGNAL(clicked()), this, SLOT(closeTabFromButton()));
-    setTabButton(index, closeButtonPosition(), closeButton);
+    insertCloseButton(index);
 }
 
 void TabBar::hideCloseButton(int index)
@@ -398,6 +443,8 @@ void TabBar::currentTabChanged(int index)
     showCloseButton(index);
     hideCloseButton(m_tabWidget->lastTabIndex());
 
+    ensureVisible(index);
+
     m_tabWidget->currentTabChanged(index);
 }
 
@@ -422,13 +469,6 @@ void TabBar::pinTab()
 
     webTab->pinTab(m_clickedTab);
 
-    if (webTab->isPinned()) {
-        m_pinnedTabsCount++;
-    }
-    else {
-        m_pinnedTabsCount--;
-    }
-
     // We need to recalculate size of all tabs and repaint tabbar
     // Unfortunately, Qt doesn't offer refresh() function as a public API
 
@@ -436,64 +476,88 @@ void TabBar::pinTab()
     setElideMode(elideMode());
 }
 
-void TabBar::pinnedTabClosed()
+void TabBar::overrideTabTextColor(int index, QColor color)
 {
-    m_pinnedTabsCount--;
+    if (!m_originalTabTextColor.isValid()) {
+        m_originalTabTextColor = tabTextColor(index);
+    }
+
+    setTabTextColor(index, color);
 }
 
-void TabBar::pinnedTabAdded()
+void TabBar::restoreTabTextColor(int index)
 {
-    m_pinnedTabsCount++;
+    setTabTextColor(index, m_originalTabTextColor);
 }
 
-int TabBar::pinnedTabsCount()
+void TabBar::showTabPreview(bool delayed)
 {
-    return m_pinnedTabsCount;
-}
+    if (delayed) {
+        int index = tabAt(mapFromGlobal(QCursor::pos()));
+        if (index == -1 || QApplication::mouseButtons() != Qt::NoButton) {
+            return;
+        }
 
-int TabBar::normalTabsCount()
-{
-    return count() - m_pinnedTabsCount;
-}
+        m_tabPreview->setPreviewIndex(index);
+        m_tabPreviewShowTimer->stop();
+    }
 
-QTabBar::ButtonPosition TabBar::iconButtonPosition()
-{
-    return (closeButtonPosition() == QTabBar::RightSide ? QTabBar::LeftSide : QTabBar::RightSide);
-}
-
-QTabBar::ButtonPosition TabBar::closeButtonPosition()
-{
-    return (QTabBar::ButtonPosition)style()->styleHint(QStyle::SH_TabBar_CloseButtonPosition, 0, this);
-}
-
-void TabBar::showTabPreview()
-{
     WebTab* webTab = qobject_cast<WebTab*>(m_tabWidget->widget(m_tabPreview->previewIndex()));
     if (!webTab) {
         return;
     }
 
-    m_tabPreviewTimer->stop();
+    m_tabPreviewHideTimer->stop();
     m_tabPreview->setWebTab(webTab, m_tabPreview->previewIndex() == currentIndex());
-    m_tabPreview->showOnRect(tabRect(m_tabPreview->previewIndex()));
+
+    QRect r(tabRect(m_tabPreview->previewIndex()));
+    r.setTopLeft(mapTo(p_QupZilla, r.topLeft()));
+    r.setBottomRight(mapTo(p_QupZilla, r.bottomRight()));
+
+    m_tabPreview->showOnRect(r);
 }
 
 void TabBar::hideTabPreview(bool delayed)
 {
+    m_tabPreviewShowTimer->stop();
+
     if (delayed) {
-        m_tabPreviewTimer->start();
+        m_tabPreviewHideTimer->start();
     }
     else {
         m_tabPreview->hideAnimated();
     }
 }
 
+void TabBar::overFlowChange(bool overFlowed)
+{
+    if (overFlowed) {
+        m_tabWidget->buttonAddTab()->setForceHidden(true);
+        m_tabWidget->buttonListTabs()->setForceHidden(true);
+        m_tabWidget->setUpLayout();
+        ensureVisible(currentIndex());
+    }
+    else {
+        m_tabWidget->buttonAddTab()->setForceHidden(false);
+        m_tabWidget->buttonListTabs()->setForceHidden(false);
+        m_tabWidget->showButtons();
+        m_tabWidget->setUpLayout();
+    }
+}
+
+void TabBar::tabInserted(int index)
+{
+    Q_UNUSED(index)
+
+    setVisible(!(count() == 1 && m_hideTabBarWithOneTab));
+}
+
 void TabBar::tabRemoved(int index)
 {
     Q_UNUSED(index)
 
-    m_tabWidget->showNavigationBar(p_QupZilla->navigationContainer());
     showCloseButton(currentIndex());
+    setVisible(!(count() == 1 && m_hideTabBarWithOneTab));
 }
 
 void TabBar::mouseDoubleClickEvent(QMouseEvent* event)
@@ -507,7 +571,7 @@ void TabBar::mouseDoubleClickEvent(QMouseEvent* event)
         return;
     }
 
-    QTabBar::mouseDoubleClickEvent(event);
+    ComboTabBar::mouseDoubleClickEvent(event);
 }
 
 void TabBar::mousePressEvent(QMouseEvent* event)
@@ -525,7 +589,7 @@ void TabBar::mousePressEvent(QMouseEvent* event)
         m_dragStartPosition = QPoint();
     }
 
-    QTabBar::mousePressEvent(event);
+    ComboTabBar::mousePressEvent(event);
 }
 
 void TabBar::mouseMoveEvent(QMouseEvent* event)
@@ -543,17 +607,18 @@ void TabBar::mouseMoveEvent(QMouseEvent* event)
     }
 
     // Tab Preview
-
     const int tab = tabAt(event->pos());
 
-    if (tab != -1 && tab != m_tabPreview->previewIndex() && event->buttons() == Qt::NoButton && m_dragStartPosition.isNull()) {
+    if (m_tabPreview->isVisible() && tab != -1 && tab != m_tabPreview->previewIndex() && event->buttons() == Qt::NoButton && m_dragStartPosition.isNull()) {
         m_tabPreview->setPreviewIndex(tab);
-        if (m_tabPreview->isVisible()) {
-            showTabPreview();
-        }
+        showTabPreview(false);
     }
 
-    QTabBar::mouseMoveEvent(event);
+    if (!m_tabPreview->isVisible()) {
+        m_tabPreviewShowTimer->start();
+    }
+
+    ComboTabBar::mouseMoveEvent(event);
 }
 
 void TabBar::mouseReleaseEvent(QMouseEvent* event)
@@ -569,7 +634,7 @@ void TabBar::mouseReleaseEvent(QMouseEvent* event)
     }
 
     if (!rect().contains(event->pos())) {
-        QTabBar::mouseReleaseEvent(event);
+        ComboTabBar::mouseReleaseEvent(event);
         return;
     }
 
@@ -583,31 +648,43 @@ void TabBar::mouseReleaseEvent(QMouseEvent* event)
         return;
     }
 
-    QTabBar::mouseReleaseEvent(event);
+    ComboTabBar::mouseReleaseEvent(event);
 }
 
 bool TabBar::event(QEvent* event)
 {
     switch (event->type()) {
     case QEvent::Leave:
-        hideTabPreview();
+        if (!rect().contains(mapFromGlobal(QCursor::pos()))) {
+            hideTabPreview();
+        }
         break;
 
-    case QEvent::ToolTip:
-        if (m_showTabPreviews) {
-            QHelpEvent* ev = static_cast<QHelpEvent*>(event);
-            if (tabAt(ev->pos()) != -1 && !m_tabPreview->isVisible() && m_dragStartPosition.isNull()) {
-                showTabPreview();
-            }
-            return true;
-        }
+    case QEvent::Wheel:
+        hideTabPreview(false);
         break;
 
     default:
         break;
     }
 
-    return QTabBar::event(event);
+    return ComboTabBar::event(event);
+}
+
+void TabBar::resizeEvent(QResizeEvent* e)
+{
+    QPoint posit;
+    posit.setY(0);
+
+    if (isRightToLeft()) {
+        posit.setX(0);
+    }
+    else {
+        posit.setX(width() - m_tabWidget->buttonListTabs()->width());
+    }
+    m_tabWidget->buttonListTabs()->move(posit);
+
+    ComboTabBar::resizeEvent(e);
 }
 
 void TabBar::wheelEvent(QWheelEvent* event)
@@ -616,7 +693,7 @@ void TabBar::wheelEvent(QWheelEvent* event)
         return;
     }
 
-    QTabBar::wheelEvent(event);
+    ComboTabBar::wheelEvent(event);
 }
 
 void TabBar::dragEnterEvent(QDragEnterEvent* event)
@@ -628,7 +705,7 @@ void TabBar::dragEnterEvent(QDragEnterEvent* event)
         return;
     }
 
-    QTabBar::dragEnterEvent(event);
+    ComboTabBar::dragEnterEvent(event);
 }
 
 void TabBar::dropEvent(QDropEvent* event)
@@ -636,7 +713,7 @@ void TabBar::dropEvent(QDropEvent* event)
     const QMimeData* mime = event->mimeData();
 
     if (!mime->hasUrls()) {
-        QTabBar::dropEvent(event);
+        ComboTabBar::dropEvent(event);
         return;
     }
 
@@ -658,66 +735,3 @@ void TabBar::disconnectObjects()
 {
     disconnect(this);
 }
-
-CloseButton::CloseButton(QWidget* parent)
-    : QAbstractButton(parent)
-{
-    setFocusPolicy(Qt::NoFocus);
-    setCursor(Qt::ArrowCursor);
-    setToolTip(QupZilla::tr("Close Tab"));
-
-    resize(sizeHint());
-}
-
-QSize CloseButton::sizeHint() const
-{
-    ensurePolished();
-    int width = style()->pixelMetric(QStyle::PM_TabCloseIndicatorWidth, 0, this);
-    int height = style()->pixelMetric(QStyle::PM_TabCloseIndicatorHeight, 0, this);
-    return QSize(width, height);
-}
-
-void CloseButton::enterEvent(QEvent* event)
-{
-    if (isEnabled()) {
-        update();
-    }
-
-    QAbstractButton::enterEvent(event);
-}
-
-void CloseButton::leaveEvent(QEvent* event)
-{
-    if (isEnabled()) {
-        update();
-    }
-
-    QAbstractButton::leaveEvent(event);
-}
-
-void CloseButton::paintEvent(QPaintEvent*)
-{
-    QPainter p(this);
-    QStyleOption opt;
-    opt.init(this);
-    opt.state |= QStyle::State_AutoRaise;
-
-    if (isEnabled() && underMouse() && !isChecked() && !isDown()) {
-        opt.state |= QStyle::State_Raised;
-    }
-    if (isChecked()) {
-        opt.state |= QStyle::State_On;
-    }
-    if (isDown()) {
-        opt.state |= QStyle::State_Sunken;
-    }
-
-    if (TabBar* tb = qobject_cast<TabBar*>(parent())) {
-        int index = tb->currentIndex();
-        if (tb->tabButton(index, tb->closeButtonPosition()) == this) {
-            opt.state |= QStyle::State_Selected;
-        }
-    }
-
-    style()->drawPrimitive(QStyle::PE_IndicatorTabClose, &opt, &p, this);
-}
