@@ -31,18 +31,20 @@
 #include "websearchbar.h"
 #include "settings.h"
 #include "qzsettings.h"
+#include "qtwin.h"
 
+#include <QTimer>
 #include <QMovie>
-#include <QMenu>
 #include <QMimeData>
 #include <QStackedWidget>
 #include <QMouseEvent>
 #include <QWebHistory>
 #include <QClipboard>
 #include <QFile>
+#include <QScrollArea>
 
 AddTabButton::AddTabButton(TabWidget* tabWidget, TabBar* tabBar)
-    : ToolButton(tabWidget)
+    : ToolButton(tabBar)
     , m_tabBar(tabBar)
     , m_tabWidget(tabWidget)
 {
@@ -61,12 +63,7 @@ void AddTabButton::wheelEvent(QWheelEvent* event)
 void AddTabButton::mouseReleaseEvent(QMouseEvent* event)
 {
     if (event->button() == Qt::MiddleButton && rect().contains(event->pos())) {
-        QString selectionClipboard = QApplication::clipboard()->text(QClipboard::Selection);
-        QUrl guessedUrl = WebView::guessUrlFromString(selectionClipboard);
-
-        if (!guessedUrl.isEmpty()) {
-            m_tabWidget->addView(guessedUrl, Qz::NT_SelectedNewEmptyTab);
-        }
+        m_tabWidget->addTabFromClipboard();
     }
 
     ToolButton::mouseReleaseEvent(event);
@@ -98,8 +95,24 @@ void AddTabButton::dropEvent(QDropEvent* event)
     }
 }
 
+void MenuTabs::mouseReleaseEvent(QMouseEvent* event)
+{
+    if (event->button() == Qt::MiddleButton) {
+        QAction* action = actionAt(event->pos());
+        if (action && action->isEnabled()) {
+            WebTab* tab = qobject_cast<WebTab*>(qvariant_cast<QWidget*>(action->data()));
+            if (tab) {
+                emit closeTab(tab->tabIndex());
+                action->setEnabled(false);
+                event->accept();
+            }
+        }
+    }
+    QMenu::mouseReleaseEvent(event);
+}
+
 TabWidget::TabWidget(QupZilla* mainClass, QWidget* parent)
-    : QTabWidget(parent)
+    : TabStackedWidget(parent)
     , p_QupZilla(mainClass)
     , m_lastTabIndex(-1)
     , m_lastBackgroundTabIndex(-1)
@@ -120,15 +133,16 @@ TabWidget::TabWidget(QupZilla* mainClass, QWidget* parent)
     connect(m_tabBar, SIGNAL(closeTab(int)), this, SLOT(closeTab(int)));
     connect(m_tabBar, SIGNAL(closeAllButCurrent(int)), this, SLOT(closeAllButCurrent(int)));
     connect(m_tabBar, SIGNAL(duplicateTab(int)), this, SLOT(duplicateTab(int)));
+    connect(m_tabBar, SIGNAL(detachTab(int)), this, SLOT(detachTab(int)));
     connect(m_tabBar, SIGNAL(tabMoved(int,int)), this, SLOT(tabMoved(int,int)));
 
     connect(m_tabBar, SIGNAL(moveAddTabButton(int)), this, SLOT(moveAddTabButton(int)));
     connect(m_tabBar, SIGNAL(showButtons()), this, SLOT(showButtons()));
     connect(m_tabBar, SIGNAL(hideButtons()), this, SLOT(hideButtons()));
 
-    m_buttonListTabs = new ToolButton(this);
+    m_buttonListTabs = new ToolButton(m_tabBar);
     m_buttonListTabs->setObjectName("tabwidget-button-opentabs");
-    m_menuTabs = new QMenu(this);
+    m_menuTabs = new MenuTabs(m_tabBar);
     m_buttonListTabs->setMenu(m_menuTabs);
     m_buttonListTabs->setPopupMode(QToolButton::InstantPopup);
     m_buttonListTabs->setToolTip(tr("List of tabs"));
@@ -140,6 +154,26 @@ TabWidget::TabWidget(QupZilla* mainClass, QWidget* parent)
     connect(m_buttonAddTab, SIGNAL(clicked()), p_QupZilla, SLOT(addTab()));
     connect(m_menuTabs, SIGNAL(aboutToShow()), this, SLOT(aboutToShowClosedTabsMenu()));
 
+    // Copy of buttons
+    m_buttonListTabs2 = new ToolButton(m_tabBar);
+    m_buttonListTabs2->setObjectName("tabwidget-button-opentabs");
+    m_buttonListTabs2->setProperty("outside-tabbar", true);
+    m_buttonListTabs2->setMenu(m_menuTabs);
+    m_buttonListTabs2->setPopupMode(QToolButton::InstantPopup);
+    m_buttonListTabs2->setToolTip(tr("List of tabs"));
+    m_buttonListTabs2->setAutoRaise(true);
+    m_buttonListTabs2->setFocusPolicy(Qt::NoFocus);
+
+    m_buttonAddTab2 = new AddTabButton(this, m_tabBar);
+    m_buttonAddTab2->setProperty("outside-tabbar", true);
+    connect(m_buttonAddTab2, SIGNAL(clicked()), p_QupZilla, SLOT(addTab()));
+
+    m_tabBar->addMainBarWidget(m_buttonAddTab2, Qt::AlignRight);
+    m_tabBar->addMainBarWidget(m_buttonListTabs2, Qt::AlignRight);
+    m_buttonAddTab2->hide();
+    m_buttonListTabs2->hide();
+    connect(m_tabBar, SIGNAL(overFlowChanged(bool)), this, SLOT(tabBarOverFlowChanged(bool)));
+
     loadSettings();
 }
 
@@ -147,7 +181,6 @@ void TabWidget::loadSettings()
 {
     Settings settings;
     settings.beginGroup("Browser-Tabs-Settings");
-    m_hideTabBarWithOneTab = settings.value("hideTabsWithOneTab", false).toBool();
     m_dontQuitWithOneTab = settings.value("dontQuitWithOneTab", false).toBool();
     m_closedInsteadOpened = settings.value("closedInsteadOpenedTabs", false).toBool();
     m_newTabAfterActive = settings.value("newTabAfterActive", true).toBool();
@@ -159,22 +192,13 @@ void TabWidget::loadSettings()
     settings.endGroup();
 
     m_tabBar->loadSettings();
-}
 
-void TabWidget::resizeEvent(QResizeEvent* e)
-{
-    QPoint posit;
-    posit.setY(0);
-    //RTL Support
-    if (QApplication::layoutDirection() == Qt::RightToLeft) {
-        posit.setX(0);
+    if (m_closedInsteadOpened) {
+        disconnect(m_menuTabs, SIGNAL(closeTab(int)), this, SLOT(closeTab(int)));
     }
     else {
-        posit.setX(width() - m_buttonListTabs->width());
+        connect(m_menuTabs, SIGNAL(closeTab(int)), this, SLOT(closeTab(int)));
     }
-    m_buttonListTabs->move(posit);
-
-    QTabWidget::resizeEvent(e);
 }
 
 WebTab* TabWidget::weTab()
@@ -199,6 +223,12 @@ void TabWidget::hideButtons()
     m_buttonAddTab->hide();
 }
 
+void TabWidget::tabBarOverFlowChanged(bool overFlowed)
+{
+    m_buttonAddTab2->setVisible(overFlowed);
+    m_buttonListTabs2->setVisible(overFlowed);
+}
+
 void TabWidget::moveAddTabButton(int posX)
 {
     int posY = (m_tabBar->height() - m_buttonAddTab->height()) / 2;
@@ -249,7 +279,7 @@ void TabWidget::aboutToShowTabsMenu()
             }
             action->setText(title);
         }
-        action->setData(i);
+        action->setData(QVariant::fromValue(qobject_cast<QWidget*>(tab)));
         connect(action, SIGNAL(triggered()), this, SLOT(actionChangeIndex()));
 
         m_menuTabs->addAction(action);
@@ -261,27 +291,38 @@ void TabWidget::aboutToShowTabsMenu()
 void TabWidget::actionChangeIndex()
 {
     if (QAction* action = qobject_cast<QAction*>(sender())) {
-        setCurrentIndex(action->data().toInt());
+        WebTab* tab = qobject_cast<WebTab*>(qvariant_cast<QWidget*>(action->data()));
+        if (tab) {
+            // needed when clicking on action of the current tab
+            m_tabBar->ensureVisible(tab->tabIndex());
+
+            setCurrentIndex(tab->tabIndex());
+        }
     }
 }
 
-int TabWidget::addView(const QUrl &url, const Qz::NewTabPositionFlags &openFlags, bool selectLine)
+int TabWidget::addView(const QUrl &url, const Qz::NewTabPositionFlags &openFlags, bool selectLine, bool pinned)
 {
-    return addView(QNetworkRequest(url), openFlags, selectLine);
+    return addView(QNetworkRequest(url), openFlags, selectLine, pinned);
 }
 
-int TabWidget::addView(const QNetworkRequest &req, const Qz::NewTabPositionFlags &openFlags, bool selectLine)
+int TabWidget::addView(const QNetworkRequest &req, const Qz::NewTabPositionFlags &openFlags, bool selectLine, bool pinned)
 {
-    return addView(req, tr("New tab"), openFlags, selectLine);
+    return addView(req, tr("New tab"), openFlags, selectLine, -1, pinned);
 }
 
-int TabWidget::addView(const QUrl &url, const QString &title, const Qz::NewTabPositionFlags &openFlags, bool selectLine, int position)
+int TabWidget::addView(const QUrl &url, const QString &title, const Qz::NewTabPositionFlags &openFlags, bool selectLine, int position, bool pinned)
 {
-    return addView(QNetworkRequest(url), title, openFlags, selectLine, position);
+    return addView(QNetworkRequest(url), title, openFlags, selectLine, position, pinned);
 }
 
-int TabWidget::addView(QNetworkRequest req, const QString &title, const Qz::NewTabPositionFlags &openFlags, bool selectLine, int position)
+int TabWidget::addView(QNetworkRequest req, const QString &title, const Qz::NewTabPositionFlags &openFlags, bool selectLine, int position, bool pinned)
 {
+#ifdef Q_OS_WIN
+    if (p_QupZilla->isTransparentBackgroundAllowed()) {
+        QtWin::extendFrameIntoClientArea(p_QupZilla);
+    }
+#endif
     QUrl url = req.url();
     m_lastTabIndex = currentIndex();
 
@@ -311,10 +352,10 @@ int TabWidget::addView(QNetworkRequest req, const QString &title, const Qz::NewT
     int index;
 
     if (position == -1) {
-        index = addTab(new WebTab(p_QupZilla, locBar), QString());
+        index = addTab(new WebTab(p_QupZilla, locBar), QString(), pinned);
     }
     else {
-        index = insertTab(position, new WebTab(p_QupZilla, locBar), QString());
+        index = insertTab(position, new WebTab(p_QupZilla, locBar), QString(), pinned);
     }
 
     TabbedWebView* webView = weTab(index)->view();
@@ -358,9 +399,44 @@ int TabWidget::addView(QNetworkRequest req, const QString &title, const Qz::NewT
         }
     }
 
+#ifdef Q_OS_WIN
+    QTimer::singleShot(0, p_QupZilla, SLOT(applyBlurToMainWindow()));
+#endif
     return index;
 }
 
+int TabWidget::addView(WebTab* tab)
+{
+    m_locationBars->addWidget(tab->locationBar());
+    tab->locationBar()->setWebView(tab->view());
+
+    int index = addTab(tab, QString());
+    setTabText(index, tab->title());
+
+    if (!tab->isLoading()) {
+        setTabIcon(index, tab->icon());
+    }
+    else {
+        startTabAnimation(index);
+    }
+
+    connect(tab->view(), SIGNAL(wantsCloseTab(int)), this, SLOT(closeTab(int)));
+    connect(tab->view(), SIGNAL(changed()), mApp, SLOT(setStateChanged()));
+    connect(tab->view(), SIGNAL(ipChanged(QString)), p_QupZilla->ipLabel(), SLOT(setText(QString)));
+
+    return index;
+}
+
+void TabWidget::addTabFromClipboard()
+{
+    QString selectionClipboard = QApplication::clipboard()->text(QClipboard::Selection);
+    QUrl guessedUrl = WebView::guessUrlFromString(selectionClipboard);
+
+    if (!guessedUrl.isEmpty()) {
+        addView(guessedUrl, Qz::NT_SelectedNewEmptyTab);
+    }
+}
+
 void TabWidget::closeTab(int index, bool force)
 {
     if (index == -1) {
@@ -391,16 +467,12 @@ void TabWidget::closeTab(int index, bool force)
         }
     }
 
-    if (webTab->isPinned()) {
-        emit pinnedTabClosed();
-    }
-
     m_locationBars->removeWidget(webView->webTab()->locationBar());
     disconnect(webView, SIGNAL(wantsCloseTab(int)), this, SLOT(closeTab(int)));
     disconnect(webView, SIGNAL(changed()), mApp, SLOT(setStateChanged()));
     disconnect(webView, SIGNAL(ipChanged(QString)), p_QupZilla->ipLabel(), SLOT(setText(QString)));
 
-    //Save last tab url and history
+    // Save last tab url and history
     m_closedTabsManager->saveView(webTab, index);
 
     if (m_isClosingToLastTabIndex && m_lastTabIndex < count() && index == currentIndex()) {
@@ -414,6 +486,11 @@ void TabWidget::closeTab(int index, bool force)
     webTab->disconnectObjects();
 
     webTab->deleteLater();
+
+    if (!m_closedInsteadOpened && m_menuTabs->isVisible()) {
+        QAction* labelAction = m_menuTabs->actions().last();
+        labelAction->setText(tr("Currently you have %1 opened tabs").arg(count() - 1));
+    }
 }
 
 void TabWidget::currentTabChanged(int index)
@@ -435,7 +512,6 @@ void TabWidget::currentTabChanged(int index)
 
     webTab->setCurrentTab();
     p_QupZilla->currentTabChanged();
-    showNavigationBar(p_QupZilla->navigationContainer());
 }
 
 void TabWidget::tabMoved(int before, int after)
@@ -448,20 +524,6 @@ void TabWidget::tabMoved(int before, int after)
     m_lastTabIndex = before;
 }
 
-void TabWidget::tabInserted(int index)
-{
-    Q_UNUSED(index)
-
-    tabBar()->setVisible(!(count() == 1 && m_hideTabBarWithOneTab));
-}
-
-void TabWidget::tabRemoved(int index)
-{
-    Q_UNUSED(index)
-
-    tabBar()->setVisible(!(count() == 1 && m_hideTabBarWithOneTab));
-}
-
 void TabWidget::startTabAnimation(int index)
 {
     if (!validIndex(index)) {
@@ -481,7 +543,6 @@ void TabWidget::startTabAnimation(int index)
     }
 
     QMovie* movie = new QMovie(":icons/other/progress.gif", QByteArray(), label);
-    movie->setSpeed(70);
     movie->start();
 
     label->setMovie(movie);
@@ -504,7 +565,7 @@ void TabWidget::setCurrentIndex(int index)
 {
     m_lastTabIndex = currentIndex();
 
-    QTabWidget::setCurrentIndex(index);
+    TabStackedWidget::setCurrentIndex(index);
 }
 
 void TabWidget::setTabIcon(int index, const QIcon &icon)
@@ -540,7 +601,7 @@ void TabWidget::setTabText(int index, const QString &text)
     }
 
     setTabToolTip(index, text);
-    QTabWidget::setTabText(index, newtext);
+    TabStackedWidget::setTabText(index, newtext);
 }
 
 void TabWidget::nextTab()
@@ -574,30 +635,11 @@ void TabWidget::reloadTab(int index)
     weTab(index)->reload();
 }
 
-void TabWidget::showTabBar()
-{
-    if (count() == 1 && m_hideTabBarWithOneTab) {
-        tabBar()->hide();
-    }
-    else {
-        tabBar()->show();
-    }
-}
-
 int TabWidget::lastTabIndex() const
 {
     return m_lastTabIndex;
 }
 
-void TabWidget::showNavigationBar(QWidget* bar)
-{
-    WebTab* tab = weTab();
-
-    if (tab) {
-        tab->showNavigationBar(bar);
-    }
-}
-
 TabBar* TabWidget::getTabBar() const
 {
     return m_tabBar;
@@ -641,6 +683,28 @@ void TabWidget::closeAllButCurrent(int index)
     }
 }
 
+void TabWidget::detachTab(int index)
+{
+    WebTab* tab = weTab(index);
+
+    if (tab->isPinned() || count() == 1) {
+        return;
+    }
+
+    m_locationBars->removeWidget(tab->locationBar());
+    disconnect(tab->view(), SIGNAL(wantsCloseTab(int)), this, SLOT(closeTab(int)));
+    disconnect(tab->view(), SIGNAL(changed()), mApp, SLOT(setStateChanged()));
+    disconnect(tab->view(), SIGNAL(ipChanged(QString)), p_QupZilla->ipLabel(), SLOT(setText(QString)));
+
+    QupZilla* window = mApp->makeNewWindow(Qz::BW_NewWindow);
+    tab->moveToWindow(window);
+    window->openWithTab(tab);
+
+    if (m_isClosingToLastTabIndex && m_lastTabIndex < count() && index == currentIndex()) {
+        setCurrentIndex(m_lastTabIndex);
+    }
+}
+
 int TabWidget::duplicateTab(int index)
 {
     if (!validIndex(index)) {
@@ -649,9 +713,9 @@ int TabWidget::duplicateTab(int index)
 
     WebTab* webTab = weTab(index);
 
-    const QUrl &url = webTab->url();
-    const QString &title = webTab->title();
-    const QByteArray &history = webTab->historyData();
+    const QUrl url = webTab->url();
+    const QString title = webTab->title();
+    const QByteArray history = webTab->historyData();
 
     QNetworkRequest req(url);
     req.setRawHeader("Referer", url.toEncoded());
@@ -840,19 +904,18 @@ void TabWidget::restorePinnedTabs()
         int addedIndex;
 
         if (!historyState.isEmpty()) {
-            addedIndex = addView(QUrl(), Qz::NT_CleanSelectedTab);
+            addedIndex = addView(QUrl(), Qz::NT_CleanSelectedTab, false, true);
 
             weTab(addedIndex)->p_restoreTab(url, historyState);
         }
         else {
-            addedIndex = addView(url);
+            addedIndex = addView(url, tr("New tab"), Qz::NT_SelectedTab, false, -1, true);
         }
 
         WebTab* webTab = weTab(addedIndex);
 
         if (webTab) {
             webTab->setPinned(true);
-            emit pinnedTabAdded();
         }
 
         m_tabBar->updatePinnedTabCloseButton(addedIndex);
