@@ -27,6 +27,7 @@
 #include "widget.h"
 #include "qztools.h"
 #include "speeddial.h"
+#include "autofill.h"
 #include "popupwebpage.h"
 #include "popupwebview.h"
 #include "networkmanagerproxy.h"
@@ -35,6 +36,7 @@
 #include "iconprovider.h"
 #include "qzsettings.h"
 #include "useragentmanager.h"
+#include "delayedfilewatcher.h"
 #include "recoverywidget.h"
 #include "html5permissions/html5permissionsmanager.h"
 #include "schemehandlers/fileschemehandler.h"
@@ -50,7 +52,6 @@
 #include <QDir>
 #include <QMouseEvent>
 #include <QWebHistory>
-#include <QFileSystemWatcher>
 #include <QTimer>
 #include <QNetworkReply>
 #include <QDebug>
@@ -65,9 +66,8 @@ QUrl WebPage::s_lastUnsupportedUrl;
 QTime WebPage::s_lastUnsupportedUrlTime;
 QList<WebPage*> WebPage::s_livingPages;
 
-WebPage::WebPage(QupZilla* mainClass)
-    : QWebPage()
-    , p_QupZilla(mainClass)
+WebPage::WebPage(QObject* parent)
+    : QWebPage(parent)
     , m_view(0)
     , m_speedDial(mApp->plugins()->speedDial())
     , m_fileWatcher(0)
@@ -147,7 +147,7 @@ void WebPage::scheduleAdjustPage()
         m_adjustingScheduled = true;
     }
     else {
-        const QSize &originalSize = webView->size();
+        const QSize originalSize = webView->size();
         QSize newSize(originalSize.width() - 1, originalSize.height() - 1);
 
         webView->resize(newSize);
@@ -186,6 +186,12 @@ bool WebPage::containsRejectedCerts(const QList<QSslCertificate> &certs)
     return matches == certs.count();
 }
 
+QWebElement WebPage::activeElement() const
+{
+    QRect activeRect = inputMethodQuery(Qt::ImMicroFocus).toRect();
+    return mainFrame()->hitTestContent(activeRect.center()).element();
+}
+
 bool WebPage::isRunningLoop()
 {
     return m_runningLoop;
@@ -233,11 +239,11 @@ void WebPage::finished()
         QFileInfo info(url().toLocalFile());
         if (info.isFile()) {
             if (!m_fileWatcher) {
-                m_fileWatcher = new QFileSystemWatcher(this);
-                connect(m_fileWatcher, SIGNAL(fileChanged(QString)), this, SLOT(watchedFileChanged(QString)));
+                m_fileWatcher = new DelayedFileWatcher(this);
+                connect(m_fileWatcher, SIGNAL(delayedFileChanged(QString)), this, SLOT(watchedFileChanged(QString)));
             }
 
-            const QString &filePath = url().toLocalFile();
+            const QString filePath = url().toLocalFile();
 
             if (QFile::exists(filePath) && !m_fileWatcher->files().contains(filePath)) {
                 m_fileWatcher->addPath(filePath);
@@ -249,7 +255,7 @@ void WebPage::finished()
     }
 
     // Autofill
-    m_autoFillData = mApp->autoFill()->completePage(this);
+    m_passwordEntries = mApp->autoFill()->completePage(this);
 
     // AdBlock
     cleanBlockedObjects();
@@ -288,16 +294,16 @@ void WebPage::handleUnsupportedContent(QNetworkReply* reply)
         return;
     }
 
-    const QUrl &url = reply->url();
+    const QUrl url = reply->url();
 
     switch (reply->error()) {
     case QNetworkReply::NoError:
         if (reply->header(QNetworkRequest::ContentTypeHeader).isValid()) {
             QString requestUrl = reply->request().url().toString(QUrl::RemoveFragment | QUrl::RemoveQuery);
             if (requestUrl.endsWith(QLatin1String(".swf"))) {
-                const QWebElement &docElement = mainFrame()->documentElement();
-                const QWebElement &object = docElement.findFirst(QString("object[src=\"%1\"]").arg(requestUrl));
-                const QWebElement &embed = docElement.findFirst(QString("embed[src=\"%1\"]").arg(requestUrl));
+                const QWebElement docElement = mainFrame()->documentElement();
+                const QWebElement object = docElement.findFirst(QString("object[src=\"%1\"]").arg(requestUrl));
+                const QWebElement embed = docElement.findFirst(QString("embed[src=\"%1\"]").arg(requestUrl));
 
                 if (!object.isNull() || !embed.isNull()) {
                     qDebug() << "WebPage::UnsupportedContent" << url << "Attempt to download flash object on site!";
@@ -332,7 +338,12 @@ void WebPage::handleUnsupportedContent(QNetworkReply* reply)
 
 void WebPage::handleUnknownProtocol(const QUrl &url)
 {
-    const QString &protocol = url.scheme();
+    const QString protocol = url.scheme();
+
+    if (protocol == QLatin1String("mailto")) {
+        desktopServicesOpen(url);
+        return;
+    }
 
     if (qzSettings->blockedProtocols.contains(protocol)) {
         qDebug() << "WebPage::handleUnknownProtocol Protocol" << protocol << "is blocked!";
@@ -346,10 +357,10 @@ void WebPage::handleUnknownProtocol(const QUrl &url)
 
     CheckBoxDialog dialog(QDialogButtonBox::Yes | QDialogButtonBox::No, view());
 
-    const QString &wrappedUrl = QzTools::alignTextToWidth(url.toString(), "<br/>", dialog.fontMetrics(), 450);
-    const QString &text = tr("QupZilla cannot handle <b>%1:</b> links. The requested link "
-                             "is <ul><li>%2</li></ul>Do you want QupZilla to try "
-                             "open this link in system application?").arg(protocol, wrappedUrl);
+    const QString wrappedUrl = QzTools::alignTextToWidth(url.toString(), "<br/>", dialog.fontMetrics(), 450);
+    const QString text = tr("QupZilla cannot handle <b>%1:</b> links. The requested link "
+                            "is <ul><li>%2</li></ul>Do you want QupZilla to try "
+                            "open this link in system application?").arg(protocol, wrappedUrl);
 
     dialog.setText(text);
     dialog.setCheckBoxText(tr("Remember my choice for this protocol"));
@@ -416,13 +427,13 @@ void WebPage::dbQuotaExceeded(QWebFrame* frame)
         return;
     }
 
-    const QWebSecurityOrigin &origin = frame->securityOrigin();
+    const QWebSecurityOrigin origin = frame->securityOrigin();
     const qint64 oldQuota = origin.databaseQuota();
 
     frame->securityOrigin().setDatabaseQuota(oldQuota * 2);
 }
 
-#if USE_QTWEBKIT_2_2
+#ifdef USE_QTWEBKIT_2_2
 void WebPage::appCacheQuotaExceeded(QWebSecurityOrigin* origin, quint64 originalQuota)
 {
     if (!origin) {
@@ -436,7 +447,7 @@ void WebPage::featurePermissionRequested(QWebFrame* frame, const QWebPage::Featu
 {
     mApp->html5permissions()->requestPermissions(this, frame, feature);
 }
-#endif
+#endif // USE_QTWEBKIT_2_2
 
 bool WebPage::event(QEvent* event)
 {
@@ -502,13 +513,6 @@ bool WebPage::acceptNavigationRequest(QWebFrame* frame, const QNetworkRequest &r
     m_lastRequestType = type;
     m_lastRequestUrl = request.url();
 
-    const QString &scheme = request.url().scheme();
-
-    if (scheme == QLatin1String("mailto")) {
-        desktopServicesOpen(request.url());
-        return false;
-    }
-
     if (type == QWebPage::NavigationTypeFormResubmitted) {
         // Don't show this dialog if app is still starting
         if (!view() || !view()->isVisible()) {
@@ -544,7 +548,15 @@ void WebPage::populateNetworkRequest(QNetworkRequest &request)
 
 QWebPage* WebPage::createWindow(QWebPage::WebWindowType type)
 {
-    return new PopupWebPage(type, p_QupZilla);
+    if (m_view) {
+        return new PopupWebPage(type, m_view->mainWindow());
+    }
+
+    if (PopupWebPage* popupPage = qobject_cast<PopupWebPage*>(this)) {
+        return new PopupWebPage(type, popupPage->mainWindow());
+    }
+
+    return 0;
 }
 
 QObject* WebPage::createPlugin(const QString &classid, const QUrl &url,
@@ -554,8 +566,8 @@ QObject* WebPage::createPlugin(const QString &classid, const QUrl &url,
     Q_UNUSED(paramNames)
     Q_UNUSED(paramValues)
 
-    if (classid == QLatin1String("RecoveryWidget") && mApp->restoreManager()) {
-        return new RecoveryWidget(qobject_cast<WebView*>(view()), p_QupZilla);
+    if (classid == QLatin1String("RecoveryWidget") && mApp->restoreManager() && m_view) {
+        return new RecoveryWidget(m_view, m_view->mainWindow());
     }
     else {
         mainFrame()->load(QUrl("qupzilla:start"));
@@ -582,12 +594,12 @@ QVector<WebPage::AdBlockedEntry> WebPage::adBlockedEntries() const
 
 bool WebPage::hasMultipleUsernames() const
 {
-    return m_autoFillData.count() > 1;
+    return m_passwordEntries.count() > 1;
 }
 
-QVector<AutoFillData> WebPage::autoFillData() const
+QVector<PasswordEntry> WebPage::autoFillData() const
 {
-    return m_autoFillData;
+    return m_passwordEntries;
 }
 
 void WebPage::cleanBlockedObjects()
@@ -597,10 +609,10 @@ void WebPage::cleanBlockedObjects()
         return;
     }
 
-    const QWebElement &docElement = mainFrame()->documentElement();
+    const QWebElement docElement = mainFrame()->documentElement();
 
     foreach (const AdBlockedEntry &entry, m_adBlockedEntries) {
-        const QString &urlString = entry.url.toString();
+        const QString urlString = entry.url.toString();
         if (urlString.endsWith(QLatin1String(".js")) || urlString.endsWith(QLatin1String(".css"))) {
             continue;
         }
@@ -635,7 +647,7 @@ void WebPage::cleanBlockedObjects()
         return;
     }
 
-    elementHiding.append(QLatin1String("{display: none !important;}\n</style>"));
+    elementHiding.append(QLatin1String("\n</style>"));
 
     QWebElement bodyElement = docElement.findFirst("body");
     bodyElement.appendInside("<style type=\"text/css\">\n/* AdBlock for QupZilla */\n" + elementHiding);
@@ -685,7 +697,7 @@ bool WebPage::extension(Extension extension, const ExtensionOption* option, Exte
             suggestedFileName = exOption->suggestedFileNames.at(0);
         }
 
-        exReturn->fileNames = QFileDialog::getOpenFileNames(0, tr("Select files to upload..."), suggestedFileName);
+        exReturn->fileNames = QzTools::getOpenFileNames("WebPage-UploadFiles", 0, tr("Select files to upload..."), suggestedFileName);
         return true;
     }
 
@@ -783,7 +795,7 @@ bool WebPage::extension(Extension extension, const ExtensionOption* option, Exte
                         WebView* view = qobject_cast<WebView*>(popupPage->view());
                         if (view) {
                             // Closing blocked popup
-                            p_QupZilla->adBlockIcon()->popupBlocked(rule, exOption->url);
+                            popupPage->mainWindow()->adBlockIcon()->popupBlocked(rule, exOption->url);
                             view->closeView();
                         }
                     }
@@ -811,7 +823,7 @@ bool WebPage::extension(Extension extension, const ExtensionOption* option, Exte
         return false;    // Downloads
     }
 
-    const QUrl &loadedUrl = exOption->url;
+    const QUrl loadedUrl = exOption->url;
     exReturn->baseUrl = loadedUrl;
 
     QFile file(":/html/errorPage.html");
@@ -986,10 +998,18 @@ QString WebPage::chooseFile(QWebFrame* originatingFrame, const QString &oldFile)
         suggFileName = oldFile;
     }
 
-    const QString &fileName = QFileDialog::getOpenFileName(originatingFrame->page()->view(), tr("Choose file..."), suggFileName);
+    const QString fileName = QzTools::getOpenFileName("WebPage-ChooseFile", originatingFrame->page()->view(), tr("Choose file..."), suggFileName);
 
     if (!fileName.isEmpty()) {
         s_lastUploadLocation = fileName;
+
+        // Check if we can read from file
+        QFile file(fileName);
+        if (!file.open(QFile::ReadOnly)) {
+            const QString msg = tr("Cannot read data from <b>%1</b>. Upload was cancelled!").arg(fileName);
+            QMessageBox::critical(view(), tr("Cannot read file!"), msg);
+            return QString();
+        }
     }
 
     return fileName;
