@@ -37,6 +37,10 @@
 #include "qtwebkit/spellcheck/speller.h"
 #endif
 
+#ifdef Q_OS_MAC
+#include "macwebviewscroller.h"
+#endif
+
 #include <QDir>
 #include <QTimer>
 #include <QDesktopServices>
@@ -74,6 +78,10 @@ WebView::WebView(QWidget* parent)
 #if QTWEBKIT_TO_2_3
     installEventFilter(this);
 #endif
+
+#ifdef Q_OS_MAC
+    new MacWebViewScroller(this);
+#endif
 }
 
 QIcon WebView::icon() const
@@ -151,15 +159,11 @@ void WebView::setPage(QWebPage* page)
 
     mApp->plugins()->emitWebPageCreated(m_page);
 
-    /* Set white background by default.
-       Fixes issue with dark themes.
-
-       See #602
-     */
+    // Set white background by default.
+    // Fixes issue with dark themes. See #602
     QPalette pal = palette();
     pal.setBrush(QPalette::Base, Qt::white);
     page->setPalette(pal);
-
 }
 
 void WebView::load(const QUrl &url)
@@ -169,7 +173,7 @@ void WebView::load(const QUrl &url)
 
 void WebView::load(const QNetworkRequest &request, QNetworkAccessManager::Operation operation, const QByteArray &body)
 {
-    const QUrl &reqUrl = request.url();
+    const QUrl reqUrl = request.url();
 
     if (reqUrl.scheme() == QLatin1String("javascript")) {
         // Getting scriptSource from PercentEncoding to properly load bookmarklets
@@ -192,8 +196,10 @@ void WebView::load(const QNetworkRequest &request, QNetworkAccessManager::Operat
         return;
     }
 
-    const QUrl &searchUrl = mApp->searchEnginesManager()->searchUrl(reqUrl.toString());
-    QWebView::load(searchUrl);
+    SearchEnginesManager::SearchResult res = mApp->searchEnginesManager()->searchResult(reqUrl.toString());
+    const QUrl searchUrl = res.request.url();
+
+    QWebView::load(res.request, res.operation, res.data);
 
     emit urlChanged(searchUrl);
     m_aboutToLoadUrl = searchUrl;
@@ -225,23 +231,16 @@ bool WebView::hasRss() const
     return m_hasRss;
 }
 
+QWebElement WebView::activeElement() const
+{
+    QRect activeRect = inputMethodQuery(Qt::ImMicroFocus).toRect();
+    return page()->mainFrame()->hitTestContent(activeRect.center()).element();
+}
+
 bool WebView::isUrlValid(const QUrl &url)
 {
-    const QString &urlScheme = url.scheme();
-    if (urlScheme == QLatin1String("data") || urlScheme == QLatin1String("qrc") ||
-            urlScheme == QLatin1String("mailto")) {
-        return true;
-    }
-
-    if (urlScheme == QLatin1String("qupzilla") || urlScheme == QLatin1String("file")) {
-        return !url.path().isEmpty();
-    }
-
-    if (url.isValid() && !url.host().isEmpty() && !urlScheme.isEmpty()) {
-        return true;
-    }
-
-    return false;
+    // Valid url must have scheme and actually contains something (therefore scheme:// is invalid)
+    return url.isValid() && !url.scheme().isEmpty() && (!url.host().isEmpty() || !url.path().isEmpty() || url.hasQuery());
 }
 
 QUrl WebView::guessUrlFromString(const QString &string)
@@ -352,6 +351,12 @@ void WebView::forward()
     }
 }
 
+void WebView::editDelete()
+{
+    QKeyEvent ev(QEvent::KeyPress, Qt::Key_Delete, Qt::NoModifier);
+    QApplication::sendEvent(this, &ev);
+}
+
 void WebView::selectAll()
 {
     triggerPageAction(QWebPage::SelectAll);
@@ -417,7 +422,7 @@ void WebView::checkRss()
 
     m_rssChecked = true;
     QWebFrame* frame = page()->mainFrame();
-    const QWebElementCollection &links = frame->findAllElements("link[type=\"application/rss+xml\"]");
+    const QWebElementCollection links = frame->findAllElements("link[type=\"application/rss+xml\"]");
 
     m_hasRss = links.count() != 0;
     emit rssChanged(m_hasRss);
@@ -441,7 +446,7 @@ void WebView::slotUrlChanged(const QUrl &url)
     }
 
     // Disable touch mocking on pages known not to work properly
-    const QString &host = url.host();
+    const QString host = url.host();
     m_disableTouchMocking = false;
 
     foreach (const QString &site, exceptions) {
@@ -461,14 +466,14 @@ void WebView::openUrlInNewWindow()
 void WebView::sendLinkByMail()
 {
     if (QAction* action = qobject_cast<QAction*>(sender())) {
-        const QUrl &mailUrl = QUrl::fromEncoded("mailto:?body=" + QUrl::toPercentEncoding(action->data().toUrl().toEncoded()));
+        const QUrl mailUrl = QUrl::fromEncoded("mailto:%20?body=" + QUrl::toPercentEncoding(action->data().toUrl().toEncoded()));
         QDesktopServices::openUrl(mailUrl);
     }
 }
 
 void WebView::sendPageByMail()
 {
-    const QUrl &mailUrl = QUrl::fromEncoded("mailto:?body=" + QUrl::toPercentEncoding(url().toEncoded()) + "&subject=" + QUrl::toPercentEncoding(title()));
+    const QUrl mailUrl = QUrl::fromEncoded("mailto:%20?body=" + QUrl::toPercentEncoding(url().toEncoded()) + "&subject=" + QUrl::toPercentEncoding(title()));
     QDesktopServices::openUrl(mailUrl);
 }
 
@@ -481,6 +486,10 @@ void WebView::copyLinkToClipboard()
 
 void WebView::savePageAs()
 {
+    if (url().isEmpty() || url().toString() == QLatin1String("about:blank")) {
+        return;
+    }
+
     QNetworkRequest request(url());
     QString suggestedFileName = QzTools::getFileNameFromUrl(url());
     if (!suggestedFileName.contains(QLatin1Char('.'))) {
@@ -497,6 +506,11 @@ void WebView::savePageAs()
     dManager->download(request, info);
 }
 
+void WebView::openUrlInNewTab(const QUrl &url, Qz::NewTabPositionFlag position)
+{
+    loadInNewTab(QNetworkRequest(url), QNetworkAccessManager::GetOperation, QByteArray(), position);
+}
+
 void WebView::downloadUrlToDisk()
 {
     if (QAction* action = qobject_cast<QAction*>(sender())) {
@@ -551,8 +565,8 @@ void WebView::searchSelectedText()
         }
     }
 
-    const QUrl &urlToLoad = mApp->searchEnginesManager()->searchUrl(engine, selectedText());
-    openUrlInNewTab(urlToLoad, Qz::NT_SelectedTab);
+    SearchEnginesManager::SearchResult res = mApp->searchEnginesManager()->searchResult(engine, selectedText());
+    loadInNewTab(res.request, res.operation, res.data, Qz::NT_SelectedTab);
 }
 
 void WebView::searchSelectedTextInBackgroundTab()
@@ -564,8 +578,8 @@ void WebView::searchSelectedTextInBackgroundTab()
         }
     }
 
-    const QUrl &urlToLoad = mApp->searchEnginesManager()->searchUrl(engine, selectedText());
-    openUrlInNewTab(urlToLoad, Qz::NT_NotSelectedTab);
+    SearchEnginesManager::SearchResult res = mApp->searchEnginesManager()->searchResult(engine, selectedText());
+    loadInNewTab(res.request, res.operation, res.data, Qz::NT_NotSelectedTab);
 }
 
 void WebView::bookmarkLink()
@@ -747,10 +761,10 @@ void WebView::checkForForm(QMenu* menu, const QWebElement &element)
         return;
     }
 
-    const QString &url = parentElement.attribute("action");
-    const QString &method = parentElement.hasAttribute("method") ? parentElement.attribute("method").toUpper() : "GET";
+    const QString url = parentElement.attribute("action");
+    const QString method = parentElement.hasAttribute("method") ? parentElement.attribute("method").toUpper() : "GET";
 
-    if (!url.isEmpty() && method == QLatin1String("GET")) {
+    if (!url.isEmpty() && (method == QLatin1String("GET") || method == QLatin1String("POST"))) {
         menu->addAction(QIcon(":icons/menu/search-icon.png"), tr("Create Search Engine"), this, SLOT(createSearchEngine()));
 
         m_clickedElement = element;
@@ -768,9 +782,20 @@ void WebView::createContextMenu(QMenu* menu, const QWebHitTestResult &hitTest, c
         m_actionsInitialized = true;
 
         pageAction(QWebPage::Cut)->setIcon(QIcon::fromTheme("edit-cut"));
+        pageAction(QWebPage::Cut)->setText(tr("Cut"));
         pageAction(QWebPage::Copy)->setIcon(QIcon::fromTheme("edit-copy"));
+        pageAction(QWebPage::Copy)->setText(tr("Copy"));
         pageAction(QWebPage::Paste)->setIcon(QIcon::fromTheme("edit-paste"));
+        pageAction(QWebPage::Paste)->setText(tr("Paste"));
         pageAction(QWebPage::SelectAll)->setIcon(QIcon::fromTheme("edit-select-all"));
+        pageAction(QWebPage::SelectAll)->setText(tr("Select All"));
+
+        pageAction(QWebPage::SetTextDirectionDefault)->setText(tr("Default"));
+        pageAction(QWebPage::SetTextDirectionLeftToRight)->setText(tr("Left to Right"));
+        pageAction(QWebPage::SetTextDirectionRightToLeft)->setText(tr("Right to Left"));
+        pageAction(QWebPage::ToggleBold)->setText(tr("Bold"));
+        pageAction(QWebPage::ToggleItalic)->setText(tr("Italic"));
+        pageAction(QWebPage::ToggleUnderline)->setText(tr("Underline"));
 
         m_actionReload = new QAction(qIconProvider->standardIcon(QStyle::SP_BrowserReload), tr("&Reload"), this);
         m_actionStop = new QAction(qIconProvider->standardIcon(QStyle::SP_BrowserStop), tr("S&top"), this);
@@ -782,6 +807,7 @@ void WebView::createContextMenu(QMenu* menu, const QWebHitTestResult &hitTest, c
         m_actionStop->setEnabled(isLoading());
     }
 
+    // cppcheck-suppress variableScope
     int spellCheckActionCount = 0;
 
 #ifdef USE_HUNSPELL
@@ -805,6 +831,7 @@ void WebView::createContextMenu(QMenu* menu, const QWebHitTestResult &hitTest, c
     }
 
     if (hitTest.isContentEditable()) {
+        // This only checks if the menu is empty (only spellchecker actions added)
         if (menu->actions().count() == spellCheckActionCount) {
             QMenu* pageMenu = page()->createStandardContextMenu();
             // Apparently createStandardContextMenu() can return null pointer
@@ -826,6 +853,11 @@ void WebView::createContextMenu(QMenu* menu, const QWebHitTestResult &hitTest, c
 
                     menu->addAction(act);
 
+                    if (act == pageAction(QWebPage::Paste)) {
+                        QAction* a = menu->addAction(QIcon::fromTheme("edit-delete"), tr("Delete"), this, SLOT(editDelete()));
+                        a->setEnabled(!selectedText().isEmpty());
+                    }
+
                     ++i;
                 }
 
@@ -885,7 +917,7 @@ void WebView::createPageContextMenu(QMenu* menu, const QPoint &pos)
         m_clickedFrame = frameAtPos;
         Menu* frameMenu = new Menu(tr("This frame"));
         frameMenu->addAction(tr("Show &only this frame"), this, SLOT(loadClickedFrame()));
-        Action* act = new Action(QIcon(":/icons/menu/new-tab.png"), tr("Show this frame in new &tab"));
+        Action* act = new Action(QIcon::fromTheme("tab-new", QIcon(":/icons/menu/new-tab.png")), tr("Show this frame in new &tab"));
         connect(act, SIGNAL(triggered()), this, SLOT(loadClickedFrameInNewTab()));
         connect(act, SIGNAL(middleClicked()), this, SLOT(loadClickedFrameInBgTab()));
         frameMenu->addAction(act);
@@ -903,7 +935,7 @@ void WebView::createPageContextMenu(QMenu* menu, const QPoint &pos)
     }
 
     menu->addSeparator();
-    menu->addAction(qIconProvider->fromTheme("user-bookmarks"), tr("Book&mark page"), this, SLOT(bookmarkLink()));
+    menu->addAction(qIconProvider->fromTheme("bookmark-new"), tr("Book&mark page"), this, SLOT(bookmarkLink()));
     menu->addAction(QIcon::fromTheme("document-save"), tr("&Save page as..."), this, SLOT(savePageAs()));
     menu->addAction(QIcon::fromTheme("edit-copy"), tr("&Copy page link"), this, SLOT(copyLinkToClipboard()))->setData(url());
     menu->addAction(QIcon::fromTheme("mail-message-new"), tr("Send page link..."), this, SLOT(sendPageByMail()));
@@ -911,11 +943,17 @@ void WebView::createPageContextMenu(QMenu* menu, const QPoint &pos)
     menu->addSeparator();
     menu->addAction(QIcon::fromTheme("edit-select-all"), tr("Select &all"), this, SLOT(selectAll()));
     menu->addSeparator();
+
     if (url().scheme() == QLatin1String("http") || url().scheme() == QLatin1String("https")) {
-        const QByteArray &w3url = "http://validator.w3.org/check?uri=" + QUrl::toPercentEncoding(url().toEncoded());
-        menu->addAction(tr("Validate page"), this, SLOT(openUrlInSelectedTab()))->setData(QUrl::fromEncoded(w3url));
+        const QUrl w3url = QUrl::fromEncoded("http://validator.w3.org/check?uri=" + QUrl::toPercentEncoding(url().toEncoded()));
+        menu->addAction(QIcon(":icons/sites/w3.png"), tr("Validate page"), this, SLOT(openUrlInSelectedTab()))->setData(w3url);
+
+        QByteArray langCode = mApp->currentLanguage().left(2).toUtf8();
+        const QUrl gturl = QUrl::fromEncoded("http://translate.google.com/translate?sl=auto&tl=" + langCode + "&u=" + QUrl::toPercentEncoding(url().toEncoded()));
+        menu->addAction(QIcon(":icons/sites/translate.png"), tr("Translate page"), this, SLOT(openUrlInSelectedTab()))->setData(gturl);
     }
 
+    menu->addSeparator();
     menu->addAction(QIcon::fromTheme("text-html"), tr("Show so&urce code"), this, SLOT(showSource()));
     menu->addAction(QIcon::fromTheme("dialog-information"), tr("Show info ab&out site"), this, SLOT(showSiteInfo()));
 }
@@ -928,14 +966,14 @@ void WebView::createLinkContextMenu(QMenu* menu, const QWebHitTestResult &hitTes
     }
 
     menu->addSeparator();
-    Action* act = new Action(QIcon(":/icons/menu/new-tab.png"), tr("Open link in new &tab"));
+    Action* act = new Action(QIcon::fromTheme("tab-new", QIcon(":/icons/menu/new-tab.png")), tr("Open link in new &tab"));
     act->setData(hitTest.linkUrl());
     connect(act, SIGNAL(triggered()), this, SLOT(userDefinedOpenUrlInNewTab()));
     connect(act, SIGNAL(middleClicked()), this, SLOT(userDefinedOpenUrlInBgTab()));
     menu->addAction(act);
     menu->addAction(QIcon::fromTheme("window-new"), tr("Open link in new &window"), this, SLOT(openUrlInNewWindow()))->setData(hitTest.linkUrl());
     menu->addSeparator();
-    menu->addAction(qIconProvider->fromTheme("user-bookmarks"), tr("B&ookmark link"), this, SLOT(bookmarkLink()))->setData(hitTest.linkUrl());
+    menu->addAction(qIconProvider->fromTheme("bookmark-new"), tr("B&ookmark link"), this, SLOT(bookmarkLink()))->setData(hitTest.linkUrl());
     menu->addAction(QIcon::fromTheme("document-save"), tr("&Save link as..."), this, SLOT(downloadUrlToDisk()))->setData(hitTest.linkUrl());
     menu->addAction(QIcon::fromTheme("mail-message-new"), tr("Send link..."), this, SLOT(sendLinkByMail()))->setData(hitTest.linkUrl());
     menu->addAction(QIcon::fromTheme("edit-copy"), tr("&Copy link address"), this, SLOT(copyLinkToClipboard()))->setData(hitTest.linkUrl());
@@ -981,13 +1019,14 @@ void WebView::createSelectedTextContextMenu(QMenu* menu, const QWebHitTestResult
     menu->addAction(QIcon::fromTheme("mail-message-new"), tr("Send text..."), this, SLOT(sendLinkByMail()))->setData(selectedText);
     menu->addSeparator();
 
-    QString langCode = mApp->currentLanguageFile().left(2);
-    QUrl googleTranslateUrl = QUrl(QString("http://translate.google.com/#auto|%1|%2").arg(langCode, selectedText));
+    QString langCode = mApp->currentLanguage().left(2).toUtf8();
+    QUrl googleTranslateUrl = QUrl(QString("http://translate.google.com/#auto/%1/%2").arg(langCode, selectedText));
     Action* gtwact = new Action(QIcon(":icons/sites/translate.png"), tr("Google Translate"));
     gtwact->setData(googleTranslateUrl);
     connect(gtwact, SIGNAL(triggered()), this, SLOT(openUrlInSelectedTab()));
     connect(gtwact, SIGNAL(middleClicked()), this, SLOT(openUrlInBackgroundTab()));
     menu->addAction(gtwact);
+
     Action* dictact = new Action(QIcon::fromTheme("accessories-dictionary"), tr("Dictionary"));
     dictact->setData(QUrl("http://" + (!langCode.isEmpty() ? langCode + "." : langCode) + "wiktionary.org/wiki/Special:Search?search=" + selectedText));
     connect(dictact, SIGNAL(triggered()), this, SLOT(openUrlInSelectedTab()));
@@ -1013,8 +1052,8 @@ void WebView::createSelectedTextContextMenu(QMenu* menu, const QWebHitTestResult
 
     menu->addSeparator();
     selectedText.truncate(20);
-    // KDE is displaying new lines in menu actions ... weird -,-
-    selectedText.replace(QLatin1Char('\n'), QLatin1Char(' ')).remove(QLatin1Char('\t'));
+    // KDE is displaying newlines in menu actions ... weird -,-
+    selectedText.replace(QLatin1Char('\n'), QLatin1Char(' ')).replace(QLatin1Char('\t'), QLatin1Char(' '));
 
     SearchEngine engine = mApp->searchEnginesManager()->activeEngine();
     Action* act = new Action(engine.icon, tr("Search \"%1 ..\" with %2").arg(selectedText, engine.name));
@@ -1149,7 +1188,7 @@ void WebView::mousePressEvent(QMouseEvent* event)
     case Qt::LeftButton: {
         QWebFrame* frame = page()->frameAt(event->pos());
         if (frame) {
-            const QUrl &link = frame->hitTestContent(event->pos()).linkUrl();
+            const QUrl link = frame->hitTestContent(event->pos()).linkUrl();
             if (event->modifiers() & Qt::ControlModifier && isUrlValid(link)) {
                 userDefinedOpenUrlInNewTab(link, event->modifiers() & Qt::ShiftModifier);
                 event->accept();
@@ -1175,7 +1214,7 @@ void WebView::mouseReleaseEvent(QMouseEvent* event)
     case Qt::MiddleButton: {
         QWebFrame* frame = page()->frameAt(event->pos());
         if (frame) {
-            const QUrl &link = frame->hitTestContent(event->pos()).linkUrl();
+            const QUrl link = frame->hitTestContent(event->pos()).linkUrl();
             if (m_clickedUrl == link && isUrlValid(link)) {
                 userDefinedOpenUrlInNewTab(link, event->modifiers() & Qt::ShiftModifier);
                 event->accept();
@@ -1208,7 +1247,26 @@ void WebView::keyPressEvent(QKeyEvent* event)
         return;
     }
 
-    switch (event->key()) {
+    int eventKey = event->key();
+
+    // The right/left arrow keys within contents with right to left (RTL) layout have
+    // reversed behavior than left to right (LTR) layout.
+    // Example: Key_Right within LTR layout triggers QWebPage::MoveToNextChar but,
+    // Key_Right within RTL layout should trigger QWebPage::MoveToPreviousChar
+
+    if (eventKey == Qt::Key_Left || eventKey == Qt::Key_Right) {
+        const QWebElement elementHasCursor = activeElement();
+        if (!elementHasCursor.isNull()) {
+            const QString direction = elementHasCursor.styleProperty("direction", QWebElement::ComputedStyle);
+            if (direction == QLatin1String("rtl")) {
+                eventKey = eventKey == Qt::Key_Left ? Qt::Key_Right : Qt::Key_Left;
+                QKeyEvent ev(event->type(), eventKey, event->modifiers(), event->text(), event->isAutoRepeat());
+                event = &ev;
+            }
+        }
+    }
+
+    switch (eventKey) {
     case Qt::Key_C:
         if (event->modifiers() == Qt::ControlModifier) {
             triggerPageAction(QWebPage::Copy);
@@ -1293,6 +1351,19 @@ void WebView::keyPressEvent(QKeyEvent* event)
         }
         break;
 
+    case Qt::Key_Insert:
+        if (event->modifiers() == Qt::ControlModifier) {
+            triggerPageAction(QWebPage::Copy);
+            event->accept();
+            return;
+        }
+        if (event->modifiers() == Qt::ShiftModifier) {
+            triggerPageAction(QWebPage::Paste);
+            event->accept();
+            return;
+        }
+        break;
+
     default:
         break;
     }
