@@ -2,27 +2,27 @@
 <context>
     <name>TestPlugin</name>
     <message>
-        <location filename="../testplugin.cpp" line="116"/>
+        <location filename="../testplugin.cpp" line="121"/>
         <source>Close</source>
         <translation>Sluit</translation>
     </message>
     <message>
-        <location filename="../testplugin.cpp" line="127"/>
+        <location filename="../testplugin.cpp" line="132"/>
         <source>Example Plugin Settings</source>
         <translation>Voorbeeldplugin-instellingen</translation>
     </message>
     <message>
-        <location filename="../testplugin.cpp" line="155"/>
+        <location filename="../testplugin.cpp" line="160"/>
         <source>My first plugin action</source>
         <translation>Mijn eerste plugin-actie</translation>
     </message>
     <message>
-        <location filename="../testplugin.cpp" line="170"/>
+        <location filename="../testplugin.cpp" line="175"/>
         <source>Hello</source>
         <translation>Hallo</translation>
     </message>
     <message>
-        <location filename="../testplugin.cpp" line="170"/>
+        <location filename="../testplugin.cpp" line="175"/>
         <source>First plugin action works :-)</source>
         <translation>Eerste plugin-actie werkt :-)</translation>
     </message>
@@ -33,7 +33,7 @@
         <location filename="../testplugin_sidebar.cpp" line="32"/>
         <location filename="../testplugin_sidebar.cpp" line="37"/>
         <source>Testing Sidebar</source>
-        <translation type="unfinished"/>
+        <translation>Testen van zijpaneel</translation>
     </message>
 </context>
 </TS>
\ No newline at end of file
